#include "artificial_inteligence.h"
#include "classmap.h"
#include <cmath>
#include <cstdlib>
#include "character/classplayer.h"

extern CURRENT_FILE_FORMAT::file_game game_data;

std::vector<character*> *artificial_inteligence::player_list=NULL;


artificial_inteligence::artificial_inteligence() :  walk_range(TILESIZE*6), target(NULL), speed_y(max_speed), acceleration_y(0.05), IA_type(0), is_ghost(false), _ai_timer(0), _ai_chain_n(0)
{
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void artificial_inteligence::exec_IA_ZIGZAG()
{
	bool must_turn = false;

	//std::cout << "1. artificial_inteligence::exec_IA_ZIGZAG() - position.x: " << position.x << ", position.y: " << position.y << std::endl;

	if (state.animation_type != ANIM_TYPE_TURN) {
		int xinc = 0;

		if (state.direction == ANIM_DIRECTION_LEFT) {
			xinc = -move_speed;
		} else {
			xinc = move_speed;
		}

        if (can_fly == false && test_change_position(xinc, TILESIZE*0.8) == true) {
			//std::cout << "*** artificial_inteligence::exec_IA_ZIGZAG() - HOLE ahead - pos.x: " << position.x << ", xinc: " << xinc << std::endl;
			must_turn = true;
		}
		if (must_turn == false && !change_position(xinc, 0)) { // change direction if colided
			//std::cout << ">>> artificial_inteligence::exec_IA_ZIGZAG() - BLOCK ahead" << std::endl;
			must_turn = true;
		}
		if (must_turn == true) {
			state.animation_type = ANIM_TYPE_TURN;
			state.animation_state = 0;
			distance.width = 0;
			state.animation_timer = timer.getTimer() + 200;
			return;
		}
		distance.width += move_speed;
		//std::cout << "artificial_inteligence::exec_IA_ZIGZAG() - name: " << name << ", distance.width: " << distance.width << ", walk_range: " << walk_range << ", state.animation_type: " << state.animation_type << std::endl;
		if (distance.width >= walk_range) {
			//std::cout << ">>> artificial_inteligence::exec_IA_ZIGZAG() - reached LIMIT" << std::endl;
			state.animation_type = ANIM_TYPE_TURN;
			state.animation_state = 0;
			distance.width = 0;
			state.animation_timer = timer.getTimer() + 200;
		}
	}
	last_execute_time = timer.getTimer() + 20;
}

void artificial_inteligence::exec_IA_VERTICAL_ZIGZAG()
{
	bool must_turn = false;

	//std::cout << "artificial_inteligence::exec_IA_ZIGZAG() - position.x: " << position.x << ", position.y: " << position.y << std::endl;

	if (state.animation_type != ANIM_TYPE_VERTICAL_TURN) {
		int yinc = 0;

		if (state.direction == ANIM_DIRECTION_LEFT) {
			yinc = -move_speed;
		} else {
			yinc = move_speed;
		}

        if (can_fly == false && test_change_position(0, yinc) == true) {
			//std::cout << "*** artificial_inteligence::exec_IA_ZIGZAG() - HOLE ahead - pos.x: " << position.x << ", xinc: " << xinc << std::endl;
			must_turn = true;
		}
		if (must_turn == false && !change_position(0, yinc)) { // change direction if colided
			//std::cout << ">>> artificial_inteligence::exec_IA_ZIGZAG() - BLOCK ahead" << std::endl;
			must_turn = true;
		}
		if (must_turn == true) {
			state.animation_type = ANIM_TYPE_VERTICAL_TURN;
			state.animation_state = 0;
			distance.width = 0;
			state.animation_timer = timer.getTimer() + 200;
			return;
		}
		distance.width += move_speed;
		//std::cout << "artificial_inteligence::exec_IA_ZIGZAG() - name: " << name << ", distance.width: " << distance.width << ", walk_range: " << walk_range << ", state.animation_type: " << state.animation_type << std::endl;
		if (distance.width >= walk_range) {
			state.animation_type = ANIM_TYPE_VERTICAL_TURN;
			state.animation_state = 0;
			distance.width = 0;
			state.animation_timer = timer.getTimer() + 200;
		}
	}
	last_execute_time = timer.getTimer() + 20;
}


// ****************************************************************************** //
// FLOATER can be GHOST (cross walls) or FISH                                     //
// of returning to start point, it just stops where player scaped or was hit      //
// ****************************************************************************** //
void artificial_inteligence::exec_IA_FLOATER(bool can_cross_walls) {
	bool can_move = false;

	struct_player_dist dist_players = dist_npc_players();

	switch (_ai_state.main_status) {
		case 0: // initial status, stopped
			if (dist_players.dist < TILESIZE*7) {
				_ai_state.main_status = 1;
				target = dist_players.pObj;
			}
			break;
		case 1: // following player
			if (target == NULL) { // error checking
				_ai_state.main_status = 0;
			}

			if (dist_players.dist > 250 || dist_players.dist <= TILESIZE+4) { // player escaped or npc reached him, stop and wait 2.5 seconds
				_ai_state.main_status = 0;
				last_execute_time = timer.getTimer()+2500;
				return;
			}

			if (target->getPosition().x > position.x) {
				if (state.direction != ANIM_DIRECTION_RIGHT) {
					state.direction = ANIM_DIRECTION_RIGHT;
				}
				if (can_cross_walls == false) {
					can_move = test_change_position(move_speed, 0);
					//std::cout << ">> exec_IA_FLOATER.move_RIGHT.can_move: " << can_move << " <<" << std::endl;
				}
				if (can_cross_walls == true || can_move == true) {
					position.x += move_speed;
				}
			} else if (target->getPosition().x < position.x) {
				if (state.direction != ANIM_DIRECTION_LEFT) {
					state.direction = ANIM_DIRECTION_LEFT;
				}
				if (can_cross_walls == false) {
					can_move = test_change_position(-move_speed, 0);
					//std::cout << ">> exec_IA_FLOATER.move_LEFT.can_move: " << can_move << " <<" << std::endl;
				}
				if (can_cross_walls == true || can_move == true) {
					position.x -= move_speed;
				}
			}

			if (target->getPosition().y > position.y) {
				if (can_cross_walls == false) {
					can_move = test_change_position(0, move_speed);
				}
				if (can_cross_walls  || can_move == true) {
					position.y += move_speed;
				}
			} else {
				if (can_cross_walls == false) {
					can_move = test_change_position(0, -move_speed);
				}
				if (can_cross_walls  || can_move == true) {
					position.y -= move_speed;
				}
			}
			last_execute_time = timer.getTimer()+130;
		default:
			break;
	}
}

void artificial_inteligence::ground_damage_players()
{
    std::vector<classPlayer*>::iterator player_it;
	for (player_it=map->_player_list.begin(); player_it != map->_player_list.end(); player_it++) {
		// check if player is on ground
		st_position npc_pos = (*player_it)->getPosition();
		npc_pos.x = (npc_pos.x + (*player_it)->get_size().width/2)/TILESIZE;
        npc_pos.y = (npc_pos.y + (*player_it)->get_size().height)/TILESIZE;
		int lock = map->getMapPointLock(npc_pos);
		//std::cout << "damage_ground_npcs - NPC[" << (*player_it)->getName() << "].lock: " << lock << ", x: " << npc_pos.x << ", y: " << npc_pos.y << std::endl;
		if (lock == TERRAIN_UNBLOCKED || lock == TERRAIN_STAIR || lock == TERRAIN_WATER) {
			continue;
		} else {
			//std::cout << "&&&&&&&&&&&&& damage_ground_npcs - DAMAGING PLAYER[" << (*player_it)->getName() << "]" << std::endl;
            (*player_it)->damage(2, false);
		}
    }
}
/*
void artificial_inteligence::execute_basic_ai()
{
	if (timer.getTimer() < _ai_timer) {
		return;
	}

    int lock_point = map->getMapPointLock(st_position(position.x/16, position.y/16));
    struct struct_player_dist dist_players = dist_npc_players();

    if (IA_type == IA_ZIGZAG) {
        exec_IA_ZIGZAG();
    } else if (IA_type == IA_VERTICAL_ZIGZAG) {
        exec_IA_VERTICAL_ZIGZAG();
    } else if (IA_type == IA_BAT) {
		if (_ai_state.main_status == IA_STATE_FOLLOW) {
            // handle error on player object
            if (target == NULL) {
                //std::cout << ">>>>>>>> IA_BAT - target NULL error" << std::endl;
				_ai_state.main_status = IA_STATE_RETURN;
                return;
            }

            if (dist_players.dist > 250 || dist_players.dist <= TILESIZE+4) { // player escaped or npc reached him
                //std::cout << ">>>>>>>> IA_BAT - CHANGE to return" << std::endl;
				_ai_state.main_status = IA_STATE_RETURN;
                return;
            }
            if (target->getPosition().x > position.x) {
                position.x += 4;
            } else if (target->getPosition().x < position.x) {
                position.x -= 4;
            }
            if (target->getPosition().y > position.y) {
                position.y += 4;
            } else if (target->getPosition().y < position.y) {
                position.y -= 4;
            }
		} else if (_ai_state.main_status == IA_STATE_RETURN) {
            //std::cout << ">>>>>>>> IA_BAT - execute return" << std::endl;
            if (target != NULL) {
                if (target->getPosition().x > position.x) {
                    position.x -= 2;
                } else if (target->getPosition().x < position.x) {
                    position.x += 2;
                }
            }
            position.y -= 4;
            if ((position.y < 0) || (lock_point > 0 && lock_point != TERRAIN_WATER)) {
				_ai_state.main_status = IA_STATE_SLEEP;
                state.animation_type = ANIM_TYPE_STAND;
				_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
                return;
            }
        } else if (state.animation_type == ANIM_TYPE_MOVE_SEMI) { // START MOVING
            //std::cout << ">>>>>>>> IA_BAT - move semi" << std::endl;
			_ai_state.main_status = IA_STATE_FOLLOW;
            state.animation_type = ANIM_TYPE_WALK;
        } else { // IA_STATE_SLEEP
            struct struct_player_dist temp = dist_npc_players();
			//std::cout << ">>>>>>>> IA_BAT - sleep - IA_state: " << IA_state << ", dist: " << abs((float)temp.dist) << std::endl;
			if (abs((float)temp.dist) < 150) {
                state.animation_type = ANIM_TYPE_MOVE_SEMI;
				_ai_state.main_status = IA_STATE_FOLLOW;
                target = temp.pObj;
            }
        }
        last_execute_time = timer.getTimer() + 200;
    } else if (IA_type == IA_ROOF_SHOOTER) {
		if (_ai_state.main_status == 0) { // sleep -> regular
            state.animation_type = ANIM_TYPE_ATTACK;
            // shoot regular
            throw_projectile_npc(TRAJECTORY_LINEAR, 2, 1);
            moveCommands.attack = 1; // TODO - attack must get projectile from a type
			_ai_state.main_status = 1;
            last_execute_time = timer.getTimer() + 800;
		} else if (_ai_state.main_status == 1) { // regular -> diagonal
            // shoot diagonal
            if (position.y > RES_H/2) {
                throw_projectile_npc(TRAJECTORY_DIAGONAL_UP, 2, 1);
            } else {
                throw_projectile_npc(TRAJECTORY_DIAGONAL_DOWN, 2, 1);
            }
            moveCommands.attack = 1; // TODO - attack must get projectile from a type
			_ai_state.main_status = 2;
            last_execute_time = timer.getTimer() + 800;
		} else if (_ai_state.main_status == 2) { // diagonal -> sleep
            state.animation_type = ANIM_TYPE_STAND;
            last_execute_time = timer.getTimer() + 2000;
			_ai_state.main_status = 0;
        }
    } else if (IA_type == IA_STAND && is_ghost == 1) {
        // TODO: change direction to face player
		if (_ai_state.main_status == 0) {
			if (abs((float)dist_players.pObj->getPosition().x - position.x) <= 96) {
                //std::cout << "IA_STAND - ATTACK START" << std::endl;
                state.animation_type = ANIM_TYPE_ATTACK;
                state.animation_state = 0;
                state.animation_timer = 0;
                last_execute_time = timer.getTimer() + 500;
				_ai_state.main_status = 1;
                if (state.direction == ANIM_DIRECTION_LEFT && dist_players.pObj->getPosition().x > position.x) {
                    state.direction = ANIM_DIRECTION_RIGHT;
                } else if (state.direction == ANIM_DIRECTION_RIGHT && dist_players.pObj->getPosition().x < position.x) {
                    state.direction = ANIM_DIRECTION_LEFT;
                }
            }
		} else if (_ai_state.main_status == 1) {
            //std::cout << "IA_STAND - ATTACK CONTINUE" << std::endl;
            throw_projectile_npc(TRAJECTORY_LINEAR, 1, 0);
            moveCommands.attack = 1;
            last_execute_time = timer.getTimer() + 500;
			_ai_state.main_status = 2;
		} else if (_ai_state.main_status == 2) {
            //std::cout << "IA_STAND - ATTACK FINISH" << std::endl;
            state.animation_type = ANIM_TYPE_STAND;
            last_execute_time = timer.getTimer() + 1500;
			_ai_state.main_status = 0;
        } else {
			std::cout << "IA_STAND - unknown state: " << _ai_state.main_status << std::endl;
			_ai_state.main_status = 0;
        }
    } else if (IA_type == IA_SHOOT_AND_GO) {
        // TODO: change direction to face player
		int dist_player = abs((float)dist_players.pObj->getPosition().x - position.x);
        //std::cout << "IA_SHOOT_AND_GO - state(" << IA_state << "), dist_player: " << dist_player << std::endl;
		if (_ai_state.main_status == 0) {
            if (dist_player <= walk_range) {
                state.animation_type = ANIM_TYPE_ATTACK;
                state.animation_state = 0;
                state.animation_timer = 0;
                last_execute_time = timer.getTimer() + 500;
				_ai_state.main_status = 1;
                if (state.direction == ANIM_DIRECTION_LEFT && dist_players.pObj->getPosition().x > position.x) {
                    state.direction = ANIM_DIRECTION_RIGHT;
                } else if (state.direction == ANIM_DIRECTION_RIGHT && dist_players.pObj->getPosition().x < position.x) {
                    state.direction = ANIM_DIRECTION_LEFT;
                }
            }
		} else if (_ai_state.main_status == 1) { // SHOOTING
            throw_projectile_npc(TRAJECTORY_LINEAR, 1, 0);
            last_execute_time = timer.getTimer() + 800;
			_ai_state.main_status = 2;
        } else { // MOVING
            if (state.direction == ANIM_DIRECTION_LEFT) {
                if (!test_change_position(-move_speed, 0)) {
                    state.animation_type = ANIM_TYPE_TURN;
                    state.animation_state = 0;
                    distance.width = 0;
                    state.animation_timer = timer.getTimer() + 200;
					_ai_state.main_status = 0;
                    return;
                }
                state.animation_type = ANIM_TYPE_WALK;
                position.x -= move_speed;
            } else {
                if (!test_change_position(move_speed, 0)) {
                    state.animation_type = ANIM_TYPE_TURN;
                    state.animation_state = 0;
                    distance.width = 0;
                    state.animation_timer = timer.getTimer() + 200;
					_ai_state.main_status = 0;
                    return;
                }
                state.animation_type = ANIM_TYPE_WALK;
                position.x += move_speed;
            }
            last_execute_time = timer.getTimer() + 20;
        }
    } else if (IA_type == IA_FLY_ZIG_ZAG) {
        exec_IA_ZIGZAG();
    } else if (IA_type == IA_FIXED_JUMPER) {
        if (state.animation_type != ANIM_TYPE_STAND) {
            state.animation_type = ANIM_TYPE_STAND;
        }
        //std::cout << ">> IA_FIXED_JUMPER - exec.position.y: " << position.y << " <<" << std::endl;
		if (_ai_state.main_status != 1) { // falling
            if (position.y < RES_H+TILESIZE) {
                position.y += 8;
                last_execute_time = timer.getTimer()+20;
            } else {
                state.animation_type = ANIM_TYPE_STAND;
                state.animation_state = 0;
                state.animation_timer = 0;
				_ai_state.main_status = 1;
                last_execute_time = timer.getTimer()+1200;
            }
        } else { // jump up
            if (position.y > 20) {
                position.y -= 6;
                last_execute_time = timer.getTimer()+20;
            } else { // reached jump maximum, wait a bit and start falling
				_ai_state.main_status = 0;
                last_execute_time = timer.getTimer()+200;
            }
        }
    } else if (IA_type == IA_SIDE_SHOOTER) {
		if (_ai_state.main_status == 0) { // sleep -> shoot
            // shoot diagonal
            struct struct_player_dist temp = dist_npc_players();
            target = temp.pObj;
            // shoot down
            if (target->getPosition().y > position.y + frameSize.height) {
                throw_projectile_npc(TRAJECTORY_DIAGONAL_DOWN, 1, 1);
                state.animation_type = ANIM_TYPE_ATTACK;
            // shoot up
            } else if (target->getPosition().y + target->get_size().height < position.y) {
                throw_projectile_npc(TRAJECTORY_DIAGONAL_UP, 1, 1);
                state.animation_type = ANIM_TYPE_STAND;
            // shoot regular
            } else {
                throw_projectile_npc(TRAJECTORY_LINEAR, 1, 1);
                state.animation_type = ANIM_TYPE_WALK;
            }
			_ai_state.main_status = 1;
            last_execute_time = timer.getTimer()+800;
		} else if (_ai_state.main_status == 1) { // shoot -> sleep
            last_execute_time = timer.getTimer() + 2000;
			_ai_state.main_status = 0;
        }
    } else if (IA_type == IA_GHOST) {
        exec_IA_GHOST();
    } else if (IA_type == IA_FISH) {
        exec_IA_FISH();
    } else if (IA_type == IA_DOLPHIN) {
        exec_IA_DOLPHIN();
    } else if (IA_type == IA_SIDETOSIDE) {
        exec_IA_SIDETOSIDE();
    } else {
        std::cout << ">>>>>>>>>>>>>> UNDEFINED IA IA_type: " << IA_type << ", Npc.name: " << name << " <<<<<<<<<<<<<<" << std::endl;
    }
}
*/

void artificial_inteligence::execute_chain_ai()
{
	if (timer.getTimer() < _ai_timer) {
		return;
	}
	if (_ai_state.main_status == IA_STATE_JUMP_TO_PLAYER) {
        ia_action_jump_to_player();
	} else if (_ai_state.main_status == IA_STATE_CHARGED_DASH) {
        ia_dash();
	} else if (_ai_state.main_status == IA_STATE_JUMP_TO_WALL) {
        ia_jump_to_wall();
	} else 	if (_ai_state.main_status == IA_STATE_WALK_TO_POINT) {
        ia_walk_to_position();
	} else if (_ai_state.main_status == IA_STATE_SHOT_STOP_TIME) {
        ia_shot_stop_time();
	} else 	if (_ai_state.main_status == IA_STATE_TELEPORT) {
        ia_teleport();
	} else 	if (_ai_state.main_status == IA_STATE_QUAKE_ATTACK) {
        ia_action_quake_attack();
	} else if (_ai_state.main_status == IA_STATE_JUMP_TO_PLAYER) {
        ia_action_jump_to_player();
	} else if (_ai_state.main_status == IA_STATE_AIR_WALK) {
        ia_action_air_walk();
	} else if (_ai_state.main_status == IA_STATE_JUMP_FALL) {
        ia_action_jump_fall();
	} else if (_ai_state.main_status == IA_STATE_WAIT_TIME) {
        state.move_timer = timer.getTimer() + 1200;
		_ai_state.main_status = IA_STATE_RETURN;
	} else 	if (_ai_state.main_status == IA_STATE_FLOAT_TO_POINT) {
		ia_float_to_position();
	//} else {
		//std::cout << "artificial_inteligence::execute_chain_ai - unknown mode " << ia_state.main_status << ". NPC: " << name << std::endl;
	}
}






void artificial_inteligence::exec_IA_GHOST()
{
	exec_IA_FLOATER(true);
}

void artificial_inteligence::exec_IA_FISH()
{
	exec_IA_FLOATER(false);
}

void artificial_inteligence::exec_IA_DOLPHIN()
{
	int blocked = 0;
	struct struct_player_dist dist_p = dist_npc_players();

	switch (_ai_state.main_status) {
		case 0: // initial status, stopped
			// sub_status is used as speed acceleration control
			if (state.animation_type != ANIM_TYPE_TURN) {
				if (state.direction == ANIM_DIRECTION_LEFT) {
					if (test_change_position(-move_speed, 0)) {
						blocked = 1;
					} else {
						position.x -= move_speed;
						distance.width -= move_speed;
					}
				} else {
					if (test_change_position(move_speed, 0)) {
						blocked = 1;
					} else {
						position.x += move_speed;
						distance.width += move_speed;
					}
				}
				// set target, if needed
				if (target == NULL) {
					target = dist_p.pObj;
				}
				if (target->getPosition().y > position.y) {
					if (test_change_position(0, move_speed) == 0) {
						position.y += move_speed;
                        distance.height += move_speed;
					}
				} else if (target->getPosition().y < position.y) {
					if (test_change_position(0, -move_speed) == 0) {
						position.y -= move_speed;
                        distance.height -= move_speed;
					}
				}
				/// @TODO: implement a kind of counter or IA_sub_status
				/*
				if (blocked == 0) {
					if (ia_npc->move_info.sub_status > 15) {
						ia_npc->move_info.sub_status--;
					}
				}
				*/
			} else {
				//ia_npc->move_info.sub_status = 200;
			}
			//last_execute_time = timer.getTimer() + ia_npc->move_info.sub_status;
            if (distance.width > walk_range || distance.height > walk_range || blocked == 1) { // maximum is one-screen width
				state.animation_type = ANIM_TYPE_TURN;
				state.animation_state = 0;
				last_execute_time = timer.getTimer()+200;
				_ai_state.main_status = 1;
				target = NULL;
			}
			break;
		case 1: // return to origin backwards
            if (distance.width <= 0 && distance.height <= 0) { // reached origin
				_ai_state.main_status = 0;
				last_execute_time = timer.getTimer()+2000;
			} else { /// @TODO: fix to use distance->width/height
				if (state.direction != ANIM_DIRECTION_LEFT) {
					if (test_change_position(-move_speed, 0)) {
						position.x -= move_speed;
					}
				} else {
					if (test_change_position(move_speed, 0)) {
						position.x += move_speed;
					}
				}
				last_execute_time = timer.getTimer()+90;
			}
			break;

		default:
			break;
	}
}

void artificial_inteligence::exec_IA_SIDETOSIDE()
{
	/// @TODO
}


// ********************************************************************************************** //
// Find the nearest player and return a struct with distance and a reference to player            //
// ********************************************************************************************** //
struct_player_dist artificial_inteligence::dist_npc_players()
{
    int dist;
	struct struct_player_dist res;
    if (!map) {
        std::cout << "ERROR: trying to calc NPC-player distance before map pointer is set on NPC" << std::endl;
        graphLib.show_debug_msg("EXIT #A.01");
        exit(-1);
    }
    if (map->_player_list.size() == 0) {
		std::cout << "ERROR: trying to calc NPC-player distance before there is a player in the game or this NPC does not have set the player_list" << std::endl;
        graphLib.show_debug_msg("EXIT #01");
        exit(-1);
	}

    res.pObj = map->_player_list.at(0);
	dist = sqrt(pow((position.x - res.pObj->getPosition().x), 2) + pow((position.y - res.pObj->getPosition().y), 2));
	res.dist_xy.x = abs((float)position.x - res.pObj->getPosition().x);
	res.dist_xy.y = abs((float)position.y - res.pObj->getPosition().y);

    res.dist = dist;
	return res;

}


void artificial_inteligence::ia_action_jump_to_player()
{
	if (_ai_state.sub_status == IA_ACTION_STATE_INITIAL) {
		struct_player_dist dist_npc_player = dist_npc_players();
		_ai_state.main_status = IA_STATE_JUMP_TO_PLAYER;
		if (position.x < dist_npc_player.pObj->getPosition().x) {
			state.direction = ANIM_DIRECTION_RIGHT;
			_ai_state.initial_position.x = position.x + dist_npc_player.dist/2;
		} else {
			state.direction = ANIM_DIRECTION_LEFT;
			_ai_state.initial_position.x = position.x - dist_npc_player.dist/2;
		}
		_ai_state.initial_position.y = position.y;
		// store initial point position to be used in jump calculations
		_ai_state.secondary_position.x = position.x;
		_ai_state.secondary_position.y = position.y;
		_ai_state.sub_status = pow(dist_npc_player.dist, 2);
		state.animation_type = ANIM_TYPE_JUMP;
		_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		_force_jump = true;

	} else if (_ai_state.sub_status == IA_ACTION_STATE_EXECUTING) {
		if (state.direction == ANIM_DIRECTION_RIGHT) {
			moveCommands.right = 1;
			//position.x += 2;
		} else {
			moveCommands.left = 1;
			//position.x -= 2;
		}
        //if (hit_ground()) std::cout << ">> IA::ia_action_jump_to_player - hit_ground(): " << hit_ground() << ", jump_state: " << state.jump_state << std::endl;
		if (state.jump_state != JUMP_UP && hit_ground() == true) {
            //std::cout << ">> IA::ia_action_jump_to_player - END" << std::endl;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
			moveCommands.right = 0;
			moveCommands.left = 0;
			_ai_timer = timer.getTimer() + 500;
		}
	}
}

void artificial_inteligence::ia_action_jump_to_roof()
{
	if (_ai_state.sub_status == IA_ACTION_STATE_INITIAL) {
		state.animation_type = ANIM_TYPE_JUMP;
		_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		int limit_y = 0;
		for (int y=position.y; y>=0; y--) {
			st_position new_pos(position.x/TILESIZE, (y)/TILESIZE);
			if (map->is_point_solid(new_pos) == true) {
				limit_y = y;
				break;
			}
		}
		_ai_state.secondary_position.y = limit_y;
		_ai_state.step = (limit_y - position.y)/max_speed;
        can_fly = true; // disable gravity
        speed_y = max_speed;
		return;
	}
	// @TODO: acceleration
	if (position.y > _ai_state.secondary_position.y) {
		ia_accelerate_up();
		//std::cout << ">> IA::ia_action_jump_to_roof << JUMPING - position.y: " << position.y << ", limit.y: " << ia_state.secondary_position.y << std::endl;
	} else {
		_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
	}
}

void artificial_inteligence::ia_action_air_walk()
{

	if (_ai_state.sub_status == IA_ACTION_STATE_INITIAL) {
		//std::cout << "artificial_inteligence::ia_action_air_walk - INITIALIZE" << std::endl;
		state.animation_type = ANIM_TYPE_WALK_AIR;
		_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		struct_player_dist dist_npc_player = dist_npc_players();
		if (position.x < dist_npc_player.pObj->getPosition().x) {
			state.direction = ANIM_DIRECTION_RIGHT;
			_ai_state.secondary_position.x = position.x + dist_npc_player.dist_xy.x;
		} else {
			state.direction = ANIM_DIRECTION_LEFT;
			_ai_state.secondary_position.x = position.x - dist_npc_player.dist_xy.x;
		}
		//std::cout << "artificial_inteligence::ia_action_air_walk - dist:" << dist_npc_player.dist << std::endl;
	} else if (_ai_state.sub_status == IA_ACTION_STATE_EXECUTING) {
		//std::cout << "artificial_inteligence::ia_action_air_walk - EXECUTE" << std::endl;
		/// @TODO use move speed and reducer for last part
		if (state.direction == ANIM_DIRECTION_RIGHT) {
			position.x += move_speed;
		} else {
			position.x -= move_speed;
		}
		if ((state.direction == ANIM_DIRECTION_RIGHT && position.x >= _ai_state.secondary_position.x) || (state.direction == ANIM_DIRECTION_LEFT && position.x <= _ai_state.secondary_position.x)) {
			//std::cout << "artificial_inteligence::ia_action_air_walk - FINISH" << std::endl;
			state.animation_type = ANIM_TYPE_JUMP;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
		}
	}
}

void artificial_inteligence::ia_action_jump_fall()
{
	//std::cout << "******** artificial_inteligence::ia_action_jump_fall" << std::endl;
	if (_ai_state.sub_status == IA_ACTION_STATE_INITIAL) {
		_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		speed_y = 2;
	} else if (_ai_state.sub_status == IA_ACTION_STATE_EXECUTING) {
		ia_accelerate_down();
		if (hit_ground() == true) {
			//std::cout << "artificial_inteligence::ia_action_jump_fall - FINISH" << std::endl;
			state.animation_type = ANIM_TYPE_STAND;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
			_ai_timer = timer.getTimer() + 1200;
            can_fly = false;
        }
	}
}

void artificial_inteligence::ia_action_quake_attack()
{
	if (_ai_state.sub_status == IA_ACTION_STATE_INITIAL) {
		_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		_ai_state.initial_position.x = 0;
	} else if (_ai_state.sub_status == IA_ACTION_STATE_EXECUTING) {
		//std::cout << "IA_STATE_QUAKE_ATTACK - ia_state.timer: EXEC" << std::endl;
		_ai_state.initial_position.x++;
		if (_ai_state.initial_position.x % 20) {
			graphLib.set_screen_adjust(st_position(-10, 0));
			ground_damage_players();
		} else if (_ai_state.initial_position.x % 25) {
			graphLib.set_screen_adjust(st_position(10, 0));
			ground_damage_players();
		}
		_ai_state.timer = timer.getTimer() + 500;
		if (_ai_state.initial_position.x > 200) {
			graphLib.set_screen_adjust(st_position(0, 0));
			state.animation_type = ANIM_TYPE_STAND;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
		}
	}
}

void artificial_inteligence::ia_accelerate_up()
{
	//std::cout << "ia_accelerate_up - 1.speed_y: " << std::fixed << speed_y << std::endl;
	speed_y -= speed_y*acceleration_y;
	//std::cout << "ia_accelerate_up - 2.speed_y: " << std::fixed << speed_y << std::endl;
	if (speed_y < 1) {
		//speed_y = acceleration_y;
		speed_y = 1;
	} else if (speed_y > max_speed) {
		speed_y = max_speed;
	}
	position.y -= speed_y;
	/// @TODO - adjustment when ground is near
}


void artificial_inteligence::ia_accelerate_down()
{
	//std::cout << "ia_accelerate_down - 1.speed_y: " << std::fixed << speed_y << std::endl;
	speed_y += speed_y*acceleration_y;
	//std::cout << "ia_accelerate_down - 2.speed_y: " << std::fixed << speed_y << std::endl;
	if (speed_y < 1) {
		speed_y = acceleration_y;
	} else if (speed_y > max_speed) {
		speed_y = max_speed;
	}
	position.y += speed_y;
	/// @TODO - adjustment when ground is near
}

void artificial_inteligence::ia_teleport()
{
	//std::cout << "artificial_inteligence::ia_teleport - state.animation_state: " << state.animation_state << ", state.animation_timer: " << state.animation_timer << ", timer: " << timer.getTimer() << std::endl;
	// if is on last frame, invert
	if (state.animation_inverse == false) {
		if (state.animation_state == frames_count()-1) {
            std::cout << "artificial_inteligence::ia_teleport - INVERT ANIMATION" << std::endl;
			state.animation_inverse = true;
			_ai_state.action_status = 0;
			state.invisible = true;
			struct_player_dist dist_npc_player = dist_npc_players();
			position.x = dist_npc_player.pObj->getPosition().x;
			state.animation_timer = timer.getTimer() + 600;
		}
	} else {
		if (_ai_state.action_status == 0 && state.animation_timer <= timer.getTimer()) {
            std::cout << "artificial_inteligence::ia_teleport - RETURN TELEPORT" << std::endl;
			_ai_state.action_status++;
			state.invisible = false;
		}
		if (state.animation_state == 0) {
            std::cout << "artificial_inteligence::ia_teleport - FINISH TELEPORT" << std::endl;
			struct_player_dist dist_npc_player = dist_npc_players();
			state.animation_type = ANIM_TYPE_STAND;
			_ai_state.main_status = IA_STATE_WAIT_TIME;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
			if (dist_npc_player.pObj->getPosition().x > position.x) {
				state.direction = ANIM_DIRECTION_RIGHT;
			} else {
				state.direction = ANIM_DIRECTION_LEFT;
			}
			state.animation_inverse = false;
			//state.animation_timer = timer.getTimer() + 3000;
		}
	}
}

void artificial_inteligence::ia_set_destiny_point(st_position dest_point)
{
	_ai_state.initial_position.x = position.x;
	_ai_state.initial_position.y  = position.y;
	_ai_state.secondary_position.x = dest_point.x;
	_ai_state.secondary_position.y = dest_point.y;
	if (_ai_state.secondary_position.x < _ai_state.initial_position.x) {
		state.direction = ANIM_DIRECTION_LEFT;
	} else {
		state.direction = ANIM_DIRECTION_RIGHT;
	}
}

// walk until reaching a given X coordinate
// @TODO - jump if needed
void artificial_inteligence::ia_walk_to_position()
{
	// @TODO - if can't walk, change action to jump to player
	//if (!test_change_position(-move_speed, 0)) {

    bool finished = false;
	if (_ai_state.secondary_position.x < _ai_state.initial_position.x) {
		position.x -= move_speed;
		if (position.x <= _ai_state.secondary_position.x) {
			finished = true;
		}
	} else {
		position.x += move_speed;
		if (position.x >= _ai_state.secondary_position.x) {
			finished = true;
		}
	}
	if (finished == true) {
		state.animation_type = ANIM_TYPE_STAND;
		_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
		state.move_timer = timer.getTimer() + 2000;
	}
}

void artificial_inteligence::ia_float_to_position()
{
	bool finished = false;
	int xinc = 0;
	int yinc = 0;
	bool reached_x = false;
	bool reached_y = false;
	/*
	if (ia_state.secondary_position.x < ia_state.initial_position.x) {
		position.x -= move_speed;
		if (position.x <= ia_state.secondary_position.x) {
			finished = true;
		}
	} else {
		position.x += move_speed;
		if (position.x >= ia_state.secondary_position.x) {
			finished = true;
		}
	}
	if (ia_state.secondary_position.y < ia_state.initial_position.y) {
		position.y -= move_speed;
		if (position.y <= ia_state.secondary_position.y) {
			finished = true;
		}
	} else {
		position.y += move_speed;
		if (position.y >= ia_state.secondary_position.y) {
			finished = true;
		}
	}
	*/

	if (_ai_state.secondary_position.x < _ai_state.initial_position.x) {
		if (position.x <= _ai_state.secondary_position.x) {
			xinc = 0;
		} else {
			xinc -= move_speed;
		}
	} else {
		if (position.x >= _ai_state.secondary_position.x) {
			xinc = 0;
		} else {
			xinc += move_speed;
		}
	}
	if (_ai_state.secondary_position.y < _ai_state.initial_position.y) {
		if (position.y <= _ai_state.secondary_position.y) {
			yinc = 0;
		} else {
			yinc -= move_speed;
		}
	} else {
		if (position.y >= _ai_state.secondary_position.y) {
			yinc = 0;
		} else {
			yinc += move_speed;
		}
	}


	if (test_change_position(xinc, 0) == false) {
		xinc = 0;
	}
	if (test_change_position(0, yinc) == false) {
		yinc = 0;
	}
	if (xinc == 0 && yinc == 0) { // both X and Y blocked, leave
		finished = true;
	} else {
		std::cout << "*** artificial_inteligence::ia_float_to_position - xinc: " << xinc << ", yinc: " << yinc << std::endl;
		position.x += xinc;
		position.y += yinc;
	}

	if (reached_x == true && reached_y == true) {
		finished = true;
	}
	// check if reached the point
	if (finished == true) {
		state.animation_type = ANIM_TYPE_STAND;
		_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
		state.move_timer = timer.getTimer() + 2000;
	}
}

void artificial_inteligence::ia_dash()
{
	bool finished = false;
	if (_ai_state.secondary_position.x < _ai_state.initial_position.x) {
		position.x -= max_speed-3;
		if (position.x <= _ai_state.secondary_position.x) {
			finished = true;
		}
	} else {
		position.x += max_speed-3;
		if (position.x >= _ai_state.secondary_position.x) {
			finished = true;
		}
	}
	if (finished == true) {
		state.animation_type = ANIM_TYPE_STAND;
		_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
		state.move_timer = timer.getTimer() + 2000;
	}
}

void artificial_inteligence::ia_shot_stop_time()
{
	graphLib.blink_screen(255, 255, 255);
	struct_player_dist dist_npc_player = dist_npc_players();
	dist_npc_player.pObj->freeze(2500);
	state.animation_type = ANIM_TYPE_STAND;
	_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
	state.move_timer = timer.getTimer() + 1000;
}

void artificial_inteligence::ia_jump_to_wall()
{
	std::cout << "AI::ia_jump_to_wall - ia_action_status: " << _ai_state.action_status << std::endl;
	if (_ai_state.action_status == 0) {
		int xinc = move_speed;
		int yinc = -move_speed;
		if(_ai_state.step == 1) {
			yinc = 0;
			_ai_state.step = 0;
		} else {
			_ai_state.step = 1;
		}
		if (state.direction == ANIM_DIRECTION_LEFT) {
			xinc = -move_speed;
		}
		// reached wall
		if (test_change_position_debug(xinc, yinc) == false) {
			std::cout << "AI::ia_jump_to_wall - reached wall - xinc: " << xinc << ", yinc: " << yinc << std::endl;
			_ai_state.action_status = 1;
			if (state.direction == ANIM_DIRECTION_LEFT) {
				state.direction = ANIM_DIRECTION_RIGHT;
			} else {
				state.direction = ANIM_DIRECTION_LEFT;
			}
			state.animation_type = ANIM_TYPE_WALK_AIR;
			state.move_timer = timer.getTimer() + 1000;
		} else {
			std::cout << "1. AI::ia_jump_to_wall - move - position.y: " << position.y << std::endl;
			position.x += xinc;
			position.y += yinc;
			std::cout << "2. AI::ia_jump_to_wall - move - position.y: " << position.y << std::endl;
		}
	} else if (_ai_state.action_status == 1) {
		std::cout << "AI::ia_jump_to_wall - grab" << std::endl;
		_ai_state.action_status = 2;

		st_position proj_pos;
		if (state.direction == ANIM_DIRECTION_LEFT) {
            proj_pos = st_position(position.x+TILESIZE/2, position.y+frameSize.height/2);
		} else {
            proj_pos = st_position(position.x+frameSize.width-TILESIZE*2, position.y+frameSize.height/2);
		}
		//projectile(int set_direction, int set_speed, st_position set_position, classMap *set_map, int damage, short int move_type, short wpn_id, std::string p_filename, st_size p_size);
		projectile_list.push_back(new projectile(game_data.game_npcs[_number].projectile_id[0], state.direction, proj_pos, map));

		state.move_timer = timer.getTimer() + 1000;
        can_fly = false;
    } else {
		std::cout << "AI::ia_jump_to_wall - falling" << std::endl;
		state.animation_type = ANIM_TYPE_JUMP;
		if (hit_ground()) {
			std::cout << "AI::ia_jump_to_wall - finish" << std::endl;
			state.animation_type = ANIM_TYPE_STAND;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
			state.move_timer = timer.getTimer() + 600;
		}
	}
}


// TODO - limit also per player
// type can be normal, diagonal_up or diagonal_down
// n is the number os projectiles to be shot
// alternated indicates if direction is the same for each shot or not
void artificial_inteligence::throw_projectile_npc(int type, int n, int alternated) {
    UNUSED(type);
    UNUSED(alternated);
    short int last_direction = -1;
    short int proj_direction;
    for (int i=0; i<n; i++) {
        st_position proj_pos;
        if (last_direction == -1) {
            last_direction = state.direction;
            proj_direction = state.direction;
        } else if (last_direction == ANIM_DIRECTION_LEFT) {
            last_direction = state.direction;
            proj_direction = ANIM_DIRECTION_RIGHT;
        } else if (last_direction == ANIM_DIRECTION_RIGHT) {
            last_direction = state.direction;
            proj_direction = ANIM_DIRECTION_LEFT;
        }
        if (proj_direction == ANIM_DIRECTION_LEFT) {
            proj_pos = st_position(position.x, position.y+frameSize.height/2);
        } else {
            proj_pos = st_position(position.x+frameSize.width, position.y+frameSize.height/2);
        }
        //projectile_list.push_back(new projectile(0, proj_direction, PROJECTILE_DEFAULT_SPEED, proj_pos, map, 1, type, -1));
		projectile_list.push_back(new projectile(game_data.game_npcs[_number].projectile_id[0], state.direction, proj_pos, map));
    }
}

/*
 * OK - IA_STAND
  * OK - IA_FOLLOW
 * OK - IA_ZIGZAG
 * - IA_SIDETOSIDE: add a way so that walk-ahead only stops when hitting a wall, not by distance
 * OK - IA_BAT
 * OK - IA_ROOF_SHOOTER
 * OK - IA_GROUND_SHOOTER
 * OK - IA_SHOOT_AND_GO
 * OK - IA_FLY_ZIG_ZAG
 * OK - IA_BUTTERFLY
 * - IA_HORIZONTAL_GO_AHEAD: works, but when reaching point, does not turn back (must also add a way to ignore distance)
 * OK - IA_HORIZONTAL_TURN
 * OK - IA_FIXED_JUMPER
 * - IA_SIDE_SHOOTER
 * OK - IA_GHOST: add a way to pass walls
 * OK - IA_FISH
 * IA_DOLPHIN
 * IA_VERTICAL_ZIGZAG
 *
*/

void artificial_inteligence::execute_ai()
{
	if (timer.getTimer() < _ai_timer) {
		return;
	}
    //std::cout << "artificial_inteligence::execute_ai - _ai_state.sub_status: " << _ai_state.sub_status << std::endl;
	//int current_action = game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n];
	// check if action is finished
	if (_ai_state.sub_status == IA_ACTION_STATE_FINISHED) {
        //std::cout << "artificial_inteligence::execute_ai - FINISHED - define next step" << std::endl;
		define_ai_next_step();
	}
	execute_ai_step();
}

void artificial_inteligence::define_ai_next_step()
{
	if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].go_to == AI_ACTION_GOTO_CHANCE) { // CHANCE
		int rand_n = rand() % 100;
        //std::cout << "artificial_inteligence::define_ai_next_step - CHANCE - rand_n: " << rand_n << std::endl;
        bool found_chance = false;
        int chance_sum = 0;
		for (int i=0; i<4; i++) {
            //std::cout << "[" << i << "].chance: " << game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[i].chance << ", chance_sum: " << chance_sum << std::endl;
            chance_sum += game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[i].chance;
            if (rand_n < chance_sum) {
                std::cout << "FOUND CHANCE at [" << i << "]" << std::endl;
				_ai_chain_n = i;
				found_chance = true;
				break;
			}
		}
        if (found_chance == false) {
			_ai_chain_n = 0;
		}
	} else {
		_ai_chain_n = game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].go_to-1;
        //std::cout << "artificial_inteligence::define_ai_next_step FORCE NEXT - " << _ai_chain_n << std::endl;
    }
	_ai_state.sub_status = IA_ACTION_STATE_INITIAL;
}

void artificial_inteligence::execute_ai_step()
{
    int ai_id = game_data.game_npcs[_number].gravity_level;
    int type = game_data.ai_types[ai_id].states[_ai_chain_n].action;
    //std::cout << "artificial_inteligence::execute_ai_step - type: " << type << std::endl;
	_ai_timer = timer.getTimer() + 20;
	if (type == AI_ACTION_WALK) {
        //std::cout << ">> artificial_inteligence::execute_ai_step - WALK" << std::endl;
		execute_ai_step_walk();
	} else if (type == AI_ACTION_WAIT_UNTIL_PLAYER_IS_IN_RANGE) {
		execute_ai_action_wait_until_player_in_range();
	} else if (type == AI_ACTION_SHOT_PROJECTILE_1) {
		execute_ai_action_trow_projectile(0, false);
	} else if (type == AI_ACTION_SHOT_PROJECTILE_2) {
		execute_ai_action_trow_projectile(1, false);
	} else if (type == AI_ACTION_SHOT_INVERT_PROJECTILE_1) {
		//std::cout << "#1 - AI_ACTION_SHOT_INVERT_PROJECTILE_1" << std::endl;
		execute_ai_action_trow_projectile(1, true);
	} else if (type == AI_ACTION_FLY) {
		execute_ai_step_fly();
    } else if (type == AI_ACTION_DASH) {
        execute_ai_step_dash();
    } else {
        std::cout << "********** AI ACTION[" << game_data.game_npcs[_number].gravity_level << "][" << _ai_chain_n << "] #" << type << " NOT IMPLEMENTED **********" << std::endl;
    }
}
//enum IA_TYPE_LIST { IA_FOLLOW, IA_ZIGZAG, IA_SIDETOSIDE, IA_BAT, IA_ROOF_SHOOTER, IA_GROUND_SHOOTER, IA_SHOOT_AND_GO, IA_FLY_ZIG_ZAG, IA_BUTTERFLY, IA_HORIZONTAL_GO_AHEAD, IA_HORIZONTAL_TURN, IA_FIXED_JUMPER, IA_SIDE_SHOOTER, IA_GHOST, IA_FISH, IA_DOLPHIN, IA_VERTICAL_ZIGZAG, IA_TYPES_COUNT };

void artificial_inteligence::execute_ai_step_walk()
{
    st_position dest_point(0, position.y);

	if (_ai_state.sub_status == IA_ACTION_STATE_INITIAL) {
        if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_WALK_OPTION_HORIZONTAL_TURN) {
            if (state.direction == ANIM_DIRECTION_LEFT) {
                std::cout << ">>>>>>> artificial_inteligence::execute_ai_step_walk - TURN TO RIGHT" << std::endl;
                state.direction = ANIM_DIRECTION_RIGHT; /// @TODO - turn if not facing the given direction
            } else {
                std::cout << ">>>>>>> artificial_inteligence::execute_ai_step_walk - TURN TO LEFT" << std::endl;
                state.direction = ANIM_DIRECTION_LEFT;
            }
        }
        if (state.direction == ANIM_DIRECTION_LEFT) {
            dest_point.x = position.x + frameSize.width/2 - walk_range;
        } else {
            dest_point.x = position.x + frameSize.width/2 + walk_range;
        }
		_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
        state.animation_type = ANIM_TYPE_WALK;
	} else {
		if (state.animation_type != ANIM_TYPE_TURN) {
			bool must_turn = false;
			int xinc = 0;

			if (state.direction == ANIM_DIRECTION_LEFT) {
				xinc = -move_speed;
			} else {
				xinc = move_speed;
			}

			if (can_fly == false && test_change_position(xinc, TILESIZE*0.8) == true) {
                //std::cout << "*** artificial_inteligence::execute_ai_step_walk() - HOLE ahead - pos.x: " << position.x << ", xinc: " << xinc << std::endl;
				must_turn = true;
			}
			if (must_turn == false && !change_position(xinc, 0)) { // change direction if colided
                //std::cout << ">>> artificial_inteligence::execute_ai_step_walk() - BLOCK ahead" << std::endl;
				must_turn = true;
			}
            if (distance.width >= walk_range) {
                must_turn = true;
            }
			if (must_turn == true) {
				state.animation_type = ANIM_TYPE_TURN;
				state.animation_state = 0;
				distance.width = 0;
				state.animation_timer = timer.getTimer() + 200;
                state.direction = !state.direction;
                _ai_timer = timer.getTimer() + game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].go_to_delay;
                _ai_state.sub_status = IA_ACTION_STATE_FINISHED;
				return;
			}
			distance.width += move_speed;
			//std::cout << "artificial_inteligence::exec_IA_ZIGZAG() - name: " << name << ", distance.width: " << distance.width << ", walk_range: " << walk_range << ", state.animation_type: " << state.animation_type << std::endl;
        } else if (have_frame_graphic(state.direction, state.animation_type, (state.animation_state+1)) == false) {
			state.animation_type = ANIM_TYPE_STAND;
			_ai_timer = timer.getTimer() + game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].go_to_delay;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
		}
		last_execute_time = timer.getTimer() + 20;
	}
}

void artificial_inteligence::execute_ai_action_wait_until_player_in_range()
{
	if (_ai_state.sub_status == IA_ACTION_STATE_INITIAL) {
		_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		state.animation_type = ANIM_TYPE_STAND;
	} else {
		struct_player_dist dist_players = dist_npc_players();
		int dist_player = abs((float)dist_players.pObj->getPosition().x - position.x);
		if (dist_player <= walk_range) {
			_ai_timer = timer.getTimer() + game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].go_to_delay;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
		}
	}
}

void artificial_inteligence::execute_ai_action_trow_projectile(unsigned short n, bool invert_direction)
{
	if (_ai_state.sub_status == IA_ACTION_STATE_INITIAL) {
        //std::cout << "artificial_inteligence::execute_ai_action_trow_projectile - START, id: " << game_data.game_npcs[_number].projectile_id[n] << ", trajectory: " << game_data.projectiles[game_data.game_npcs[_number].projectile_id[n]].trajectory << std::endl;
		state.animation_type = ANIM_TYPE_ATTACK;
		state.animation_state = 0;
		state.animation_timer = 0;
		_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		st_position proj_pos;
		int proj_direction = state.direction;
		if (invert_direction == true) {
			proj_direction = !proj_direction;
		}
		if (proj_direction == ANIM_DIRECTION_LEFT) {
            proj_pos = st_position(position.x+TILESIZE/2, position.y+frameSize.height/2);
		} else {
            proj_pos = st_position(position.x+frameSize.width-TILESIZE*2, position.y+frameSize.height/2);
		}
		projectile *temp_proj = new projectile(game_data.game_npcs[_number].projectile_id[n], proj_direction, proj_pos, map);
		projectile_list.push_back(temp_proj);

		if (game_data.projectiles[game_data.game_npcs[_number].projectile_id[n]].trajectory == TRAJECTORY_PLAYER_DIRECTION) {
			//std::cout << "shooting follow player trajectory" << std::endl;
			if (!is_player() && map->_player_list.size() > 0) {
				//std::cout << "found player" << std::endl;
				character* p_player = map->_player_list.at(0);
				temp_proj->set_target_position(p_player->get_position_ref());
			//} else {
				//std::cout << "player NOT found" << std::endl;
			}
		}

	} else {
		//std::cout << "anim_type: " << state.animation_type << ", type_attack: " << ANIM_TYPE_ATTACK << std::endl;
		if (have_frame_graphic(state.direction, state.animation_type, (state.animation_state+1)) == false) {
			//std::cout << "artificial_inteligence::execute_ai_action_trow_projectile - FINISH - delay: " << game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].go_to_delay << std::endl;
			_ai_timer = timer.getTimer() + game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].go_to_delay;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
			state.animation_type = ANIM_TYPE_STAND;
		}
	}
}

void artificial_inteligence::execute_ai_step_fly()
{
	if (_ai_state.sub_status == IA_ACTION_STATE_INITIAL) {
        //std::cout << "artificial_inteligence::execute_ai_step_fly - START" << std::endl;
        if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_TO_PLAYER) {
            _dest_point = position;
        } else if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_TO_RANDOM_POINT) {
            int rand_x = rand() % RES_W;
            int rand_y = rand() % RES_H;
            _dest_point = st_position(rand_x, rand_y);
        } else if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_HORIZONTAL_AHEAD) {
            //std::cout << "artificial_inteligence::execute_ai_step_fly - HORIZONTAL AHEAD" << std::endl;
            if (state.direction == ANIM_DIRECTION_LEFT) {
                _dest_point.x = position.x + frameSize.width/2 - walk_range;
            } else {
                _dest_point.x = position.x + frameSize.width/2 + walk_range;
            }
        } else if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_UP) {
            _dest_point.y = position.y + frameSize.height/2 - walk_range;
            //std::cout << "artificial_inteligence::execute_ai_step_fly - AI_ACTION_FLY_OPTION_UP - walk_range: " << walk_range << ", dest_point.y: " << _dest_point.y << std::endl;
        } else if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_DOWN) {
            _dest_point.y = position.y + frameSize.height/2 + walk_range;
            //std::cout << "artificial_inteligence::execute_ai_step_fly - AI_ACTION_FLY_OPTION_DOWN - walk_range: " << walk_range << ", dest_point.y: " << _dest_point.y << std::endl;
        } else if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_FALL) {
            _dest_point.y = RES_H + frameSize.height + 2;
            //std::cout << "artificial_inteligence::execute_ai_step_fly - AI_ACTION_FLY_OPTION_FALL - walk_range: " << walk_range << ", dest_point.y: " << _dest_point.y << std::endl;
        }


		state.animation_type = ANIM_TYPE_WALK;
		/*
		if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_OPTIONS_LEFT) {
			state.direction = ANIM_DIRECTION_LEFT; /// @TODO - turn if not facing the given direction
		} else {
			state.direction = ANIM_DIRECTION_RIGHT;
		}
		*/
		_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
	} else {
		if (state.animation_type == ANIM_TYPE_TURN && have_frame_graphic(state.direction, state.animation_type, (state.animation_state+1)) == false) {
            //std::cout << "*************** artificial_inteligence::execute_ai_step_fly - reached last turn frame, reset to stand" << std::endl;
			state.animation_type = ANIM_TYPE_STAND;
            state.direction = !state.direction;
			_ai_timer = timer.getTimer() + game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].go_to_delay;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
			return;
		}

        if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_HORIZONTAL_AHEAD) {
            if (state.direction == ANIM_DIRECTION_LEFT) {
                //std::cout << "artificial_inteligence::execute_ai_step_fly - HORIZONTAL LEFT" << std::endl;
                if (move_to_point(_dest_point, move_speed, 0, is_ghost) == true) {
                    _ai_state.sub_status = IA_ACTION_STATE_FINISHED;
                }
            } else {
                //std::cout << "artificial_inteligence::execute_ai_step_fly - HORIZONTAL RIGHT" << std::endl;
                if (move_to_point(_dest_point, move_speed, 0, is_ghost) == true) {
                    _ai_state.sub_status = IA_ACTION_STATE_FINISHED;
                }
            }
        } else if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_HORIZONTAL_TURN && state.animation_type != ANIM_TYPE_TURN) {
            //std::cout << "artificial_inteligence::execute_ai_step_fly - HORIZONTAL TURN" << std::endl;
            state.animation_type = ANIM_TYPE_TURN;
            state.animation_state = 0;
            distance.width = 0;
            state.animation_timer = timer.getTimer() + 200;
        } else if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_UP) {
			//std::cout << "artificial_inteligence::execute_ai_step_fly - UP" << std::endl;
            if (move_to_point(_dest_point, 0, move_speed, is_ghost) == true) {
                _ai_state.sub_status = IA_ACTION_STATE_FINISHED;
            }
        } else if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_DOWN) {
            //std::cout << "artificial_inteligence::execute_ai_step_fly - DOWN - dest_point.y: " << _dest_point.y << std::endl;
            if (move_to_point(_dest_point, 0, move_speed, is_ghost) == true) {
                _ai_state.sub_status = IA_ACTION_STATE_FINISHED;
            }
        } else if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_FALL) {
            //std::cout << "artificial_inteligence::execute_ai_step_fly - DOWN - dest_point.y: " << _dest_point.y << std::endl;
            if (move_to_point(_dest_point, 0, move_speed, is_ghost) == true) {
                _ai_state.sub_status = IA_ACTION_STATE_FINISHED;
            }
        } else if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_TO_PLAYER) {
			struct_player_dist dist_players = dist_npc_players();
            if (move_to_point(dist_players.pObj->getPosition(), move_speed, move_speed, is_ghost) == true) {
                _ai_state.sub_status = IA_ACTION_STATE_FINISHED;
            }
        } else if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_TO_SAVED_POINT || game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_FLY_OPTION_TO_RANDOM_POINT) {
			//std::cout << "artificial_inteligence::execute_ai_step_fly - POINT" << std::endl;
            if (move_to_point(_dest_point, move_speed, move_speed, is_ghost) == true) {
                _ai_state.sub_status = IA_ACTION_STATE_FINISHED;
            }

        }
    }
}

void artificial_inteligence::execute_ai_step_dash()
{
    st_position dest_point(0, position.y);
    if (_ai_state.sub_status == IA_ACTION_STATE_INITIAL) {
        if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_DASH_OPTION_LEFT) {
            state.direction = ANIM_DIRECTION_LEFT; /// @TODO - turn if not facing the given direction
            dest_point.x = position.x + frameSize.width/2 - walk_range;
        } else {
            state.direction = ANIM_DIRECTION_RIGHT;
            dest_point.x = position.x + frameSize.width/2 + walk_range;
        }
        state.animation_type = ANIM_TYPE_SLIDE;
        _ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
    } else {

        if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_DASH_OPTION_LEFT) {
            if (move_to_point(dest_point, max_speed-3, 0, is_ghost) == true) {
                _ai_state.sub_status = IA_ACTION_STATE_FINISHED;
            }
        } else if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_DASH_OPTION_RIGHT) {
            if (move_to_point(dest_point, max_speed-3, 0, is_ghost) == true) {
                _ai_state.sub_status = IA_ACTION_STATE_FINISHED;
            }
        } else if (game_data.ai_types[game_data.game_npcs[_number].gravity_level].states[_ai_chain_n].extra_parameter == AI_ACTION_DASH_OPTION_TO_PLAYER) {
            struct_player_dist dist_players = dist_npc_players();

            //std::cout << "artificial_inteligence::execute_ai_step_dash - PLAYER - dist_players.dist: " << dist_players.dist << std::endl;

            if (dist_players.dist < TILESIZE/2) {
                //std::cout << "artificial_inteligence::execute_ai_step_fly - PLAYER - FINISH" << std::endl;
                state.animation_type = ANIM_TYPE_STAND;
                _ai_state.sub_status = IA_ACTION_STATE_FINISHED;
                return;
            } else {
                if (move_to_point(dist_players.pObj->getPosition(), max_speed-3, max_speed-3, is_ghost) == true) {
                    _ai_state.sub_status = IA_ACTION_STATE_FINISHED;
                }
            }
        }
    }
    last_execute_time = timer.getTimer() + 20;
}

bool artificial_inteligence::move_to_point(st_position dest_point, int speed_x, int speed_y, bool can_pass_walls)
{
    int xinc = 0;
    int yinc = 0;

    // invert direction if needed
    if (position.x > dest_point.x && state.direction != ANIM_DIRECTION_LEFT) {
        state.direction = ANIM_DIRECTION_LEFT;
    }
    if (position.x < dest_point.x && state.direction != ANIM_DIRECTION_RIGHT) {
        state.direction = ANIM_DIRECTION_RIGHT;
    }

    if (abs(dest_point.x - position.x) < speed_x) {
        speed_x = abs(dest_point.x - position.x);
    }
    if (abs(dest_point.y - position.y) < speed_y) {
        speed_y = abs(dest_point.y - position.y);
    }

    if (dest_point.x > position.x) {
        xinc = speed_x;
    } else if (dest_point.x < position.x) {
        xinc = -speed_x;
    }
    if (dest_point.y > position.y) {
        yinc = speed_y;
    } else if (dest_point.y < position.y) {
        yinc = -speed_y;
    }


    // checking
    bool can_move_x = true;
    bool can_move_y = true;
    //std::cout << ">> artificial_inteligence::move_to_point - dest_point.y: " << dest_point.y << ", position.y: " << position.y << ", yinc: " << yinc << ", can_move_y: " << can_move_y << std::endl;
    if (can_pass_walls == false) {
        can_move_x = test_change_position(xinc, 0);
        can_move_y = test_change_position(0, yinc);
    }
    if ((xinc == 0 && yinc == 0) || (can_move_x == false && can_move_y == false) || (can_move_x == false && yinc == 0) || (can_move_y == false && xinc == 0)) {
        return true;
    } else {
        if (can_move_x == true) {
            position.x += xinc;
        }
        if (can_move_y == true) {
            position.y += yinc;
        }
    }
    return false;
}
