#ifndef ARTIFICIAL_INTELIGENCE_H
#define ARTIFICIAL_INTELIGENCE_H

#include "character.h"
#include <cmath>


/**
 * @brief AI sub-state, indicates if the current state is starting (must set initial values), executing or finishing (must set next step and set values)
 */
enum e_action_states { IA_ACTION_STATE_INITIAL, IA_ACTION_STATE_EXECUTING, IA_ACTION_STATE_FINISHED  };

/**
 * @brief holds information about the distance of the current NPC for the closest player
 */
struct struct_player_dist {
    int dist; /**< distance (diagonal), in pixels, to the player */
    character *pObj; /**< pointer to the player object */
    st_position dist_xy; /**< distance separated for X and Y axis */
};


/**
 * @brief
 */
struct struct_ia_state {
    short int main_status;										/**< AI main state */
    short int sub_status;                                       /**< TODO */
    short int action_status;                                    /**< TODO */
    st_position initial_position;                               /**< TODO */
    st_position secondary_position;                             /**< TODO */
    int timer;                                                  /**< TODO */
    int step;                                                   /**< TODO */
	struct_ia_state()
	{
		main_status = 0;
		sub_status = 0;
		action_status = IA_ACTION_STATE_INITIAL;
		timer = 0;
		step = 0;
	}
};

/**
 * @brief class, child of character and parent of classNPC, that holds artificial inteligence methods used by NPC class
 */
class artificial_inteligence : public character
{

public:
    /**
     * @brief constructor
     */
    artificial_inteligence();

    // AI methods @TODO: deprecated, must be replaced by AI chain system
    /**
     * @brief
     */
    void exec_IA_ZIGZAG();

    /**
     * @brief
     */
    void exec_IA_VERTICAL_ZIGZAG();

    /**
     * @brief
     */
    void exec_IA_GHOST();

    /**
     * @brief
     */
    void exec_IA_FISH();

    /**
     * @brief
     */
    void exec_IA_DOLPHIN();

    /**
     * @brief
     */
    void exec_IA_SIDETOSIDE();

    /**
     * @brief
     * @param can_cross_walls
     */
    void exec_IA_FLOATER(bool can_cross_walls);

    /**
     * @brief
     */
    void ground_damage_players();

protected:
    /**
     * @brief
     * @return struct_player_dist
     */
    struct_player_dist dist_npc_players();

    // AI ACTIONS
    /**
     * @brief
     */
    void ia_action_jump_to_player();

    /**
     * @brief
     */
    void ia_action_jump_to_roof();

    /**
     * @brief
     */
    void ia_action_air_walk();

    /**
     * @brief
     */
    void ia_action_jump_fall();

    /**
     * @brief
     */
    void ia_action_quake_attack();

    /**
     * @brief
     */
    void ia_accelerate_up();

    /**
     * @brief
     */
    void ia_accelerate_down();

    /**
     * @brief
     */
    void ia_teleport();

    /**
     * @brief
     * @param dest_point
     */
    void ia_set_destiny_point(st_position dest_point);

    /**
     * @brief
     */
    void ia_walk_to_position();

    /**
     * @brief same as walk to position, but for floaters
     */
    void ia_float_to_position();

    /**
     * @brief
     */
    void ia_dash();

    /**
     * @brief
     */
    void ia_shot_stop_time();

    /**
     * @brief
     */
    void ia_jump_to_wall();

    /**
     * @brief
     * @param type
     * @param n
     * @param alternated
     */
    void throw_projectile_npc(int type, int n, int alternated);

    // main methods
    /**
     * @brief
     */
    void execute_basic_ai(); // for basic NPC AI

    /**
     * @brief current (new) method for AI execution: a chain of small actions defined by game_data
     */
    void execute_chain_ai(); // more complex AI (for bosses)


    // NEW AI
    /**
     * @brief
     */
    void execute_ai();

    /**
     * @brief
     */
    void define_ai_next_step();

    /**
     * @brief
     */
    void execute_ai_step();

    /**
     * @brief
     */
    void execute_ai_step_walk();

    /**
     * @brief
     */
    void execute_ai_action_wait_until_player_in_range();

    /**
     * @brief
     * @param n
     * @param invert_direction
     */
    void execute_ai_action_trow_projectile(unsigned short n, bool invert_direction);

    /**
     * @brief
     */
    void execute_ai_step_fly();

    /**
     * @brief
     */
    void execute_ai_step_dash();

    /**
     * @brief moves to a point, returns true when point was reached or can't reach the target
     * @param dest_point point of destiny
     * @param speed_x how fast must move on X axis
     * @param speed_y how fast must move on Y axis
     * @param can_pass_walls the NPC can cross walls like a ghost or not
     * @return bool true -> point reached
     */
    bool move_to_point(st_position dest_point, int speed_x, int speed_y, bool can_pass_walls);




protected:
    st_size distance; /**< TODO */
    short int walk_range;								// defines how long from the start point it can go /**< TODO */
    character *target;									// number of the player that is target /**< TODO */
    float speed_y; /**< TODO */
    float acceleration_y; /**< TODO */
    st_position _origin_point;							// used as target for moving /**< TODO */
    st_position _dest_point; /**< TODO */


    struct_ia_state _ai_state; /**< TODO */
    bool is_ghost;										// if can shoot or not /**< TODO */


	// "external" members
    static std::vector<character*> *player_list;					// reference to the players list /**< TODO */
    int IA_type;										// IA types: no_move, wait, attack, zig-zag (loop), side to side (origin to direction) /**< TODO */
    unsigned int _ai_timer;										// used to not execute AI before a while /**< TODO */
    unsigned short _ai_chain_n;							// indicates witch one of the four AI actions is the NPC on /**< TODO */

};

#endif // ARTIFICIAL_INTELIGENCE_H
