#include "classboss.h"

#include "classmap.h"

#include "timerlib.h"
extern timerLib timer;

#include "scenes/dialogs.h"

#include "defines.h"
#include <cmath>
#include <cstdlib>
#include "game.h"
extern game gameControl;

extern CURRENT_FILE_FORMAT::file_game game_data;

classboss::classboss(std::string set_name) : _initialized(0), _screen_blinked(false)
{
	name = set_name;
    this->set_is_boss(true);
	_ai_state.initial_position.y = -200;
	position.y = _ai_state.initial_position.y;
    hitPoints.total = PLAYER_INITIAL_HP;
    hitPoints.current = hitPoints.total;
	//std::cout << "classboss::classboss [" << name << "] - is_boss: " << _is_boss << std::endl;
}

classboss::classboss(int stage_id, int map_id, int main_id, int id) : classnpc(stage_id, map_id, main_id, id)
{
	_initialized = 0;
	_screen_blinked = false;
	this->set_is_boss(true);
	_ai_state.initial_position.y = -50;
	position.y = _ai_state.initial_position.y;
	hitPoints.total = PLAYER_INITIAL_HP;
    hitPoints.current = hitPoints.total;
	//std::cout << "BOSS 1 - stage_id: " << stage_id << ", map_id: " << map_id << ", main_id: " << main_id << ", id: " << id << std::endl;
}


void classboss::boss_move() {
	move_projectiles();
	//std::cout << "classboss::boss_move - _initialized: " << _initialized << std::endl;
    if (_initialized == 0 && _is_boss == true) { /// @TODO: move this logic to map (player should not move while boss is presenting)
		_initialized++;
		state.animation_type = ANIM_TYPE_TELEPORT;
        gameControl.boss_intro(_is_boss);
		return;
    } else if (_initialized == 1 && _is_boss == true) {
		if (position.x > RES_H/3 && gravity(true) == false) {
			_initialized++;
		}
        state.move_timer = timer.getTimer()+30;
		return;
	}


	//std::cout << "classboss::move [" << name << "]" << std::endl;
	/*
	int mapScrollX = map->getMapScrolling().x;
	int lock_point = map->getMapPointLock(st_position(position.x/16, position.y/16));
	struct struct_player_dist dist_players = dist_npc_players();
	*/

	//std::cout << "classboss::boss_move - name: " << name << std::endl;

	if (first_run == 0) {
		// show boss dialogs here
		first_run = 1;
		//boss_energy = temp_npc;
		//play_boss_music();
		//show_dialog(STAGE_N, 1);
	}

	if (_ai_timer > timer.getTimer()) {
		//gravity(false);
		return;
	}


	if (name == "Daisie Bot") {
		exec_daisiebot();
	} else if (name == "Ape Bot") {
		exec_apebot();
	} else if (name == "Mummy Bot") {
		if (state.animation_type != ANIM_TYPE_SPECIAL_ATTACK) {
			state.animation_type = ANIM_TYPE_SPECIAL_ATTACK;
		}
		exec_mummybot();
	} else if (name == "Dynamite Bot") {
		if (state.animation_type != ANIM_TYPE_STAND) {
			state.animation_type = ANIM_TYPE_STAND;
		}
		exec_dynamitebot();
	} else if (name == "Spike Bot") {
		exec_spikebot();
	} else if (name == "Techno Bot") {
		exec_technobot();
	} else if (name == "Mage Bot") {
		exec_magebot();
	} else if (name == "Seahorse Bot") {
        //exec_seahorse();
        execute_ai();
	} else {
        execute_ai();
		gravity(false);
	}
	execute_chain_ai();
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void classboss::execute()
{
	if (hitPoints.current <= 0 || position.x < map->getMapScrolling().x-TILESIZE*2 || position.x > map->getMapScrolling().x+RES_W+TILESIZE*2) {
		return;
	}
	if (last_execute_time > timer.getTimer()) {
		return;
	}
	if (state.move_timer > timer.getTimer()) {
		move_projectiles();
		return;
	}
	//std::cout << "classboss::execute" << std::endl;
	boss_move();
	charMove();
}


void classboss::exec_daisiebot()
{
	//std::cout << "classboss::exec_daisiebot - START - ia_state.main_status: " << ia_state.main_status << std::endl;

	if (_ai_state.main_status == IA_STATE_WAIT_TIME || _ai_state.main_status == IA_STATE_RETURN) {
		std::cout << "classboss::exec_daisiebot - WAIT -> STAND" << std::endl;
		_ai_state.main_status = IA_STAND;
	}

	// stand -> shot rainbow -> cross screen -> use flowers
	if (_ai_state.main_status == IA_STAND) {
		std::cout << "classboss::exec_daisiebot - STAND -> CROSS" << std::endl;
		_ai_state.main_status = IA_STATE_CROSS_SCREEN;
		return;
	}

	if (_ai_state.main_status == IA_STATE_SHOT_FLOWER) {
		std::cout << "classboss::exec_daisiebot - IA_STATE_SHOT_XXX" << std::endl;
		state.animation_type = ANIM_TYPE_STAND;
		/// @TODO (use attacks/weapons system) shot_flower(temp_npc);
		/// temp_npc->projectile.timer = getTimer()+60;

		st_position proj_pos;
		if (state.direction == ANIM_DIRECTION_LEFT) {
			proj_pos = st_position(position.x+TILESIZE/2, position.y+frameSize.height/2);
		} else {
			proj_pos = st_position(position.x+frameSize.width-TILESIZE*2, position.y+frameSize.height/2);
		}
		int rand_n = rand() % 100;
		if (rand_n < 50) {
			// RAINBOW
			std::cout << "classboss::exec_daisiebot - IA_STATE_SHOT_RAINBOW" << std::endl;
            //projectile_list.push_back(new projectile(game_data.weapons[2].id_projectile, state.direction, PROJECTILE_DEFAULT_SPEED, proj_pos, map, game_data.weapons[2].damage, game_data.projectiles[game_data.weapons[2].id_projectile].projectile_type, 2));
			projectile_list.push_back(new projectile(game_data.game_npcs[_number].projectile_id[0], state.direction, proj_pos, map));
		} else {
			// FLOWER
			std::cout << "classboss::exec_daisiebot - IA_STATE_SHOT_FLOWER" << std::endl;
			proj_pos.y = position.y+frameSize.height - 7;
			projectile_list.push_back(new projectile(game_data.game_npcs[_number].projectile_id[1], state.direction, proj_pos, map));
		}

		_ai_state.initial_position.x = position.x;
		_ai_state.initial_position.y = position.y;
		_ai_state.main_status = IA_STATE_WAIT_TIME;
		state.move_timer = timer.getTimer()+3000;
	}

	/// @TODO: move to generic npc, once more will use it
	if (_ai_state.main_status == IA_STATE_CROSS_SCREEN) {
		state.animation_type = ANIM_TYPE_WALK;
		//std::cout << "classboss::exec_daisiebot - IA_STATE_CROSS_SCREEN" << std::endl;
		// check if must stop/invert
		state.move_timer = timer.getTimer() + 10;
		if (state.direction == ANIM_DIRECTION_LEFT) {
			int map_tile_x = (position.x-move_speed)/16;
			int map_tile_y = (position.y+frameSize.height-2)/16;
			if (distance.width < 18*TILESIZE && (position.x-move_speed > 0 && (map->getMapPointLock(st_position(map_tile_x, map_tile_y)) == 0 || map->getMapPointLock(st_position(map_tile_x, map_tile_y)) == TERRAIN_WATER))) {
				position.x-= move_speed;
				distance.width += move_speed;
			} else {
				distance.width = 0;
				invert_direction();
				_ai_state.main_status = IA_STATE_SHOT_FLOWER;
			}
		} else { // ANIM_RIGHT
			int map_tile_x = (position.x+frameSize.width+move_speed)/16;
			int map_tile_y = (position.y+frameSize.height-2)/16;
			//map->getMapPointLock(st_position(map_tile_x, map_tile_y))
			if (distance.width < 18*TILESIZE && (map->getMapPointLock(st_position(map_tile_x, map_tile_y)) == 0 || map->getMapPointLock(st_position(map_tile_x, map_tile_y)) == TERRAIN_WATER)) {
				position.x+= move_speed;
				distance.width += move_speed;
			} else {
				distance.width = 0;
				invert_direction();
				_ai_state.main_status = IA_STATE_SHOT_FLOWER;
			}
		}
	}
	//std::cout << "classboss::exec_daisiebot - END" << std::endl;
}




void classboss::exec_apebot()
{
	//std::cout << "classboss::exec_apebot - ia_state.main_status: " << ia_state.main_status << std::endl;

	apebot_state_machine(); // initial state or finished sub_state, so pick a new state

	if (_ai_state.main_status == IA_STATE_JUMP_TO_ROOF) {
		if (_ai_state.sub_status != IA_ACTION_STATE_FINISHED) {
			ia_action_jump_to_roof();
		} else { // FINISHED
			_ai_state.main_status = IA_STATE_WAIT_TIME;
		}
	} else if (_ai_state.main_status == IA_STATE_AIR_WALK) {
		ia_action_air_walk();
	}
}

void classboss::apebot_state_machine()
{
	if (_ai_state.main_status == IA_INITIAL_STATE || _ai_state.sub_status == IA_ACTION_STATE_FINISHED) {
		if (_ai_state.main_status == IA_STATE_JUMP_TO_ROOF) { // when jump to roof is finished, move to air_walk
			//std::cout << ">>> apebot_state_machine - IA_STATE_JUMP_TO_ROOF -> IA_STATE_AIR_WALK" << std::endl;
			_ai_state.sub_status = IA_ACTION_STATE_INITIAL;
			_ai_state.main_status = IA_STATE_AIR_WALK;
			_ai_state.timer = timer.getTimer() + 1200;
			return;
		}
		if (_ai_state.main_status == IA_STATE_AIR_WALK) { // when air_walk is finished, move to fall_down
			//std::cout << ">>> apebot_state_machine - IA_STATE_AIR_WALK -> IA_STATE_JUMP_FALL" << std::endl;
			_ai_state.main_status = IA_STATE_JUMP_FALL;
			_ai_state.sub_status = IA_ACTION_STATE_INITIAL;
			return;
		}


		int rand_n = rand() % 100;
        //std::cout << "<<<< PICK NEW ACTION - rand_n: " << rand_n << " >>>" << std::endl;
		if (rand_n < 20) {
			//std::cout << "<<<< NEW ACTION - IA_STATE_QUAKE_ATTACK >>>" << std::endl;
			_ai_state.main_status = IA_STATE_QUAKE_ATTACK;
			state.animation_type = ANIM_TYPE_SPECIAL_ATTACK;
			state.move_timer = timer.getTimer() + 100;
		} else if (rand_n < 80) {
			//std::cout << "<<<< NEW ACTION - IA_STATE_JUMP_TO_PLAYER >>>" << std::endl;
			_ai_state.main_status = IA_STATE_JUMP_TO_PLAYER;
			_ai_state.sub_status = IA_ACTION_STATE_INITIAL;
		} else {
			//std::cout << "<<<< NEW ACTION - IA_STATE_JUMP_TO_ROOF >>>" << std::endl;
			_ai_state.main_status = IA_STATE_JUMP_TO_ROOF;
			state.animation_type = ANIM_TYPE_JUMP;
		}
		_ai_state.sub_status = IA_ACTION_STATE_INITIAL;
	}
}

void classboss::mummybot_state_machine()
{
	if (_ai_state.main_status == IA_INITIAL_STATE || _ai_state.sub_status == IA_ACTION_STATE_FINISHED) {
		int rand_n = rand() % 100;
		std::cout << "mummybot_state_machine - rand_n: " << rand_n << std::endl;
		if (rand_n < 80) {
			std::cout << "mummybot_state_machine - START SHOT " << std::endl;
			_ai_state.main_status = IA_STATE_WAIT_TIME;
			state.animation_type = ANIM_TYPE_ATTACK;
			state.move_timer = timer.getTimer() + 1000;
			st_position proj_pos;
			if (state.direction == ANIM_DIRECTION_LEFT) {
				proj_pos = st_position(position.x+TILESIZE/2, position.y+frameSize.height/2);
			} else {
				proj_pos = st_position(position.x+frameSize.width-TILESIZE*2, position.y+frameSize.height/2);
			}
			//projectile(int set_direction, int set_speed, st_position set_position, classMap *set_map, int damage, short int move_type, short wpn_id, std::string p_filename, st_size p_size);
			projectile_list.push_back(new projectile(game_data.game_npcs[_number].projectile_id[0], state.direction, proj_pos, map));

			state.move_timer = timer.getTimer() + 500;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
		} else {
			std::cout << "mummybot_state_machine - START TELEPORT" << std::endl;
			_ai_state.main_status = IA_STATE_TELEPORT;
			state.animation_type = ANIM_TYPE_SPECIAL_ATTACK;
			state.move_timer = timer.getTimer() + 40;
			_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		}

	}
}

void classboss::exec_mummybot()
{
	mummybot_state_machine();

}

void classboss::exec_dynamitebot()
{
	dynamitebot_state_machine();
}

void classboss::dynamitebot_state_machine()
{
	// dash -> walk x2 -> shot -> repeat
	if (_ai_state.main_status == IA_INITIAL_STATE || _ai_state.sub_status == IA_ACTION_STATE_FINISHED) {
		// initial -> dash OR stop-time -> dash
		if (_ai_state.main_status == 0 || (_ai_state.sub_status == IA_ACTION_STATE_FINISHED && _ai_state.main_status == IA_STATE_SHOT_STOP_TIME)) {
			if (_ai_state.main_status == 0) {
				std::cout << "DYNAMITE - initial -> dash" << std::endl;
			} else {
				std::cout << "DYNAMITE - stop-time -> dash" << std::endl;
			}

			struct_player_dist dist_npc_player = dist_npc_players(); // @TEMP
			ia_set_destiny_point(dist_npc_player.pObj->getPosition()); // @TEMP
			_ai_state.main_status = IA_STATE_CHARGED_DASH;
			state.animation_type = ANIM_TYPE_SPECIAL_ATTACK;
			state.move_timer = timer.getTimer() + 40;
			_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		// dash -> walk
		} else if (_ai_state.sub_status == IA_ACTION_STATE_FINISHED && _ai_state.main_status == IA_STATE_CHARGED_DASH) {
			std::cout << "DYNAMITE - dash -> walk" << std::endl;
			// after dashing, walk following player (twice)
			struct_player_dist dist_npc_player = dist_npc_players();
			ia_set_destiny_point(dist_npc_player.pObj->getPosition());
			_ai_state.main_status = IA_STATE_WALK_TO_POINT;
			state.animation_type = ANIM_TYPE_WALK;
			state.move_timer = timer.getTimer() + 40;
			_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		// walk -> shot
		} else if (_ai_state.sub_status == IA_ACTION_STATE_FINISHED && _ai_state.main_status == IA_STATE_WALK_TO_POINT) {
			std::cout << "DYNAMITE - walk -> shot" << std::endl;
			_ai_state.main_status = IA_STATE_SHOT_STOP_TIME;
			state.animation_type = ANIM_TYPE_ATTACK;
			state.move_timer = timer.getTimer() + 40;
			_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		}
	}
}

void classboss::exec_spikebot()
{
	spikebot_state_machine();
}

void classboss::spikebot_state_machine()
{
	if (_ai_state.main_status == IA_INITIAL_STATE || _ai_state.sub_status == IA_ACTION_STATE_FINISHED) {
		int rand_n = rand() % 100;
		std::cout << "classboss::spikebot_state_machine - PICK NEW" << std::endl;
		// jump to wall
		if (rand_n < 33) {
			std::cout << "classboss::spikebot_state_machine - new jump_to_walk" << std::endl;
			struct_player_dist dist_npc_player = dist_npc_players();
			if (position.x < dist_npc_player.pObj->getPosition().x) {
				state.direction = ANIM_DIRECTION_RIGHT;
			} else {
				state.direction = ANIM_DIRECTION_LEFT;
			}
			_ai_state.action_status = 0;
			state.animation_type = ANIM_TYPE_JUMP;
			_ai_state.main_status = IA_STATE_JUMP_TO_WALL;
			_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		// dash
		} else if (rand_n < 66) {
			std::cout << "classboss::spikebot_state_machine - new slide" << std::endl;
			struct_player_dist dist_npc_player = dist_npc_players(); // @TEMP
			ia_set_destiny_point(dist_npc_player.pObj->getPosition()); // @TEMP
			_ai_state.main_status = IA_STATE_CHARGED_DASH;
			state.animation_type = ANIM_TYPE_SLIDE;
			state.move_timer = timer.getTimer() + 40;
			_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		// jump to player
		} else {
			_ai_state.main_status = IA_STATE_JUMP_TO_PLAYER;
			_ai_state.sub_status = IA_ACTION_STATE_INITIAL;
		}
	}
}

void classboss::exec_technobot()
{
	technobot_state_machine();
}

void classboss::technobot_state_machine()
{
	if (_ai_state.main_status == IA_INITIAL_STATE || _ai_state.sub_status == IA_ACTION_STATE_FINISHED) {
		int rand_n = rand() % 100;
		if (rand_n < 33) { // PLUG ATTACK
			if (projectile_list.size() == 0) {
				std::cout << "classboss::spikebot_state_machine - new plug attack" << std::endl;
				state.animation_type = ANIM_TYPE_ATTACK;
				st_position proj_pos;
				if (state.direction == ANIM_DIRECTION_LEFT) {
					proj_pos = st_position(position.x+TILESIZE/2, position.y+frameSize.height/2);
				} else {
					proj_pos = st_position(position.x+frameSize.width-TILESIZE*2, position.y+frameSize.height/2);
				}
				projectile_list.push_back(new projectile(game_data.game_npcs[_number].projectile_id[0], state.direction, proj_pos, map));
			} else {
				state.animation_type = ANIM_TYPE_STAND;
			}

			state.move_timer = timer.getTimer() + 400;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
		} else if (rand_n < 66) { // BOMB ATTACK
			state.animation_type = ANIM_TYPE_ATTACK;
			std::cout << "classboss::spikebot_state_machine - bomb attack" << std::endl;
			if (projectile_list.size() == 0) {
				st_position proj_pos;
				if (state.direction == ANIM_DIRECTION_LEFT) {
					proj_pos = st_position(position.x+TILESIZE/2, position.y+frameSize.height/2);
				} else {
					proj_pos = st_position(position.x+frameSize.width-TILESIZE*2, position.y+frameSize.height/2);
				}
                //projectile_list.push_back(new projectile(game_data.weapons[wpn_id].id_projectile, state.direction, PROJECTILE_DEFAULT_SPEED, proj_pos, map, game_data.weapons[wpn_id].damage, game_data.projectiles[game_data.weapons[wpn_id].id_projectile].projectile_type, wpn_id));
				projectile_list.push_back(new projectile(game_data.game_npcs[_number].projectile_id[0], state.direction, proj_pos, map));
			} else {
				state.animation_type = ANIM_TYPE_STAND;
			}

			state.move_timer = timer.getTimer() + 400;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
		} else { // JUMP TO PLAYER
			_ai_state.main_status = IA_STATE_JUMP_TO_PLAYER;
			_ai_state.sub_status = IA_ACTION_STATE_INITIAL;
		}
	}
}

void classboss::exec_magebot()
{
	magebot_state_machine();
}

void classboss::magebot_state_machine()
{
	if (_ai_state.main_status == IA_INITIAL_STATE || _ai_state.sub_status == IA_ACTION_STATE_FINISHED) {
		int rand_n = rand() % 100;
		if (rand_n < 25) {
			std::cout << "magebot_state_machine - walk to player" << std::endl;
			struct_player_dist dist_npc_player = dist_npc_players();
			ia_set_destiny_point(dist_npc_player.pObj->getPosition());
			_ai_state.main_status = IA_STATE_WALK_TO_POINT;
			state.animation_type = ANIM_TYPE_WALK;
			state.move_timer = timer.getTimer() + 40;
			_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		} else if (rand_n < 50) {
			std::cout << "magebot_state_machine - shot star circle" << std::endl;
			st_position proj_pos;
			proj_pos = st_position(position.x+frameSize.width/2 - 28, position.y+frameSize.height/2);
			projectile *new_proj = new projectile(game_data.game_npcs[_number].projectile_id[0], state.direction, proj_pos, map);
			new_proj->set_owner_position(&position);
			new_proj->set_owner_direction(&state.direction);
			projectile_list.push_back(new_proj);
			state.animation_type = ANIM_TYPE_STAND;
			show();
			state.move_timer = timer.getTimer() + 2000;
			_ai_state.sub_status = IA_ACTION_STATE_FINISHED;
		} else if (rand_n < 75) {
			std::cout << "magebot_state_machine - wait" << std::endl;
			_ai_state.main_status = IA_STATE_SLEEP;
			state.move_timer = timer.getTimer() + 2000;
			state.animation_type = ANIM_TYPE_STAND;
			show();
			state.move_timer = timer.getTimer() + 2000;
		} else {
			std::cout << "magebot_state_machine - jump to player" << std::endl;
			_ai_state.main_status = IA_STATE_JUMP_TO_PLAYER;
			_ai_state.sub_status = IA_ACTION_STATE_INITIAL;
		}
	}
}

void classboss::exec_seahorse()
{
	seahorse_state_machine();
}

void classboss::seahorse_state_machine()
{
	if (_ai_state.main_status == IA_INITIAL_STATE || _ai_state.sub_status == IA_ACTION_STATE_FINISHED) {
		int rand_n = rand() % 100;
		std::cout << "classboss::seahorse_state_machine - PICK NEW - rand_n: " << rand_n << std::endl;
		// swimn to player
		if (rand_n < 33) {
			std::cout << "classboss::seahorse_state_machine - IA_STATE_FLOAT_TO_POINT" << std::endl;
			struct_player_dist dist_npc_player = dist_npc_players();
			ia_set_destiny_point(dist_npc_player.pObj->getPosition());
			_ai_state.main_status = IA_STATE_FLOAT_TO_POINT;
			state.animation_type = ANIM_TYPE_WALK;
			state.move_timer = timer.getTimer() + 40;
			_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		// dash
		} else if (rand_n < 66) {
			std::cout << "classboss::seahorse_state_machine - IA_STATE_CHARGED_DASH" << std::endl;
			struct_player_dist dist_npc_player = dist_npc_players(); // @TEMP
			if (position.x > dist_npc_player.pObj->getPosition().x) {
				state.direction = ANIM_DIRECTION_LEFT;
			} else {
				state.direction = ANIM_DIRECTION_RIGHT;
			}
			ia_set_destiny_point(dist_npc_player.pObj->getPosition()); // @TEMP
			_ai_state.main_status = IA_STATE_CHARGED_DASH;
			state.animation_type = ANIM_TYPE_SLIDE;
			state.move_timer = timer.getTimer() + 40;
			_ai_state.sub_status = IA_ACTION_STATE_EXECUTING;
		// shoot
		} else {
			std::cout << "classboss::seahorse_state_machine - SHOT" << std::endl;
			struct_player_dist dist_npc_player = dist_npc_players();
			if (position.x > dist_npc_player.pObj->getPosition().x) {
				state.direction = ANIM_DIRECTION_LEFT;
			} else {
				state.direction = ANIM_DIRECTION_RIGHT;
			}
			st_position proj_pos;
			if (state.direction == ANIM_DIRECTION_LEFT) {
				proj_pos = st_position(position.x+TILESIZE/2, position.y+frameSize.height/4);
			} else {
				proj_pos = st_position(position.x+frameSize.width-TILESIZE*2, position.y+frameSize.height/4);
			}
			//projectile(int set_direction, int set_speed, st_position set_position, classMap *set_map, int damage, short int move_type, short wpn_id, std::string p_filename, st_size p_size);
			std::cout << "dist_npc_player.dist_xy.y: " << dist_npc_player.dist_xy.y << std::endl;
            //projectile_list.push_back(new projectile(game_data.weapons[wpn_id].id_projectile, state.direction, PROJECTILE_DEFAULT_SPEED, proj_pos, map, game_data.weapons[wpn_id].damage, proj_trajectory, wpn_id));
			projectile_list.push_back(new projectile(game_data.game_npcs[_number].projectile_id[0], state.direction, proj_pos, map));
			state.move_timer = timer.getTimer() + 1000;
		}
    }
}

void classboss::death()
{
    state.jump_state = NO_JUMP;
    dead = true;
    if (_ai_state.main_status == IA_STATE_QUAKE_ATTACK) {
        graphLib.set_screen_adjust(st_position(0, 0));
    }
}

bool classboss::is_boss()
{
    return true;
}
