#include <sstream>
#include <list>

#include "classplayer.h"
#include "inputlib.h"
extern inputLib input;
#include "class_config.h"
extern class_config config_manager;

#include "soundlib.h"
extern soundLib soundManager;

#include "game.h"
extern game gameControl;

extern CURRENT_FILE_FORMAT::file_game game_data;

extern format_v_2_0_1::st_save game_save;

#include "classmap.h"

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
classPlayer::classPlayer(std::string set_name, int playerNumber) : teleporter_n(-1), selected_weapon(WEAPON_DEFAULT), l_key_released(true), r_key_released(true)
{
	_number = playerNumber-1;
	max_projectiles = game_data.players[_number].max_shots;
    //std::cout << "classPlayer::classPlayer - max_projectiles: " << max_projectiles << std::endl;
	position.y = -TILESIZE;
	position.x = 80;
	hit_duration = 2500;
	//hitPoints.total = game_data.players[_number].HP;
	hitPoints.total = PLAYER_INITIAL_HP;
	hitPoints.current = hitPoints.total;
	name = set_name;
	add_graphic();
	init_weapon_colors();
	initFrames();
	shield_type = SHIELD_FRONT; /// @TODO: from editor
	// load items from save
	if (game_data.players[_number].can_slide == true) {
		slide_type = 1;
	}
    _jump_accel = game_data.players[_number].jump_gravity;
    move_speed = game_data.players[_number].move_speed;
    _touch_damage_reductor = game_data.players[_number].touch_damage_reduction;
    if (_touch_damage_reductor > 0) {
        _hit_move_back_dist = TILESIZE;
    }
    _charged_shot_projectile_id = game_data.players[_number].full_charged_projectile_id;
    //std::cout << "Player[" << name << "].jump_gravity: " << _jump_accel << std::endl;
}


void classPlayer::init_weapon_colors()
{
	for (int i=0; i<3; i++) {
		color_keys[i] = game_data.players[_number].color_keys[i];
	}
	for (int i=0; i<MAX_WEAPON_N; i++) {
		weapon_colors[i] = game_data.players[_number].weapon_colors[i];
	}
}

bool classPlayer::get_item(object_colision &obj_info)
{
	if (character::get_item(obj_info)) {
		return true;
	}

	bool res = false;
	// deal with non-blocking items
	if (obj_info._object != NULL && obj_info._object->finished() == false) {
		//std::cout << "classPlayer::get_item" << std::endl;
		switch (obj_info._object->get_type()) {
		case OBJ_ENERGY_TANK:
            game_save.items.energy_tanks++;
			obj_info._object->set_finished(true);
			soundManager.play_sfx(SFX_GOT_ITEM);
			break;
		case OBJ_WEAPON_TANK:
            game_save.items.weapon_tanks++;
			obj_info._object->set_finished(true);
			soundManager.play_sfx(SFX_GOT_ITEM);
			break;
		case OBJ_LIFE:
            game_save.items.lifes++;
			obj_info._object->set_finished(true);
			soundManager.play_sfx(SFX_GOT_ITEM);
			break;
		case OBJ_WEAPON_PILL_BIG:
			obj_info._object->set_finished(true);
			recharge(ENERGY_TYPE_WEAPON, ENERGY_ITEM_BIG);
			break;
		case OBJ_WEAPON_PILL_SMALL:
			obj_info._object->set_finished(true);
			recharge(ENERGY_TYPE_WEAPON, ENERGY_ITEM_SMALL);
			break;
		default:
			//std::cout << "classPlayer::get_item - unknown item type: " << obj_info._object->get_type() << std::endl;
			break;
		}
	}
	return res;
}

void classPlayer::recharge(e_energy_types _en_type, int value)
{
	if (_en_type == ENERGY_TYPE_HP) {
		character::recharge(_en_type, value);
	} else if (_en_type == ENERGY_TYPE_WEAPON) {
        if (game_save.items.weapons[selected_weapon] < PLAYER_INITIAL_HP) {
            if (game_save.items.weapons[selected_weapon] + value <= PLAYER_INITIAL_HP) {
                game_save.items.weapons[selected_weapon] += value;
			} else {
                game_save.items.weapons[selected_weapon] = PLAYER_INITIAL_HP;
			}
			soundManager.play_sfx(SFX_GOT_ENERGY);
			if (value > ENERGY_ITEM_SMALL) {
				soundManager.play_sfx(SFX_GOT_ENERGY);
			}
		}
	}
}

void classPlayer::attack()
{
	st_position proj_pos;
	short int weapon_id = 0;

	if (selected_weapon == WEAPON_DEFAULT) {
		character::attack();
		return;
    } else if (game_save.items.weapons[selected_weapon] <= 0) {
		std::cout << "invalid weapon" << std::endl;
		return;
	}

	if (moveCommands.attack == 0 && attack_button_released == false) {
		attack_button_released = true;
		return;
	}

	if (max_projectiles <= get_projectile_count()) {
		//std::cout << "attack - projectile limit reached - max_projectiles: " << max_projectiles << ", projectile_list.size(): " << projectile_list.size() << ", get_projectile_count(): " << get_projectile_count() << std::endl;
		return;
	}

	if (moveCommands.attack != 0 && (timer.getTimer()-state.attack_timer) > 100 && attack_button_released == true) {
		attack_button_released = false;
		if (state.direction == ANIM_DIRECTION_LEFT) {
            proj_pos = st_position(position.x+TILESIZE/2, position.y+frameSize.height/2);
		} else {
            proj_pos = st_position(position.x+frameSize.width-TILESIZE*2, position.y+frameSize.height/2);
		}
		switch (selected_weapon) {
		/// @TODO: must code from game_data, this should not be needed
			case WEAPON_APEBOT:
				weapon_id = 1;
				break;
			case WEAPON_DAISIEBOT:
				weapon_id = 2;
				break;
			case WEAPON_DYNAMITEBOT:
				weapon_id = 3;
				break;
			case WEAPON_MUMMYBOT:
				weapon_id = 4;
				proj_pos.y -= 6;
				break;
			case WEAPON_SPIKEBOT:
				weapon_id = 5;
				break;
			case WEAPON_TECHNOBOT:
				weapon_id = 6;
				break;
			case WEAPON_MAGEBOT:
				weapon_id = 7;
				break;
			case WEAPON_SEAHORSEBOT:
				weapon_id = 8;
				break;
			// items
			case WEAPON_ITEM_COIL:
				add_coil_object();
				break;
			case WEAPON_ITEM_JET:
				add_jet_object();
				break;
			default:
				break;
		}
		if (weapon_id == 0) { /// @TODO - this is a temporary exit to handle incomplete weapons
			return;
		}

		projectile *temp_proj = new projectile(game_data.weapons[weapon_id].id_projectile, state.direction, proj_pos, map);
		projectile_list.push_back(temp_proj);
        temp_proj->set_weapon_id(weapon_id);


		if (game_data.projectiles[game_data.weapons[weapon_id].id_projectile].trajectory == TRAJECTORY_CENTERED) {
			temp_proj->set_owner_direction(&state.direction);
			temp_proj->set_owner_position(&position);
		} else if (game_data.projectiles[game_data.weapons[weapon_id].id_projectile].trajectory == TRAJECTORY_FOLLOW) {
			classnpc* temp = find_nearest_npc();
			if (temp != NULL) {
				temp_proj->set_target_position(temp->get_position_ref());
			}
		}

		//std::cout << "Added projectile - id: " << game_data.weapons[weapon_id].id_projectile << std::endl;

		if (selected_weapon != WEAPON_DEFAULT) {
            game_save.items.weapons[selected_weapon]--;
		}


		attack_state = ATTACK_START;
		state.attack_timer = timer.getTimer();
		if (state.animation_type == ANIM_TYPE_STAND) {
			state.animation_type = ANIM_TYPE_ATTACK;
		} else if (state.animation_type == ANIM_TYPE_JUMP) {
			state.animation_type = ANIM_TYPE_JUMP_ATTACK;
		} else if (state.animation_type == ANIM_TYPE_STAIRS || state.animation_type == ANIM_TYPE_STAIRS_SEMI || state.animation_type == ANIM_TYPE_STAIRS_MOVE) {
			state.animation_type = ANIM_TYPE_STAIRS_ATTACK;
		} else if (state.animation_type == ANIM_TYPE_WALK) {
			std::cout << "+++++++++++ PLAYER - set animation to ANIM_TYPE_WALK_ATTACK" << std::endl;
			state.animation_type = ANIM_TYPE_WALK_ATTACK;
		}
	}
}

void classPlayer::damage_ground_npcs()
{
	/// @TODO - this part must be done only ONCE
	// find quake in projectiles list
	int projectile_n = -1;
	for (int i =0; i<MAX_FILE_PROJECTILES; i++) {
		if (game_data.projectiles[i].trajectory == TRAJECTORY_QUAKE) {
			projectile_n = i;
			break;
		}
	}
	if (projectile_n == -1) {
		std::cout << "damage_ground_npcs - could not find projectile of QUAKE trajectory type" << std::endl;
		return;
	}
	// find quake weapon in weapons list
	int weapon_n = -1;
	for (int i = 0; i<MAX_WEAPON_N; i++) {
		if (game_data.weapons[i].id_projectile == projectile_n) {
			weapon_n = i;
			break;
		}
	}
	// could not find the weapon
	if (weapon_n == -1) {
		std::cout << "damage_ground_npcs - could not find weapon with projectile of id '" << weapon_n << "'" << std::endl;
		return;
	}
	std::list<classnpc*>::iterator enemy_it;
	for (enemy_it=map->_npc_list.begin(); enemy_it != map->_npc_list.end(); enemy_it++) {
        if ((*enemy_it)->is_on_visible_screen() == false) {
			continue;
		}

		// check if NPC is vulnerable to quake (all bosses except the one with weakness are not)
		short damage = game_data.game_npcs[(*enemy_it)->get_number()].weakness[weapon_n].damage_multiplier;

		// check if NPC is on ground
		st_position npc_pos = (*enemy_it)->getPosition();
		npc_pos.x = (npc_pos.x + (*enemy_it)->get_size().width/2)/TILESIZE;
        npc_pos.y = (npc_pos.y + (*enemy_it)->get_size().height)/TILESIZE;
		int lock = map->getMapPointLock(npc_pos);
		//std::cout << "damage_ground_npcs - NPC[" << (*enemy_it)->getName() << "].lock: " << lock << ", x: " << npc_pos.x << ", y: " << npc_pos.y << std::endl;
		if (lock == TERRAIN_UNBLOCKED || lock == TERRAIN_STAIR || lock == TERRAIN_WATER) {
			continue;
		} else {
			std::cout << "&&&&&&&&&&&&& damage_ground_npcs - DAMAGING NPC[" << (*enemy_it)->getName() << "]" << std::endl;
            (*enemy_it)->damage(damage, false);
		}

	}
}



// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void classPlayer::initFrames()
{
	frameSize.width = 29;
    frameSize.height = 32;

	graphicsLib_gSurface playerSpriteSurface;
	std::stringstream filename;
	filename << FILEPATH << "/data/images/sprites/p" << (_number+1) << ".png";
	graphLib.surfaceFromFile(filename.str(), &playerSpriteSurface);
	if (playerSpriteSurface.gSurface == NULL) {
		std::cout << "initFrames - Error loading player surface from file\n";
		return;
	}

	// STAND
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_STAND, 3, 0, playerSpriteSurface, 5000);
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_STAND, 4, 0, playerSpriteSurface, 150);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_STAND, 3, 1, playerSpriteSurface, 5000);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_STAND, 4, 1, playerSpriteSurface, 150);
	// WALK
	short walk_frame_duration = 80;
	if (_number == 0) {
		walk_frame_duration = 120;
	}
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_WALK, 6, 0, playerSpriteSurface, walk_frame_duration);
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_WALK, 7, 0, playerSpriteSurface, walk_frame_duration);
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_WALK, 8, 0, playerSpriteSurface, walk_frame_duration);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_WALK, 6, 1, playerSpriteSurface, walk_frame_duration);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_WALK, 7, 1, playerSpriteSurface, walk_frame_duration);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_WALK, 8, 1, playerSpriteSurface, walk_frame_duration);
	// JUMP
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_JUMP, 9, 0, playerSpriteSurface, 150);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_JUMP, 9, 1, playerSpriteSurface, 150);
	// ATTACK
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_ATTACK, 11, 0, playerSpriteSurface, 150);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_ATTACK, 11, 1, playerSpriteSurface, 150);
	// ATTACK + JUMP
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_JUMP_ATTACK, 10, 0, playerSpriteSurface, 80);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_JUMP_ATTACK, 10, 1, playerSpriteSurface, 80);
	// ATTACK + WALK
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_WALK_ATTACK, 12, 0, playerSpriteSurface, 150);
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_WALK_ATTACK, 13, 0, playerSpriteSurface, 150);
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_WALK_ATTACK, 14, 0, playerSpriteSurface, 150);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_WALK_ATTACK, 12, 1, playerSpriteSurface, 150);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_WALK_ATTACK, 13, 1, playerSpriteSurface, 150);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_WALK_ATTACK, 14, 1, playerSpriteSurface, 150);
	// HIT
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_HIT, 15, 1, playerSpriteSurface, 150);
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_HIT, 15, 0, playerSpriteSurface, 150);
	// TELEPORT
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_TELEPORT, 0, 0, playerSpriteSurface, 750);
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_TELEPORT, 1, 0, playerSpriteSurface, 300);
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_TELEPORT, 2, 0, playerSpriteSurface, 150);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_TELEPORT, 0, 1, playerSpriteSurface, 750);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_TELEPORT, 1, 1, playerSpriteSurface, 300);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_TELEPORT, 2, 1, playerSpriteSurface, 150);
	// STAIRS
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_STAIRS, 17, 0, playerSpriteSurface, 5000);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_STAIRS, 17, 1, playerSpriteSurface, 5000);
	// stairs semi
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_STAIRS_SEMI, 18, 0, playerSpriteSurface, 5000);
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_STAIRS_SEMI, 18, 1, playerSpriteSurface, 5000);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_STAIRS_SEMI, 18, 0, playerSpriteSurface, 5000);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_STAIRS_SEMI, 18, 1, playerSpriteSurface, 5000);
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_STAIRS, 17, 0, playerSpriteSurface, 5000);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_STAIRS, 17, 1, playerSpriteSurface, 5000);
	// stairs + move
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_STAIRS_MOVE, 17, 0, playerSpriteSurface, 200);
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_STAIRS_MOVE, 17, 1, playerSpriteSurface, 200);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_STAIRS_MOVE, 17, 0, playerSpriteSurface, 200);
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_STAIRS_MOVE, 17, 1, playerSpriteSurface, 200);
	// stairs + attack
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_STAIRS_ATTACK, 19, 1, playerSpriteSurface, 200);
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_STAIRS_ATTACK, 19, 0, playerSpriteSurface, 200);

    // slide
    addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_SLIDE, 20, 1, playerSpriteSurface, 1000);
    addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_SLIDE, 20, 0, playerSpriteSurface, 1000);

	// shield
	addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_SHIELD, 22, 1, playerSpriteSurface, 100);
	//addSpriteFrame(ANIM_DIRECTION_LEFT, ANIM_TYPE_SHIELD, 21, 1, playerSpriteSurface, 100);
	addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_SHIELD, 22, 0, playerSpriteSurface, 100);
	//addSpriteFrame(ANIM_DIRECTION_RIGHT, ANIM_TYPE_SHIELD, 21, 0, playerSpriteSurface, 100);

	playerSpriteSurface.freeGraphic();
}


void classPlayer::execute()
{
	move();
    charMove();
}

void classPlayer::execute_projectiles()
{
    // animate projectiles
    vector<projectile*>::iterator it;
    bool ignore_hit_timer = false;
    if (game_data.players[_number].simultaneous_shots > 1) {
        ignore_hit_timer = true;
    }

    for (it=projectile_list.begin(); it<projectile_list.end(); it++) {
        if ((*it)->is_finished == true) {
            projectile_list.erase(it);
            break;
        }
        st_size moved = (*it)->move();

        //std::cout << "projectile.move_type: " << (*it)->get_move_type() << std::endl;

        /// @TODO projectiles that are tele-guided
        if ((*it)->get_move_type() == TRAJECTORY_CHAIN) {
            (*it)->set_y(position.y+frameSize.height/2);
        } else if ((*it)->get_move_type() == TRAJECTORY_QUAKE) {
            damage_ground_npcs();
            continue;
        }
        (*it)->draw();
        if ((*it)->is_reflected == true) {
            continue;
        }
        // check colision agains enemies
        std::list<classnpc*>::iterator enemy_it;
        for (enemy_it=map->_npc_list.begin(); enemy_it != map->_npc_list.end(); enemy_it++) {
            if ((*it)->is_finished == true) {
                projectile_list.erase(it);
                break;
            }
            if ((*enemy_it)->is_on_visible_screen() == false) {
                continue;
            }
            if ((*enemy_it)->is_dead() == true) {
                continue;
            }



            //classnpc* enemy = (*enemy_it);
            if ((*it)->check_colision(st_rectangle((*enemy_it)->getPosition().x, (*enemy_it)->getPosition().y, (*enemy_it)->get_size().width, (*enemy_it)->get_size().height), st_position(moved.width, moved.height)) == true) {
                if ((*enemy_it)->is_shielded((*it)->get_direction()) == true) { // shielded NPC -> reflects shot
                    (*it)->reflect();
                    continue;
                }
                if ((*enemy_it)->is_invisible() == true) { // invisible NPC -> ignore shot
                    continue;
                }





                // check if have hit area, and if hit it
                st_rectangle enemy_hit_area = (*enemy_it)->get_hit_area();
                st_size enemy_size = (*enemy_it)->get_size();
                enemy_hit_area.x += (*enemy_it)->getPosition().x-1;
                enemy_hit_area.y += (*enemy_it)->getPosition().y;
                if (enemy_hit_area.w != enemy_size.width && enemy_hit_area.h != enemy_size.height) {
                    if ((*it)->check_colision(enemy_hit_area, st_position(moved.width, moved.height)) == false) { // hit body, but not the hit area -> reflect
                        std::cout << "Projectile reflected - enemy_hit_area.w: " << enemy_hit_area.w << ", enemy_hit_area.h: " << enemy_hit_area.h << std::endl;
                        (*it)->reflect();
                        continue;
                    }
                }

                short wpn_id = (*it)->get_weapon_id();

                std::cout << "******* wpn_id: " << wpn_id << std::endl;

                if (wpn_id != -1) {
                    std::cout << ">> player weapon damage #1" << std::endl;
                    std::cout << "******* (*enemy_it)->is_using_circle_weapon(): " << (*enemy_it)->is_using_circle_weapon() << ", (*it)->get_trajectory(): " << (*it)->get_trajectory() << ", TRAJECTORY_CHAIN: " << TRAJECTORY_CHAIN << std::endl;
                    // NPC using cicrcle weapon, is only be destroyed by CHAIN, but NPC won't take damage
                    if ((*enemy_it)->is_using_circle_weapon() == true ) {
                        if ((*it)->get_trajectory() == TRAJECTORY_CHAIN) {
                            std::cout << "PLAYER projectile hit NPC centered-weapon" << std::endl;
                            (*enemy_it)->consume_projectile();
                        }
                        (*it)->consume_projectile();
                        return;
                    }

                    if ((*it)->get_damage() > 0) {
                        int multiplier = game_data.game_npcs[(*enemy_it)->get_number()].weakness[wpn_id].damage_multiplier;
                        //std::cout << ">> player weapon(" << wpn_id << ") damage(" << (*it)->get_damage() << ") #1 - multiplier: " << multiplier << std::endl;
                        if (multiplier <= 0) {
                            multiplier = 1;
                        }
                        (*enemy_it)->damage((*it)->get_damage() * game_data.game_npcs[(*enemy_it)->get_number()].weakness[wpn_id].damage_multiplier, ignore_hit_timer);
                    }
                } else {
                    std::cout << ">> player weapon damage #2" << std::endl;
                    if ((*enemy_it)->is_using_circle_weapon() == true ) {
                        (*it)->consume_projectile();
                        return;
                    }
                    (*enemy_it)->damage((*it)->get_damage(), ignore_hit_timer);
                }
                if ((*it)->get_damage() > 0) {
                    (*it)->consume_projectile();
                    soundManager.play_sfx(SFX_NPC_HIT);
                }
            }
        }
    }
}

void classPlayer::move()
{
	if (input.p1_input[BTN_DOWN] == 1) {
		moveCommands.down = 1;
	} else {
		moveCommands.down = 0;
	}
	if (input.p1_input[BTN_UP] == 1) {
		moveCommands.up = 1;
	} else {
		moveCommands.up = 0;
	}
	if (input.p1_input[BTN_LEFT] == 1) {
		moveCommands.left = 1;
	} else {
		moveCommands.left = 0;
	}
	if (input.p1_input[BTN_RIGHT] == 1) {
		moveCommands.right = 1;
	} else {
		moveCommands.right = 0;
	}
	if (input.p1_input[BTN_JUMP] == 1) {
		moveCommands.jump = 1;
	} else {
		moveCommands.jump = 0;
	}
	if (input.p1_input[BTN_ATTACK] == 1) {
		moveCommands.attack = 1;
	} else {
		moveCommands.attack = 0;
	}
	if (input.p1_input[BTN_SHIELD] == 1) {
		moveCommands.shield = 1;
	} else {
		moveCommands.shield = 0;
	}
	if (input.p1_input[BTN_DASH] == 1) {
		moveCommands.dash = 1;
	} else {
		moveCommands.dash = 0;
	}

	if (input.p1_input[BTN_L] != 1 && l_key_released == false) {
		l_key_released = true;
	}
	int wpn_max = WEAPON_COUNT;
	wpn_max--;
	if (input.p1_input[BTN_L] == 1 && l_key_released == true) {
		int selected_weapon_c = selected_weapon;
		selected_weapon_c = find_next_weapon(selected_weapon, -1);
        /*
		if (selected_weapon_c < WEAPON_DEFAULT) {
			selected_weapon_c = wpn_max-1;
		}
        */
        if (selected_weapon_c != -1) {
            set_weapon((WEAPON_ICONS_ENUM)selected_weapon_c);
        }
		l_key_released = false;
        std::cout << ">>> LBUTTON - selected_weapon: " << selected_weapon << ", selected_weapon_c: " << selected_weapon_c << ", WEAPON_COUNT: " << WEAPON_COUNT << std::endl;
	}

	if (input.p1_input[BTN_R] != 1 && r_key_released == false) {
		r_key_released = true;
	}
	if (input.p1_input[BTN_R] == 1 && r_key_released == true) {
		int selected_weapon_c = selected_weapon;
		selected_weapon_c = find_next_weapon(selected_weapon, 1);
		if (selected_weapon_c >= wpn_max) {
			selected_weapon_c = WEAPON_DEFAULT;
		}
        if (selected_weapon_c != -1) {
            set_weapon((WEAPON_ICONS_ENUM)selected_weapon_c);
        }
		r_key_released = false;
        std::cout << ">>> RBUTTON - selected_weapon: " << selected_weapon << ", selected_weapon_c: " << selected_weapon_c << ", WEAPON_COUNT: " << WEAPON_COUNT << std::endl;
	}


	// send commands to the platform in special cases
	if (_platform != NULL) {
		if (_platform->get_type() == OBJ_ITEM_FLY && timer.getTimer() > _platform->get_timer()) {
            game_save.items.weapons[selected_weapon]--;
			_platform->set_timer(timer.getTimer()+240);
		}
		//std::cout << ">>> PLAYER SEND COMMAND FOR " << _platform->get_name() << ", type: " << _platform->get_type() << std::endl;
		if (moveCommands.up == 1) {
			_platform->command_up();
		}
		if (moveCommands.down == 1) {
			_platform->command_down();
		}
	}

    if (game_data.players[_number].have_shield == true && moveCommands.up == 0 && moveCommands.down == 0 && moveCommands.left == 0 && moveCommands.right == 0 && moveCommands.jump == 0 && moveCommands.attack == 0 && moveCommands.shield == 1) {
		if (state.animation_type != ANIM_TYPE_SHIELD) {
			std::cout << "playerClass::initShield CHANGE anim_type: " << state.animation_type << " to " << ANIM_TYPE_SHIELD << std::endl;
			state.animation_type = ANIM_TYPE_SHIELD;
			state.animation_timer = 0;
			state.animation_state = 0;
		}
		return;
	} else if (state.animation_type == ANIM_TYPE_SHIELD) {
		//std::cout << "playerClass::initShield REMOVE shield" << std::endl;
		if (is_player()) std::cout << "********* reset to STAND #15 **********" << std::endl;
		state.animation_type = ANIM_TYPE_STAND;
	}
    execute_projectiles();
}


void classPlayer::set_teleporter(int n)
{
	//std::cout << "******** classPlayer::set_teleporter - current: " << teleporter_n << ", new: " << n << std::endl;
	teleporter_n = n;
}

int classPlayer::get_teleporter()
{
	return teleporter_n;
}



void classPlayer::teleport_stand()
{
	unsigned int waitTimer = timer.getTimer()+500;
	state.animation_state = ANIM_TYPE_TELEPORT;
	/*
	if (p2Obj) {
		p2Obj->sprite->anim_type = ANIM_TELEPORT;
	}
	*/
	soundManager.play_sfx(SFX_TELEPORT);
	while (waitTimer > timer.getTimer()) {
		graphLib.updateScreen();
		show();
		/*
		if (p2Obj) {
			show_sprite(p2Obj->sprite, game_screen);
		}
		*/
		//drawMap3rdLevel(game_screen);
		//updateScreen(game_screen);
		input.waitTime(20);
	}
}

void classPlayer::death()
{
    reset_charging_shot();
	map->clear_animations();
	dead = true;
	state.jump_state = NO_JUMP;
	input.clean();
	state.direction = ANIM_DIRECTION_RIGHT;
	gameControl.draw_explosion(realPosition.x, realPosition.y, false);
	//set_position(st_position(80, -TILESIZE));
}

void classPlayer::reset_hp()
{
	hitPoints.current = hitPoints.total;
}

void classPlayer::change_player_color(bool full_change)
{
	if (full_change == false) {
		graphLib.change_surface_color(color_keys[0], weapon_colors[selected_weapon].color1, &(character_graphics_list.find(name)->second)[state.direction][state.animation_type][state.animation_state].frameSurface);
		graphLib.change_surface_color(color_keys[0], weapon_colors[selected_weapon].color2, &(character_graphics_list.find(name)->second)[state.direction][state.animation_type][state.animation_state].frameSurface);
		graphLib.change_surface_color(color_keys[0], weapon_colors[selected_weapon].color3, &(character_graphics_list.find(name)->second)[state.direction][state.animation_type][state.animation_state].frameSurface);
	} else {
		for (int i=0; i<ANIM_DIRECTION_COUNT; i++) {
			for (int j=0; j<ANIM_TYPE_COUNT; j++) {
				for (int k=0; k<ANIM_FRAMES_COUNT; k++) {
					if (weapon_colors[selected_weapon].color1.r != -1) {
						graphLib.change_surface_color(color_keys[0], weapon_colors[selected_weapon].color1, &(character_graphics_list.find(name)->second)[i][j][k].frameSurface);
					}
					if (weapon_colors[selected_weapon].color2.r != -1) {
						graphLib.change_surface_color(color_keys[1], weapon_colors[selected_weapon].color2, &(character_graphics_list.find(name)->second)[i][j][k].frameSurface);
					}
					if (weapon_colors[selected_weapon].color3.r != -1) {
						graphLib.change_surface_color(color_keys[2], weapon_colors[selected_weapon].color3, &(character_graphics_list.find(name)->second)[i][j][k].frameSurface);
					}
				}
			}
		}
	}
	//static std::map<std::string, st_spriteFrame[ANIM_DIRECTION_COUNT][ANIM_TYPE_COUNT][ANIM_FRAMES_COUNT]> character_graphics_list;

}

void classPlayer::set_weapon(enum WEAPON_ICONS_ENUM weapon_n)
{
	selected_weapon = weapon_n;
	change_player_color(true);
}

short classPlayer::get_weapon_value(int weapon_n)
{
	if (weapon_n == 0) {
		return hitPoints.current;
	}
    return game_save.items.weapons[weapon_n];
}

void classPlayer::set_weapon_value(unsigned short weapon_n, unsigned short int value)
{
    game_save.items.weapons[weapon_n] = value;
}



format_v2_0::file_weapon_colors classPlayer::get_weapon_colors(short int weapon_n)
{
	return weapon_colors[weapon_n];
}

WEAPON_ICONS_ENUM classPlayer::get_selected_weapon()
{
	return selected_weapon;
}

void classPlayer::refill_weapons()
{
	for (int i=0; i<WEAPON_COUNT; i++) {
        game_save.items.weapons[i] = PLAYER_INITIAL_HP;
	}
}

void classPlayer::set_teleport_minimal_y(int y)
{
    _teleport_minimal_y = y;
}

bool classPlayer::can_fly()
{
    return false;
}


void classPlayer::add_coil_object()
{
    if (game_save.items.weapons[selected_weapon] > 0) {
		std::cout << ">>>>>>> adding coil object" << std::endl;
		st_position obj_pos;
		obj_pos.y = position.y/TILESIZE;
		if (state.direction == ANIM_DIRECTION_LEFT) {
			obj_pos.x = (position.x - TILESIZE)/TILESIZE;
		} else {
			obj_pos.x = (position.x + frameSize.width + TILESIZE)/TILESIZE;
		}
		std::cout << ">>>> obj_pos.x: " << obj_pos.x << ", obj_pos.y: " << obj_pos.y << std::endl;
        object temp_obj(16, this->map, obj_pos); /// @TODO - remove hardcoded number
		temp_obj.set_duration(2500);
        temp_obj.set_colision_mode(COLISION_MODE_Y);
		map->add_object(temp_obj);
        game_save.items.weapons[selected_weapon]--;
	}
}

void classPlayer::add_jet_object()
{
    if (game_save.items.weapons[selected_weapon] > 0) {
		std::cout << ">>>>>>> adding JET object" << std::endl;
		st_position obj_pos;
		obj_pos.y = position.y/TILESIZE;
		if (state.direction == ANIM_DIRECTION_LEFT) {
			obj_pos.x = (position.x - TILESIZE*2)/TILESIZE;
		} else {
			obj_pos.x = (position.x + frameSize.width + TILESIZE)/TILESIZE;
		}
		std::cout << ">>>> obj_pos.x: " << obj_pos.x << ", obj_pos.y: " << obj_pos.y << std::endl;
		object temp_obj(17, this->map, obj_pos);
		temp_obj.set_duration(4500);
		temp_obj.set_direction(state.direction);
		map->add_object(temp_obj);
	}
}

classnpc *classPlayer::find_nearest_npc()
{
	int lower_dist = 999;
	classnpc* ret = NULL;
	std::list<classnpc*>::iterator enemy_it;

	for (enemy_it=map->_npc_list.begin(); enemy_it != map->_npc_list.end(); enemy_it++) {
		if ((*enemy_it)->is_on_screen() == false) {
			continue;
		}

		st_position npc_pos = (*enemy_it)->getPosition();
		npc_pos.x = (npc_pos.x + (*enemy_it)->get_size().width/2)/TILESIZE;
        npc_pos.y = (npc_pos.y + (*enemy_it)->get_size().height)/TILESIZE;
		int dist = sqrt(pow((float)(position.x - npc_pos.x), (float)2) + pow((float)(position.y - npc_pos.y ), (float)2));
		if (dist < lower_dist) {
			lower_dist = dist;
			ret = (*enemy_it);
		}
	}
	return ret;
}

int classPlayer::find_next_weapon(int current, int move)
{
    if (move == 1) { // move forward
        for (int i=current+1; i<9; i++) { // move until reaching last possible weapon
            int stage_n = i;
            if (i >= 5) { // jump over position 5
                stage_n = i+1;
            }
            if (game_save.stages[stage_n] == 1) {
				return i;
			}
		}
        // extra for coil and jet
        if (current < 9 && game_save.stages[COIL_GOT_STAGE] == 1) {
            return 9;
        }
        if (current < 10 && game_save.stages[JET_GOT_STAGE] == 1) {
            return 10;
        }
        for (int i=0; i<current; i++) { // now move from beggining to selected weapon
            int stage_n = i;
            if (i >= 5) { // jump over position 5
                stage_n = i+1;
            }
            if (game_save.stages[stage_n] == 1) {
				return i;
			}
		}
        if (current == 8) { // if is on position before coil
            if (game_save.stages[COIL_GOT_STAGE] == 1) {
                return 9;
            }
        }
        if (current == 9) { // if is on position before jet (coil)
            std::cout << "ON COILD POS #2 - have-jet: " << game_save.stages[JET_GOT_STAGE] << std::endl;
            if (game_save.stages[JET_GOT_STAGE] == 1) {
                return 10;
            }
        }
    } else {
		for (int i=current-1; i>=0; i--) {
            int stage_n = i;

            // extra for coil and jet
            if (current == 10 && game_save.stages[COIL_GOT_STAGE] == 1) {
                return 9;
            }


            if (i >= 5) { // jump over position 5
                stage_n = i+1;
            }
            if (game_save.stages[stage_n] == 1) {
                return i;
            }



        }

        if (current == 10) { // if is on position before coil
            if (game_save.stages[COIL_GOT_STAGE] == 1) {
                return 9;
            }
        }
        if (current == 0) { // if is on position before jet (coil)
            std::cout << "ON COILD POS #2 - have-jet: " << game_save.stages[JET_GOT_STAGE] << std::endl;
            if (game_save.stages[JET_GOT_STAGE] == 1) {
                return 10;
            }
        }

		for (int i=8; i>current; i--) {
            int stage_n = i;
            if (i >= 5) { // jump over position 5
                stage_n = i+1;
            }
            if (game_save.stages[stage_n] == 1) {
                return i;
            }
        }
	}
    return -1; // did not find any weapons besides the current one
}


void classPlayer::reset_charging_shot()
{
    if (selected_weapon != WEAPON_DEFAULT) { // only do this, if using normal weapon
        return;
    }
    state.attack_timer = 0;
    attack_button_released = true;
    soundManager.stop_repeated_sfx();
    if (color_keys[0].r != -1) {
        change_char_color(color_keys[0], color_keys[0], true);
    }
    if (color_keys[1].r != -1) {
        change_char_color(color_keys[1], color_keys[1], true);
    }
}

