#include <cstring>
#include <iostream>
#include <fstream>
#include <sstream>
#include <cmath>
#include <cstdlib>


using namespace std;

#include "classmap.h"
#include "objects/object.h"
#include "character/character.h"
#include "character/classboss.h"
#include "colision_detection.h"
#include "file/file_io.h"

extern string FILEPATH;
extern graphicsLib graphLib;

#include "game.h"
extern game gameControl;

#include "inputlib.h"
extern inputLib input;

extern CURRENT_FILE_FORMAT::file_game game_data;
extern format_v_2_0_3::file_stage stage_data;
extern bool freeze_weapon_effect;

extern format_v_2_0_1::st_save game_save;


// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
classMap::classMap() : stage_number(-1), number(-1), bg1_scroll(0), bg2_scroll(0), _platform_leave_counter(0)
{

    //std::cout << "classMap::classMap ******** backgrounds[0].filename: " << stage_data.stages[0].maps[0].backgrounds[0].filename << std::endl;
    //std::cout << "classMap::classMap ******** backgrounds[1].filename: " << stage_data.stages[0].maps[0].backgrounds[1].filename << std::endl;

	int i;
	graphLib.initSurface(st_size(MAP_W*TILESIZE, MAP_H*TILESIZE), &mapSurface);

    if (!mapSurface.gSurface) {
        graphLib.show_debug_msg("EXIT #21.MALLOC");
        exit(0);
    }


	for (i=0; i<MAP_W; i++) {
		for (int j=0; j<MAP_H; j++) {
			map_tiles.tiles[i][j].tile1.x = -1;
			map_tiles.tiles[i][j].tile1.y = -1;
		}
	}
	for (i=0; i<MAP_W; i++) {
		wall_scroll_lock[i] = false;
	}
}


// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
classMap::~classMap()
{
    mapSurface.freeGraphic();
	clean_map();
	bg1_surface.freeGraphic();
	bg2_surface.freeGraphic();
}

void classMap::clean_map()
{
	// remove npc list
    // TODO: find a fix for this
	while (!_npc_list.empty()) {
		delete _npc_list.front();
		_npc_list.pop_front();
    }
	_player_list.erase(_player_list.begin(), _player_list.end());
	object_list.erase(object_list.begin(), object_list.end());
    animation_list.erase(animation_list.begin(), animation_list.end());
	bg1_surface.freeGraphic();
    bg2_surface.freeGraphic();
}

void classMap::reset_map()
{
    // reset objects
    for (std::list<object>::iterator it=object_list.begin(); it!=object_list.end(); it++) {
        object& temp_obj = (*it);
        temp_obj.reset();
    }
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void classMap::setStageNumber(int setStageN) {
	//std::cout << "classMap::setStageNumber - setStageN: " << setStageN << std::endl;
	stage_number = setStageN;
}

short classMap::get_stage_number()
{
	return stage_number;
}

short classMap::get_number()
{
	return number;
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void classMap::setMapNumber(int setMapN) {
	number = setMapN;
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void classMap::loadMap()
{
	if (stage_number == -1) {
		cout << "ERROR::map::loadMap - stage number was not set, can't load it before setting the number.\n";
		return;
	}
	if (number == -1) {
		cout << "ERROR::map::loadMap - map number was not set, can't load it before setting the number.\n";
		return;
	}

	clean_map();

	//std::cout << "loading map[" << number << "] from stage[" << stage_number << "]" << std::endl;
	for (int i=0; i<MAP_W; i++) {
		for (int j=0; j<MAP_H; j++) {
            map_tiles.tiles[i][j].locked = stage_data.maps[number].tiles[i][j].locked;
            map_tiles.tiles[i][j].tile1.x = stage_data.maps[number].tiles[i][j].tile1.x;
            map_tiles.tiles[i][j].tile1.y = stage_data.maps[number].tiles[i][j].tile1.y;
            map_tiles.tiles[i][j].tile2.x = stage_data.maps[number].tiles[i][j].tile2.x;
            map_tiles.tiles[i][j].tile2.y = stage_data.maps[number].tiles[i][j].tile2.y;
            map_tiles.tiles[i][j].tile3.x = stage_data.maps[number].tiles[i][j].tile3.x;
            map_tiles.tiles[i][j].tile3.y = stage_data.maps[number].tiles[i][j].tile3.y;
		}
	}

	bool column_locked = true;
	for (int i=0; i<MAP_W; i++) {
		column_locked = true;
		for (int j=0; j<MAP_H; j++) {
			if (map_tiles.tiles[i][j].locked != 1 && map_tiles.tiles[i][j].locked != TERRAIN_DOOR && map_tiles.tiles[i][j].locked != TERRAIN_SCROLL_LOCK) {
				column_locked = false;
				break;
			}
		}
		wall_scroll_lock[i] = column_locked;
	}

	load_map_npcs();

	load_map_objects();

    create_dynamic_background_surfaces();

#ifdef HANDLELD // portable consoles aren't strong enought for two dynamic backgrounds
stage_data.maps[number].backgrounds[0].speed = 0;
#endif


	drawMap();
}


// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void classMap::drawMap()
{

    if (bg1_surface.width > 0 && stage_data.maps[number].backgrounds[0].speed == 0.0) {
		//std::cout << "classMap::drawMap - stage_number: " << stage_number << ", number: " << number << " has bg1 static" << std::endl;
		for (int i=0; i<mapSurface.width/bg1_surface.width+1; i++) {
            graphLib.copyArea(st_position(i*bg1_surface.width, stage_data.maps[number].backgrounds[0].adjust_y), &bg1_surface, &mapSurface);
		}
		bg1_surface.freeGraphic();
	}
    if (bg2_surface.width > 0 && stage_data.maps[number].backgrounds[1].speed == 0.0) {
		//std::cout << "classMap::drawMap - stage_number: " << stage_number << ", number: " << number << " has bg2 static" << std::endl;
		for (int i=0; i<mapSurface.width/bg2_surface.width+1; i++) {
            graphLib.copyArea(st_position(i*bg2_surface.width, stage_data.maps[number].backgrounds[1].adjust_y), &bg2_surface, &mapSurface);
		}
		bg2_surface.freeGraphic();
	}
	struct st_position pos_origin;
	struct st_position pos_destiny;
	for (int i=0; i<MAP_W; i++) {
		for (int j=0; j<MAP_H; j++) {
			// level 1
			pos_origin.x = map_tiles.tiles[i][j].tile1.x;
			pos_origin.y = map_tiles.tiles[i][j].tile1.y;
			pos_destiny.x = i*TILESIZE;
			pos_destiny.y = j*TILESIZE;

            if (!mapSurface.gSurface) {
                graphLib.show_debug_msg("EXIT #21.A");
                exit(0);
            }

            graphLib.placeTile(pos_origin, pos_destiny, &mapSurface, game_save.stages[gameControl.currentStage]);
			// level 2
			pos_origin.x = map_tiles.tiles[i][j].tile2.x;
			pos_origin.y = map_tiles.tiles[i][j].tile2.y;
            graphLib.placeTile(pos_origin, pos_destiny, &mapSurface, game_save.stages[gameControl.currentStage]);
		}
	}
    //std::cout << "graphLib.gameScreen.width: " << graphLib.gameScreen.width << std::endl;
    //graphLib.clear_area(0, 0, 600, graphLib.gameScreen.height, 0, 0, 0);
}


// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void classMap::showMap()
{
    std::list<animation>::iterator animation_it;
	struct st_rectangle origin_rectangle;

	origin_rectangle.x = scroll.x;
	origin_rectangle.y = scroll.y;
    origin_rectangle.w = RES_W;
    origin_rectangle.h = RES_H;

    /// @TODO - remove and make the bgcolor to be placed inside dynamic background  #1
    graphLib.clear_surface_area(0, 0, RES_W, RES_H, stage_data.maps[number].background_color.r, stage_data.maps[number].background_color.g, stage_data.maps[number].background_color.b, graphLib.gameScreen);
    draw_dynamic_backgrounds();
    graphLib.showMapSurfaceRegion(&mapSurface, origin_rectangle);


	/// @TODO: remove "finished" animations
    for (animation_it = animation_list.begin(); animation_it != animation_list.end(); animation_it++) {
        if ((*animation_it).finished() == true) {
            animation_list.erase(animation_it);
            break;
        } else {
            (*animation_it).execute(); // TODO: must pass scroll map to npcs somwhow...
        }
    }
}

// ********************************************************************************************** //
// show the third level of tiles                                                                  //
// ********************************************************************************************** //
void classMap::showAbove(int scroll_y)
{
	// only show pieces that in current screen position
	short start_point = scroll.x/TILESIZE;
	if (start_point > 0) { start_point--; }
	short end_point = (scroll.x+RES_W)/TILESIZE;
	if (end_point < MAP_W-1) { end_point++; }
	//std::cout << "showAbove - start_point: " << start_point << ", end_point: " << end_point << std::endl;

	// draw 3rd tile level
	struct st_position pos_origin;
	struct st_position pos_destiny;
	for (int i=start_point; i<end_point; i++) {
		for (int j=0; j<MAP_H; j++) {
			if (map_tiles.tiles[i][j].tile3.x != -1 && map_tiles.tiles[i][j].tile3.y != -1) {
				pos_origin.x = map_tiles.tiles[i][j].tile3.x;
				pos_origin.y = map_tiles.tiles[i][j].tile3.y;
				pos_destiny.x = (i*TILESIZE)-scroll.x;
				pos_destiny.y = (j*TILESIZE)-scroll.y;
				pos_destiny.y += scroll_y;

                if (!graphLib.gameScreen.gSurface) {
                    graphLib.show_debug_msg("EXIT #21.B");
                    exit(0);
                }


                graphLib.placeTile(pos_origin, pos_destiny, &graphLib.gameScreen, game_save.stages[gameControl.currentStage]);
			}
		}
	}
}

bool classMap::is_point_solid(st_position pos)
{
	short int lock_p = getMapPointLock(pos);
	if (lock_p != TERRAIN_UNBLOCKED && lock_p != TERRAIN_WATER && lock_p != TERRAIN_CHECKPOINT && lock_p != TERRAIN_SCROLL_LOCK) {
		return true;
	}
	return false;
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
int classMap::getMapPointLock(st_position pos)
{
	if (pos.x < 0 || pos.y < 0 || pos.y > RES_H/TILESIZE || pos.x > MAP_W) {
		return TERRAIN_UNBLOCKED;
	}
	return map_tiles.tiles[pos.x][pos.y].locked;
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void classMap::changeScrolling(st_position pos, bool check_lock)
{
    //std::cout << "classMap::changeScrolling - x-inc: " << pos.x << ", scroll.x/TILESIZE: " << (scroll.x/TILESIZE) << ", RES_W/TILESIZE: " << (RES_W/TILESIZE) << ", MAP_W: " << MAP_W << std::endl;
	if (pos.x != 0) {
		//scrolled = pos;
	}
    float bg1_speed = (float)stage_data.maps[number].backgrounds[0].speed/10;
    float bg2_speed = (float)stage_data.maps[number].backgrounds[1].speed/10;

    //std::cout << fixed << "bg1_speed: " << bg1_speed << std::endl;
    //std::cout << fixed << "bg2_speed: " << bg2_speed << std::endl;

	// moving player to right, screen to left
	if (pos.x > 0 && ((scroll.x/TILESIZE+RES_W/TILESIZE)-1 < MAP_W)) {
		//std::cout << "classMap::changeScrolling - MUST SCROLL #1" << std::endl;
		int x_chance = pos.x;
		if (pos.x >= TILESIZE) { // if change is too big, do not update (TODO: must check all wall until lock)
            x_chance = 1;
		}
		int tile_x = (scroll.x+RES_W-TILESIZE+2)/TILESIZE;
		if (check_lock == false || wall_scroll_lock[tile_x] == false) {
            //std::cout << "#1 classMap::changeScrolling - 1 - tile-tested: " << ((scroll.x/TILESIZE+RES_W/TILESIZE)-1) << std::endl;
			scroll.x += x_chance;
			bg1_scroll -= ((float)x_chance*bg1_speed);
            //std::cout << "*********** bg1_scroll: " << bg1_scroll << std::endl;
            if (bg1_scroll < -RES_W) {
                bg1_scroll = 0;
			}
			bg2_scroll -= ((float)x_chance*bg2_speed);
            //std::cout << "Change RIGHT-BG1POS to:" << bg1_scroll << ", BG2POS: " << bg2_scroll << ", bg2_surface.width: " << bg2_surface.width << std::endl;
            if (bg2_scroll < -RES_W) {
                //std::cout << "RIGHT RESET BG2POS" << bg2_scroll << std::endl;
				bg2_scroll = 0;
			}
		}
	} else if (pos.x < 0) {
		int x_chance = pos.x;
		if (pos.x < -TILESIZE) {
            x_chance = -1;
		}
		if (scroll.x/TILESIZE >= 0) { // if change is too big, do not update (TODO: must check all wall until lock)
			int tile_x = (scroll.x+TILESIZE-2)/TILESIZE;
            //std::cout << "#2 LEFT changeScrolling - scroll.x: " << scroll.x << ", testing tile_x: " << tile_x << std::endl;
			if (check_lock == false || wall_scroll_lock[tile_x] == false) {
				//std::cout << "classMap::changeScrolling - 2" << std::endl;
				scroll.x += x_chance;
				bg1_scroll -= ((float)x_chance*bg1_speed);
                if (bg1_scroll > 0) {
                    bg1_scroll = -RES_W; // erro aqui
				}
				bg2_scroll -= ((float)x_chance*bg2_speed);
                //std::cout << "Change LEFT-BG1POS to:" << bg1_scroll << ", BG2POS: " << bg2_scroll << std::endl;
                if (bg2_scroll > 0) {
					//std::cout << "LEFT RESET BG2POS" << bg2_scroll << std::endl;
                    bg2_scroll = -RES_W;
				}
			}
		}
	}

	scroll.y += pos.y;
}


// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void classMap::set_scrolling(st_position pos)
{
    std::cout << "------- classMap::set_scrolling - map: " << number << ", pos.x: " << pos.x << "-------" << std::endl;
	scrolled = pos;
	scroll.x = pos.x;
	scroll.y = pos.y;
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
st_position classMap::getMapScrolling()
{
    return scroll;
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void classMap::load_map_npcs()
{
    // remove all elements currently in the list
	while (!_npc_list.empty()) {
        if (_npc_list.front() != NULL) {
            delete _npc_list.front();
        }
		_npc_list.pop_front();
    }
    classnpc* new_npc;

	//std::cout << "classmap::load_map_npcs - stage: " << stage_number << ", map: " << number << std::endl;

	for (int i=0; i<MAX_MAP_NPC_N; i++) {
        if (stage_data.maps[number].map_npcs[i].id_npc != -1) {
            if (stage_data.boss.id_npc == stage_data.maps[number].map_npcs[i].id_npc) {
                new_npc = new classboss(stage_number, number, stage_data.maps[number].map_npcs[i].id_npc, i);
				new_npc->set_stage_boss();
            } else if (game_data.game_npcs[stage_data.maps[number].map_npcs[i].id_npc].is_boss == true) {
                new_npc = new classboss(stage_number, number, stage_data.maps[number].map_npcs[i].id_npc, i);
			} else {
                new_npc = new classnpc(stage_number, number, stage_data.maps[number].map_npcs[i].id_npc, i);
			}
			new_npc->set_map(this);
            _npc_list.push_back(new_npc); // insert new npc at the list-end
		}
	}
}


void classMap::draw_dynamic_backgrounds()
{
    if (bg1_surface.width > 0 && stage_data.maps[number].backgrounds[0].speed != 0) {
        // draw leftmost part
        graphLib.copyAreaWithAdjust(st_position(bg1_scroll, stage_data.maps[number].backgrounds[0].adjust_y), &bg1_surface, &graphLib.gameScreen);
    }


    if (bg2_surface.width > 0 && stage_data.maps[number].backgrounds[0].speed != 0) {
        // draw leftmost part
        graphLib.copyAreaWithAdjust(st_position(bg2_scroll, stage_data.maps[number].backgrounds[1].adjust_y), &bg2_surface, &graphLib.gameScreen);
    }
}


void classMap::draw_dynamic_backgrounds_into_surface(graphicsLib_gSurface &surface, int y_adjust)
{
    graphLib.clear_surface_area(0, 0, surface.width, surface.height, stage_data.maps[number].background_color.r, stage_data.maps[number].background_color.g, stage_data.maps[number].background_color.b, surface);
    if (bg1_surface.width > 0 && stage_data.maps[number].backgrounds[0].speed != 0) {
        // draw leftmost part
        graphLib.copyAreaWithAdjust(st_position(0, y_adjust), &bg1_surface, &surface);
    }


    if (bg2_surface.width > 0 && stage_data.maps[number].backgrounds[0].speed != 0) {
        // draw leftmost part
        graphLib.copyAreaWithAdjust(st_position(0, y_adjust), &bg2_surface, &surface);
        // draw remaining complete parts
    }

}

void classMap::add_object(object obj)
{
    object_list.push_back(obj);
}

int classMap::get_first_lock_on_left(int x_pos)
{
    //std::cout << "classMap::get_first_lock_on_left - x_pos: " << x_pos << std::endl;
    for (int i=x_pos; i>= 0; i--) {
        if (wall_scroll_lock[i] == true) {
            //std::cout << "classMap::get_first_lock_on_left - found lock at: " << i << std::endl;
            return i*TILESIZE;
        }
    }
    return -1;
}

void classMap::drop_item(st_position position)
{
    int rand_n = rand() % 100;
    //std::cout << ">>>>>>> classMap::drop_item - rand_n: " << rand_n << std::endl;
    if (rand_n <= 35) {
        st_position obj_pos;
        obj_pos.y = position.y/TILESIZE;
        obj_pos.x = (position.x - TILESIZE)/TILESIZE;
        std::cout << ">>>> classMap::drop_item  - obj_pos.x: " << obj_pos.x << ", obj_pos.y: " << obj_pos.y << std::endl;
        DROP_ITEMS_LIST obj_type;
        if (rand_n < 5) {
            std::cout << ">>>>>>> classMap::drop_item - DROP_ITEM_1UP" << std::endl;
            obj_type = DROP_ITEM_1UP;
        } else if (rand_n < 15) {
            std::cout << ">>>>>>> classMap::drop_item - DROP_ITEM_ENERGY_SMALL" << std::endl;
            obj_type = DROP_ITEM_ENERGY_SMALL;
        } else if (rand_n < 25) {
            std::cout << ">>>>>>> classMap::drop_item - DROP_ITEM_WEAPON_SMALL" << std::endl;
            obj_type = DROP_ITEM_WEAPON_SMALL;
        } else if (rand_n < 30) {
            std::cout << ">>>>>>> classMap::drop_item - DROP_ITEM_ENERGY_BIG" << std::endl;
            obj_type = DROP_ITEM_ENERGY_BIG;
        } else {
            std::cout << ">>>>>>> classMap::drop_item - DROP_ITEM_WEAPON_BIG" << std::endl;
            obj_type = DROP_ITEM_WEAPON_BIG;
        }
        short obj_type_n = gameControl.get_drop_item_id(obj_type);
        if (obj_type_n == -1) {
            std::cout << ">>>>>>>>> obj_type_n(" << obj_type_n << ") invalid for obj_type(" << obj_type << ")" << std::endl;
            return;
        }
        object temp_obj(obj_type_n, this, obj_pos);
        temp_obj.set_duration(4500);
        add_object(temp_obj);
    }
}



// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void classMap::load_map_objects() {
	std::map<std::string, object>::iterator it;

	// remove all elements currently in the list
	std::list<object>::iterator it_main_list;
	for (it_main_list=object_list.begin(); it_main_list != object_list.end(); it_main_list++) {
		object_list.erase(it_main_list);
	}
    std::list<animation>::iterator it_animation;
    for (it_animation=animation_list.begin(); it_animation != animation_list.end(); it_animation++) {
        animation_list.erase(it_animation);
    }

	for (int i=0; i<MAX_MAP_NPC_N; i++) {
        if (stage_data.maps[number].map_objects[i].id_object != -1) {
            //int temp_id = stage_data.maps[number].map_objects[i].id_object;
			object temp_obj(stage_data.maps[number].map_objects[i].id_object, this, stage_data.maps[number].map_objects[i].start_point);
			object_list.push_back(temp_obj);
		}
	}
}



std::string classMap::cent_number(int n) {
	std::stringstream ss;

	if (n < 10) {
		ss << "00" << n;
	} else if (n < 100) {
		ss << "0" << n;
	} else {
		ss << n;
	}
	return ss.str();
}

st_position classMap::get_last_scrolled()
{
	return scrolled;
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void classMap::reset_scrolled()
{
	scrolled.x = 0;
	scrolled.y = 0;
}

bool classMap::value_in_range(int value, int min, int max)
{
    return (value >= min) && (value <= max);
}

void classMap::create_dynamic_background_surfaces()
{
    graphicsLib_gSurface temp_surface;
    if (strlen(stage_data.maps[number].backgrounds[0].filename) > 0) {
        std::string bg1_filename(FILEPATH+"data/images/map_backgrounds/" + stage_data.maps[number].backgrounds[0].filename);
        if (game_save.stages[gameControl.currentStage] == 1) {
            std::string bg1_beaten_filename = bg1_filename.substr(0, bg1_filename.length()-4);
            bg1_beaten_filename = bg1_beaten_filename + "_beaten.png";
            file_io fio;
            std::cout << "beaten-bg1: " << bg1_beaten_filename << std::endl;
            if (fio.file_exists(bg1_beaten_filename) == true) {
                std::cout << "beaten_bg1 EXISTS" << std::endl;
                graphLib.surfaceFromFile(bg1_beaten_filename, &temp_surface);
                create_dynamic_background_surface(bg1_surface, temp_surface);
            } else {
                std::cout << "beaten_bg1 NOT found" << std::endl;
                graphLib.surfaceFromFile(bg1_filename, &temp_surface);
                create_dynamic_background_surface(bg1_surface, temp_surface);
            }
        } else {
            graphLib.surfaceFromFile(bg1_filename, &temp_surface);
            create_dynamic_background_surface(bg1_surface, temp_surface);
        }
    } else {
        bg1_surface.freeGraphic();
    }

    if (strlen(stage_data.maps[number].backgrounds[1].filename) > 0) {
        std::string bg2_filename(FILEPATH+"data/images/map_backgrounds/"+ stage_data.maps[number].backgrounds[1].filename);
        if (game_save.stages[gameControl.currentStage] == 1) {
            std::string bg2_beaten_filename = bg2_filename.substr(0, bg2_filename.length()-4);
            bg2_beaten_filename = bg2_beaten_filename + "_beaten.png";
            file_io fio;
            if (fio.file_exists(bg2_beaten_filename) == true) {
                graphLib.surfaceFromFile(bg2_beaten_filename, &temp_surface);
                create_dynamic_background_surface(bg2_surface, temp_surface);
            } else {
                graphLib.surfaceFromFile(bg2_filename, &temp_surface);
                create_dynamic_background_surface(bg2_surface, temp_surface);
            }
        } else {
            graphLib.surfaceFromFile(bg2_filename, &temp_surface);
            create_dynamic_background_surface(bg2_surface, temp_surface);
        }
    } else {
        bg2_surface.freeGraphic();
    }
}

void classMap::create_dynamic_background_surface(graphicsLib_gSurface &dest_surface, graphicsLib_gSurface &image_surface)
{
    // initialize dest_surface
    graphLib.initSurface(st_size(RES_W*2, RES_H), &dest_surface);
    graphLib.clear_surface_area(0, 0, image_surface.width, image_surface.height, stage_data.maps[number].background_color.r, stage_data.maps[number].background_color.g, stage_data.maps[number].background_color.b, dest_surface);
    int total_w = 0;
    while (total_w < RES_W*2) {
        //if (total_w + image_surface.width < RES_W*2) {
            graphLib.copyArea(st_position(total_w, 0), &image_surface, &dest_surface);
            total_w += image_surface.width;
        //}
    }
}


// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
object_colision classMap::colision_player_object(character* playerObj, const short int x_inc, const short int y_inc, short int reduce_x, short int reduce_y)
{
    //std::cout << ">>>>>>>>>>>>>>> object_colision classMap::colision_player_object <<<<<<<<<<<<<<<" << std::endl;
	//struct st_position p1, p2, p3, p4;
	//struct st_position obj1, obj2, obj3, obj4;
	//short int passX=0, passY=0;
	int blocked = 0;
	object* res_obj = NULL;
	int obj_mod_y = 2;

    //std::cout << "*** classMap::colision_player_object py: " << playerObj->getPosition().y << ", reduce_y: " << reduce_y << ", real_y: " << playerObj->getPosition().y+reduce_y << std::endl;

	/// @TODO: decrease y_inc til 1
	colision_detection rect_colision_obj;

	// only check all platforms if player it not above one
	/// @TODO - must take direction into account - if moving to left, it doesn't matter if player is X-locked on the right side (this logic must be placed into the colision class)
	if (playerObj->get_platform() == NULL) {
		for (std::list<object>::iterator it=object_list.begin(); it!=object_list.end(); it++) {
            int obj_blocked = 0;
			bool entered_platform = false;
			/// @TODO: check only platforms near player (in the screen)
			object& temp_obj = (*it);
            if (temp_obj.is_hidden() == true) {
                continue;
            }
            st_rectangle obj_rect(temp_obj.get_position().x, temp_obj.get_position().y+obj_mod_y, temp_obj.get_size().width, temp_obj.get_size().height);
            st_rectangle p_rect(playerObj->getPosition().x+x_inc+reduce_x/2-1, playerObj->getPosition().y+reduce_y, playerObj->get_size().width-reduce_x/2-1, playerObj->get_size().height-reduce_y);

            bool before_colision = rect_colision_obj.rect_overlap(obj_rect, p_rect);
            if (before_colision == true && temp_obj.get_colision_mode() != COLISION_MODE_Y) {
                std::cout << ">>>>>>>>> BLOCK 2 - classMap::colision_player_object - py: " << (playerObj->getPosition().y+reduce_y) << ", objy+h: " << (obj_rect.y+obj_rect.h) << std::endl;
                obj_blocked = BLOCK_X;
				blocked = BLOCK_X;
			}

            st_rectangle p_rect2(playerObj->getPosition().x+x_inc+reduce_x/2-1, playerObj->getPosition().y+reduce_y+y_inc, playerObj->get_size().width-reduce_x/2-1, playerObj->get_size().height-reduce_y);

            // value, range-min, range-max
            bool xObjOver = value_in_range(obj_rect.x, p_rect2.x, p_rect2.x + p_rect2.w);
            bool xPlayerOver = value_in_range(p_rect2.x, obj_rect.x, obj_rect.x + obj_rect.w);
            bool xOverlap = xObjOver == true || xPlayerOver == true;
            bool yOverlap = value_in_range(obj_rect.y, p_rect2.y, p_rect2.y + p_rect2.h) || value_in_range(p_rect2.y, obj_rect.y, obj_rect.y + obj_rect.h);


            //if (rect_colision_obj.rect_overlap(obj_rect, p_rect2) == true) {
            if (xOverlap == true && yOverlap == true) {
                //std::cout << "** obj_rect.x: " << obj_rect.x << ", obj_rect.y: " << obj_rect.y << ", p_rect2.x: " << p_rect2.x << ", p_rect2.y: " << p_rect2.y << ", p_rect2.h: " << p_rect2.h << std::endl;
                //std::cout << "** xObjOver: " << xObjOver << ", xPlayerOver: " << xPlayerOver << ", xOverlap: "  << xOverlap << ", yOverlap: " << yOverlap << std::endl;


                if (blocked == 0) {
					blocked = BLOCK_Y;
				} else {
					blocked = BLOCK_XY;
				}
                if (obj_blocked == 0) {
                    obj_blocked = BLOCK_Y;
                } else {
                    obj_blocked = BLOCK_XY;
                }

                //std::cout << "******** blocked: " << blocked << ", obj_blocked: " << obj_blocked << std::endl;
			}
            if (before_colision == false && y_inc > 0 && (obj_blocked == BLOCK_Y || obj_blocked == BLOCK_XY) && p_rect.y <= temp_obj.get_position().y) {
                //std::cout << ">>>>>>>> entered_platform!!!!!!! <<<<< classmap::colision_player_object - obj_rect.x: " << obj_rect.x << ", obj_rect.y: " << obj_rect.y << ", obj_rect.w: " << obj_rect.w << ", obj_rect.h: " << obj_rect.h << std::endl;
				entered_platform = true;
            //} else {
                //std::cout << ">>>>>>>> not falling, can't enter platform" << std::endl;
            }
			//if (blocked != 0) std::cout << ">>>>>>>> obj blocked: " << blocked << ", obj-type: " << temp_obj.get_type() << std::endl;
			if (temp_obj.get_type() == -1) {
				std::cout << ">>>>>>>> invalid object type" << std::endl;
			}


            bool obj_is_hidden = temp_obj.is_hidden();
            if (obj_is_hidden == false && (temp_obj.get_type() == OBJ_MOVING_PLATFORM_UPDOWN || temp_obj.get_type() == OBJ_MOVING_PLATFORM_LEFTRIGHT || temp_obj.get_type() == OBJ_DISAPPEARING_BLOCK)) {
				if (entered_platform == true && playerObj->get_platform() == NULL && (blocked == 2 || blocked == 3)) {
                    std::cout << "***** SET PLATFORM 1 - obj_is_hidden: " << obj_is_hidden << "*****" << std::endl;
					playerObj->set_platform(&temp_obj);
					if (temp_obj.get_type() == OBJ_FALL_PLATFORM) {
						temp_obj.set_direction(ANIM_DIRECTION_LEFT);
					}
				} else if (entered_platform == true && playerObj->get_platform() == NULL && blocked == 1) {
					std::cout << "***** SET PLATFORM 2 *****" << std::endl;
					playerObj->set_platform(&temp_obj);
				}
				if (blocked != 0) {
					return object_colision(blocked, &(*it));
				}
			} else if (temp_obj.get_type() == OBJ_ITEM_FLY) {
				//printf(">> testing fly, p4.y: %d, obj.y: %d <<\n", p4.y, obj1.y);
				if (entered_platform == true && playerObj->get_platform() == NULL && (blocked == 2 || blocked == 3) && y_inc > 0) {
					std::cout << "***** SET PLATFORM 3 *****" << std::endl;
					playerObj->set_platform(&temp_obj);
					if (temp_obj.get_distance() == 0) {
						temp_obj.start();
						temp_obj.set_distance(1);
						temp_obj.set_timer(timer.getTimer()+30);
					}
				}
				if (blocked != 0) {
                    std::cout << "******** #1 return.blocked: " << blocked << std::endl;
					return object_colision(blocked, &(*it));
				}
			} else if (temp_obj.get_type() == OBJ_ITEM_JUMP) {
				//printf(">> testing fly, p4.y: %d, obj.y: %d <<\n", p4.y, obj1.y);
				if (entered_platform == true && playerObj->get_platform() == NULL && (blocked == 2 || blocked == 3) && y_inc > 0) {
					//std::cout << "***** SET PLATFORM 4 *****" << std::endl;
					/*
					playerObj->set_platform(&temp_obj);
					if (temp_obj.get_distance() == 0) {
						temp_obj.set_distance(1);
						temp_obj.set_direction(ANIM_DIRECTION_UP);
						temp_obj.set_timer(timer.getTimer()+30);
					}
					*/
					playerObj->activate_super_jump();
					playerObj->activete_force_jump();
					temp_obj.start();
				}
				if (blocked != 0) {
                    //std::cout << "******** #2 return.blocked: " << blocked << std::endl;
					return object_colision(blocked, &(*it));
				}
            } else if (temp_obj.is_hidden() == false && temp_obj.is_started() == false && entered_platform == true && temp_obj.get_type() == OBJ_ACTIVE_DISAPPEARING_BLOCK) {
                temp_obj.start();
			} else if (entered_platform == true && playerObj->get_platform() == NULL && (blocked == 2 || blocked == 3) && temp_obj.get_type() == OBJ_FALL_PLATFORM) {
                //std::cout << "***** SET PLATFORM 5 *****" << std::endl;
				playerObj->set_platform(&temp_obj);
				if (temp_obj.get_state() == OBJ_STATE_STAND) {
					temp_obj.set_state(OBJ_STATE_MOVE);
				}
				temp_obj.set_timer(timer.getTimer()+30);
                //std::cout << "******** #3 return.blocked: " << blocked << std::endl;
				return object_colision(blocked, &(*it));
			} else if (entered_platform == true && playerObj->get_platform() == NULL && (blocked == 2 || blocked == 3) && temp_obj.get_type() == OBJ_FLY_PLATFORM) {
                //std::cout << "***** SET PLATFORM 6 *****" << std::endl;
				playerObj->set_platform(&temp_obj);
				temp_obj.set_timer(timer.getTimer()+30);
				if (temp_obj.get_state() == OBJ_STATE_STAND) {
					temp_obj.set_state(OBJ_STATE_MOVE);
				}
                //std::cout << "******** #4 return.blocked: " << blocked << std::endl;
				return object_colision(blocked, &(*it));
			} else if (blocked != 0) {
				res_obj = &(*it);
				break;
			}
		}
	} else {
		object* temp_obj = playerObj->get_platform();
        if (temp_obj->is_hidden() == true) {
            playerObj->set_platform(NULL);
        } else {
            st_rectangle obj_rect(temp_obj->get_position().x, temp_obj->get_position().y+obj_mod_y, temp_obj->get_size().width, temp_obj->get_size().height-1);
            //std::cout << ">> check-out-of-platform - diff_y_p_obj: " << diff_y_p_obj << std::endl;
            //std::cout << ">> check-out-of-platform - WILL CHECK" << std::endl;


            st_rectangle p_rect(playerObj->getPosition().x+x_inc+reduce_x/2, playerObj->getPosition().y+y_inc+reduce_y, playerObj->get_size().width-reduce_x/2, playerObj->get_size().height-reduce_y);

            //std::cout << ">>>>>> y_inc: " << y_inc << ", p_rect.x: " << p_rect.x << ", p_rect.y: " << p_rect.y << ", p_rect.w: " << p_rect.w << ", p_rect.h: " << p_rect.h << std::endl;
            //std::cout << ">> obj_rect.x: " << obj_rect.x << ", obj_rect.y: " << obj_rect.y << ", obj_rect.w: " << obj_rect.w << ", obj_rect.h: " << obj_rect.h << std::endl;

            if (rect_colision_obj.rect_overlap(obj_rect, p_rect) == true) {
                //std::cout << ">> check-out-of-platform - FALSE - x_inc: " << x_inc << ", y_inc: " << y_inc << std::endl;
                if (blocked == 0) {
                    blocked = 2;
                } else {
                    blocked = 3;
                }
            } else {
                //std::cout << ">> check-out-of-platform - TRUE - x_inc: " << x_inc << ", y_inc: " << y_inc << std::endl;
                blocked = 0;
            }
        }
	}


	//std::cout << "classMap::colision_player_object - blocked: " << blocked << std::endl;
	// got out of platform
	if (blocked == 0 && playerObj->get_platform() != NULL) {
		//  for player item, platform must only be removed only if the item was already adtivated
		if (playerObj->get_platform()->get_type() == OBJ_ITEM_FLY || playerObj->get_platform()->get_type() == OBJ_ITEM_JUMP) {
			if (playerObj->get_platform()->get_distance() > 0 && y_inc != 0) {
				printf("colision_player_object: got out of jump/fly item, blocked: %d, type: %d, direction: %d\n", blocked, playerObj->get_platform()->get_type(), playerObj->get_platform()->get_direction());
				playerObj->set_platform(NULL);
			} else {
				return object_colision(0, NULL);
			}
        } else if (playerObj->get_platform()->is_hidden() == true) {
            std::cout << ">>>>> platform is hidden, remove" << std::endl;
            playerObj->set_platform(NULL);
		} else {
			_platform_leave_counter++;
			if (_platform_leave_counter > 10) {
				if ((playerObj->get_platform()->get_type() == OBJ_FALL_PLATFORM || playerObj->get_platform()->get_type() == OBJ_FLY_PLATFORM) && playerObj->get_platform()->get_state() == OBJ_STATE_MOVE) {
					playerObj->get_platform()->set_state(OBJ_STATE_RETURN);
				}
				printf("colision_player_object: got out of platform, xinc: %d, yinc: %d, type: %d\n", x_inc, y_inc, playerObj->get_platform()->get_type());
				playerObj->set_platform(NULL);
				_platform_leave_counter = 0;
			}
		}
	} else if (blocked != 0 && playerObj->get_platform() != NULL) {
		_platform_leave_counter = 0;
	}
	return object_colision(blocked, res_obj);
}


void classMap::clean_map_npcs_projectiles()
{
    std::list<classnpc*>::iterator it;
    for (it=_npc_list.begin(); it!=_npc_list.end(); it++) {
		(*it)->clean_projectiles();
    }
}

graphicsLib_gSurface* classMap::get_map_surface()
{
	return &mapSurface;
}

bool classMap::get_map_point_wall_lock(int x)
{
	return wall_scroll_lock[x/TILESIZE];
}

void classMap::move_map(const short int move_x, const short int move_y)
{
	set_scrolling(st_position(scroll.x+move_x, scroll.y+move_y));
}


// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
short int classMap::colision_player_npcs(character* playerObj, const short int x_inc, const short int y_inc, short int reduce_x, short int reduce_y)
{
    UNUSED(x_inc);
    UNUSED(y_inc);
	struct st_rectangle p_rect, npc_rect;
    std::list<classnpc*>::iterator npc_it;


	//reduce = abs((float)16-playerObj->sprite->w)*0.5;

	// ponto 3, topo/esquerda
	if (playerObj->get_direction() == ANIM_DIRECTION_LEFT) {
		p_rect.x = playerObj->getPosition().x + reduce_x;
		p_rect.w = playerObj->get_size().width;
	} else {
		p_rect.x = playerObj->getPosition().x;
		p_rect.w = playerObj->get_size().width - reduce_x;
	}
	p_rect.y = playerObj->getPosition().y + reduce_y;
    p_rect.h = playerObj->get_size().height;

	//std::cout << "colision_player_npcs - p1.x: " << p1.x << ", p1.y: " << p1.y << std::endl;

	for (npc_it = _npc_list.begin(); npc_it != _npc_list.end(); npc_it++) {
		classnpc* temp_obj = (*npc_it);
		if ((*npc_it)->is_player_friend() == true) {
			//std::cout << "colision_player_npcs - FRIEND" << std::endl;
			continue;
		}
		if ((*npc_it)->is_dead() == true) {
			//std::cout << "colision_player_npcs - DEAD" << std::endl;
			continue;
		}
		if ((*npc_it)->is_invisible() == true) {
			//std::cout << "colision_player_npcs - INVISIBLE" << std::endl;
			continue;
		}
		if (temp_obj->get_size().width >= TILESIZE) {
			npc_rect.x = temp_obj->getPosition().x+PLAYER_NPC_COLLISION_REDUTOR;
			npc_rect.w = temp_obj->get_size().width-PLAYER_NPC_COLLISION_REDUTOR;
		}
        if (temp_obj->get_size().height >= TILESIZE) {
			npc_rect.y = temp_obj->getPosition().y+PLAYER_NPC_COLLISION_REDUTOR;
            npc_rect.h = temp_obj->get_size().height-PLAYER_NPC_COLLISION_REDUTOR;
		}

		colision_detection rect_colision_obj;
		if (rect_colision_obj.rect_overlap(npc_rect, p_rect) == true) {
			//std::cout << "colision_player_npcs[" << temp_obj->getName() << "] - COLISION!" << std::endl;
			if (npc_rect.h > p_rect.h) {
				return 2;
			} else {
				return 1;
			}
		}
    }
	return 0;
}

/// @TODO: fix animation. investigate a better way for drawing it (code is way too confusing)
void classMap::redraw_boss_door(bool is_close, int nTiles, int tileX, short player_number) {
	//is_close = false; // THIS IS A TEMPORARY FIX

	//std::cout << "classMap::redraw_boss_door - is_close: " << is_close << std::endl;

	input.waitTime(10);
	//for (k=0; k<tilePieces; k++) {
	for (int k=0; k<nTiles; k++) {
		//if (is_close == false) { std::cout << "classMap::redraw_boss_door - nTiles: " << nTiles << ", tilePieces: " << tilePieces << ", tileCount: " << tileCount << std::endl; }
		// redraw screen
		showMap();
		graphLib.updateScreen();
		int tiles_showed;
		if (is_close == false) {
			tiles_showed = k;
		} else {
			tiles_showed = 0;
		}
		for (int i=0; i<MAP_W; i++) {
			for (int j=0; j<MAP_H; j++) {
				if (map_tiles.tiles[i][j].tile3.x != -1 && map_tiles.tiles[i][j].tile3.y != -1) {
						if (i == tileX && map_tiles.tiles[i][j].locked == TERRAIN_DOOR) {
							//std::cout << "****** redraw_boss_door - k: " << k << ", tiles_showed: " << tiles_showed << ", nTiles: " << nTiles << std::endl;
							if (is_close == false) {
								if (tiles_showed < nTiles) {

                                    if (!graphLib.gameScreen.gSurface) {
                                        graphLib.show_debug_msg("EXIT #21.C");
                                        exit(0);
                                    }


                                    graphLib.placeTile(st_position(map_tiles.tiles[i][j].tile3.x, map_tiles.tiles[i][j].tile3.y), st_position((i*TILESIZE)-scroll.x, (j*TILESIZE)-scroll.y), &graphLib.gameScreen, game_save.stages[gameControl.currentStage]);
									graphLib.updateScreen();
									tiles_showed++;
								}
							} else {
								if (tiles_showed < k) {

                                    if (!graphLib.gameScreen.gSurface) {
                                        graphLib.show_debug_msg("EXIT #21.D");
                                        exit(0);
                                    }


                                    graphLib.placeTile(st_position(map_tiles.tiles[i][j].tile3.x, map_tiles.tiles[i][j].tile3.y), st_position((i*TILESIZE)-scroll.x, (j*TILESIZE)-scroll.y), &graphLib.gameScreen, game_save.stages[gameControl.currentStage]);
									graphLib.updateScreen();
									tiles_showed++;
								}
							}
						} else {

                            if (!graphLib.gameScreen.gSurface) {
                                graphLib.show_debug_msg("EXIT #21.E");
                                exit(0);
                            }


                            graphLib.placeTile(st_position(map_tiles.tiles[i][j].tile3.x, map_tiles.tiles[i][j].tile3.y), st_position((i*TILESIZE)+scroll.x, (j*TILESIZE)-scroll.y), &graphLib.gameScreen, game_save.stages[gameControl.currentStage]);
						}
				}
			}
		}
		_player_list.at(0)->show();
		graphLib.draw_hp_bar(_player_list.at(0)->get_current_hp(), player_number, WEAPON_DEFAULT);
		//show_sprite(p1Obj->sprite, game_screen);
		//draw_hp_bar(p1Obj);
		graphLib.updateScreen();
		input.waitTime(100);
	}
	input.waitTime(100);
}


void classMap::add_animation(ANIMATION_TYPES pos_type, graphicsLib_gSurface* surface, const st_position &pos, st_position adjust_pos, unsigned int frame_time, unsigned int repeat_times, int direction, st_size framesize)
{
	animation_list.push_back(animation(pos_type, surface, pos, adjust_pos, frame_time, repeat_times, direction, framesize, &scroll));
}

void classMap::clear_animations()
{
	animation_list.erase(animation_list.begin(), animation_list.end());
    // remove all temporary items
    std::list<object>::iterator it;
    for (it=object_list.begin(); it != object_list.end(); it++) {
        object& temp_obj = (*it);
        if (temp_obj.get_duration() > 0) {
            object_list.erase(it);
        }
    }
}

void classMap::set_player_list(std::vector<classPlayer*> player_list)
{
	_player_list = player_list;
}

void classMap::spawn_map_npc(short npc_id, st_position npc_pos, short int direction)
{
	classnpc* new_npc;
	new_npc = new classnpc(stage_number, number, npc_id, npc_pos, direction);
	new_npc->set_map(this);
    _npc_list.push_back(new_npc); // insert new npc at the list-end
}

void classMap::move_npcs() /// @TODO - check out of screen
{
	int i = 0;
    std::list<classnpc*>::iterator npc_it;
    //std::cout << "*************** classMap::showMap - npc_list.size: " << npc_list.size() << std::endl;
    for (npc_it = _npc_list.begin(); npc_it != _npc_list.end(); npc_it++) {
		// check if NPC is outside the visible area
		st_position npc_pos = (*npc_it)->get_real_position();
		short dead_state = (*npc_it)->get_dead_state();
		//if ((*npc_it)->getName() == "Brabuleta") std::cout << "npc[" << i << "] " << (*npc_it)->getName() << " - dead_state: " << dead_state << ", on_screen: " << (*npc_it)->is_on_screen() << std::endl;

		if ((*npc_it)->is_on_screen() != true) {
            if (dead_state == 2 && (*npc_it)->is_boss() == false) {
				(*npc_it)->revive();
			}
            continue; // no need for moving NPCs that are out of sight
        } else if (dead_state == 1) {// drop item
            drop_item((*npc_it)->getPosition());
        }

        // if is showing stage boss on a stage already finished, just teleport out, victory is yours!
        if ((*npc_it)->is_stage_boss() == true && (*npc_it)->is_on_visible_screen() == true && game_save.stages[gameControl.currentStage] == 1) {
            gameControl.got_weapon();
            return;
        }

		//std::cout << ">>>>>>>>>>>>>>>>>> classMap::showMap - executing npc[" << i << "] '" << (*npc_it)->getName() << "'" << std::endl;

        if (freeze_weapon_effect == false && dead_state == 0) {
            (*npc_it)->execute(); // TODO: must pass scroll map to npcs somwhow...
        }

		if (dead_state == 1) {
            gameControl.remove_all_projectiles();

            if ((*npc_it)->is_stage_boss() == false) {
                add_animation(ANIMATION_STATIC, &graphLib.explosion32, st_position((*npc_it)->getPosition().x, (*npc_it)->getPosition().y), st_position(-8, -8), 80, 2, (*npc_it)->get_direction(), st_size(32, 32));
                if ((*npc_it)->is_boss()) {
                    gameControl.check_player_return_teleport();
                }
            } else {
				std::cout << "classMap::showMap - killed boss" << std::endl;
				graphLib.set_screen_adjust(st_position(0, 0));
                gameControl.draw_explosion(npc_pos.x, npc_pos.y, true);
                if (stage_number == SKULLCASTLE5) { /// @TODO: check only if boss (must find a way to set multiple bosses in skull castle 1 by using the ones from 8-stages)
                    gameControl.show_ending();
                    return;
                } else {
                    gameControl.got_weapon();
                }
			}
			return;
		}
		i++;
    }
}

void classMap::show_npcs() /// @TODO - check out of screen
{
    std::list<classnpc*>::iterator npc_it;
	//std::cout << "*************** classMap::showMap - npc_list.size: " << npc_list.size() << std::endl;
    for (npc_it = _npc_list.begin(); npc_it != _npc_list.end(); npc_it++) {
        if (gameControl.must_show_boss_hp() && (*npc_it)->is_boss() && (*npc_it)->is_on_visible_screen() == true) {
            //std::cout << ">>>>>>>>>>>>>>>>>> classMap::showMap - executing npc '" << (*npc_it)->getName() << "'" << ", hp: " << (*npc_it)->get_current_hp() << std::endl;
			graphLib.draw_hp_bar((*npc_it)->get_current_hp(), -1, -1);
		}
		if ((*npc_it)->is_dead() == false) {
			(*npc_it)->show();
        }
    }
}

void classMap::move_objects()
{
	std::list<object>::iterator object_it;
	for (object_it = object_list.begin(); object_it != object_list.end(); object_it++) {
		if ((*object_it).finished() == true) {
			object_list.erase(object_it);
			break;
		} else {
			(*object_it).execute(); // TODO: must pass scroll map to npcs somwhow...
		}
	}
}

bool classMap::boss_hit_ground()
{
	std::list<classnpc*>::iterator npc_it;
	for (npc_it = _npc_list.begin(); npc_it != _npc_list.end(); npc_it++) {
        if ((*npc_it)->is_boss() == true && (*npc_it)->is_on_visible_screen() == true) {
            //std::cout << "boss_intro - move boss to ground - pos.y: " << (*npc_it)->getPosition().y << std::endl;
            if (((*npc_it)->getPosition().y > RES_H/3 || (*npc_it)->get_size().height > RES_H/2) && (*npc_it)->hit_ground()) {
				return true;
			}
			break;
		}
    }
	return false;
}

void classMap::reset_map_npcs()
{
	load_map_npcs();
}







