#include "common.h"

extern std::vector<std::string> PROJECTILE_TRAJECTORIES_NAMES;
extern std::vector<std::string> AI_ACTION_NAMES;
extern std::vector<std::string> AI_ACTION_WALK_OPTIONS;
extern std::vector<std::string> AI_ACTION_FLY_OPTIONS;
extern std::vector<std::string> AI_ACTION_JUMP_OPTIONS;
extern std::vector<std::string> AI_ACTION_AIR_WALK_OPTIONS;
extern std::vector<std::string> AI_ACTION_TELEPORT_OPTIONS;
extern std::vector<std::string> AI_ACTION_DASH_OPTIONS;
extern std::vector<std::string> AI_ACTION_GRAB_WALL_OPTIONS;

extern CURRENT_FILE_FORMAT::file_stages stage_data;
#include "mediator.h"
extern Mediator *dataExchanger;
extern char EDITOR_FILEPATH[512];

common::common()
{
}

common::~common()
{
}


void common::fill_files_combo(std::string directory, QComboBox* combo, bool show_none)
{
    combo->clear(); // delete all previous entries

    combo->addItem(QString("")); // for "empty"
    std::string str_filepath(FILEPATH+directory);
    QString filepath(str_filepath.c_str());
    QDir *dir = new QDir(filepath);
    if (!dir->exists()) {
        std::cout << ">> MainWindow::fill_files_combo ERROR: Directory '" << str_filepath << " does not exist. <<" << std::endl;
        exit(-1);
    }
    dir->setFilter(QDir::Files | QDir::NoSymLinks | QDir::NoDotAndDotDot);
    dir->setSorting(QDir::Size | QDir::Reversed);
	if (show_none == true) {
		combo->addItem(QString("None"));
	}
    QFileInfoList list = dir->entryInfoList();
    for (int i = 0; i < list.size(); ++i) {
        QFileInfo fileInfo = list.at(i);
        if (fileInfo.fileName().length() > 30) {
            std::cout << "ERROR: file '" << fileInfo.fileName().toStdString() << "' surpasses the maximum number of file-characters (" << CHAR_FILENAME_SIZE << ")" << std::endl;
        } else {
            combo->addItem(QString(fileInfo.fileName()));
        }
    }
    combo->repaint();
}

void common::fill_graphicfiles_listwidget(std::string directory, QListWidget* listWidget)
{
    listWidget->clear();

    QListWidgetItem* item;

    std::string str_filepath(FILEPATH+directory);
    QString filepath(str_filepath.c_str());
    QDir *dir = new QDir(filepath);
    if (!dir->exists()) {
		//std::cout << ">> MainWindow::fill_graphicfiles_listwidget ERROR: Directory '" << str_filepath << " does not exist. <<" << std::endl;
        exit(-1);
    }
    dir->setFilter(QDir::Files | QDir::NoSymLinks | QDir::NoDotAndDotDot);
    dir->setSorting(QDir::Size | QDir::Reversed);
    QFileInfoList list = dir->entryInfoList();

    for (int i = 0; i < list.size(); ++i) {
        QFileInfo fileInfo = list.at(i);
        if (fileInfo.fileName().length() > 30) {
            std::cout << "ERROR: file '" << fileInfo.fileName().toStdString() << "' surpasses the maximum number of file-characters (" << CHAR_FILENAME_SIZE << ")" << std::endl;
        } else {
            item = new QListWidgetItem;
            item->setText(fileInfo.fileName());
            std::string filename = FILEPATH + directory + "/" + fileInfo.fileName().toStdString();
            //std::cout << ">> MainWindow::fill_graphicfiles_listwidget DEBUG: filename: '" << filename << std::endl;
            QPixmap image(filename.c_str());
            image = image.copy(0, 0, image.width(), image.height());
            image = image.scaled(32, 32);
            item->setIcon(image);
            listWidget->addItem(item);
        }
    }
    listWidget->repaint();
}

void common::fill_graphicfiles_combobox(std::string directory, QComboBox *comboWidget)
{
    comboWidget->clear(); // delete all previous entries

    std::string str_filepath(FILEPATH+directory);
    QString filepath(str_filepath.c_str());
    QDir *dir = new QDir(filepath);
    if (!dir->exists()) {
        //std::cout << ">> MainWindow::fill_graphicfiles_listwidget ERROR: Directory '" << str_filepath << " does not exist. <<" << std::endl;
        exit(-1);
    }
    dir->setFilter(QDir::Files | QDir::NoSymLinks | QDir::NoDotAndDotDot);
    dir->setSorting(QDir::Size | QDir::Reversed);
    QFileInfoList list = dir->entryInfoList();

    for (int i = 0; i < list.size(); ++i) {
        QFileInfo fileInfo = list.at(i);
        if (fileInfo.fileName().length() > 30) {
            std::cout << "ERROR: file '" << fileInfo.fileName().toStdString() << "' surpasses the maximum number of file-characters (" << CHAR_FILENAME_SIZE << ")" << std::endl;
        } else {
            std::string filename = FILEPATH + directory + "/" + fileInfo.fileName().toStdString();
            QIcon icon(filename.c_str());
            comboWidget->addItem(icon, fileInfo.fileName());
        }
    }
    comboWidget->repaint();
}


void common::fill_iatypes_combo(QComboBox* combo)
{
    combo->clear(); // delete all previous entries

    char* IA_TYPE_NAMES[IA_TYPES_COUNT] = { "IA_STAND", "IA_WAIT", "IA_FOLLOW", "IA_ZIGZAG", "IA_SIDETOSIDE", "IA_BAT", "IA_ROOF_SHOOTER", "IA_GROUND_SHOOTER", "IA_SHOOT_AND_GO", "IA_FLY_ZIG_ZAG", "IA_BUTTERFLY", "IA_HORIZONTAL_GO_AHEAD", "IA_HORIZONTAL_TURN", "IA_FIXED_JUMPER", "IA_SIDE_SHOOTER", "IA_GHOST", "IA_FISH", "IA_DOLPHIN", "IA_VERTICAL_ZIGZAG" };

    for (int i=0; i<IA_TYPES_COUNT; i++) {
        combo->addItem(QString(IA_TYPE_NAMES[i]));
    }
    combo->update();
}

void common::fill_npc_combo(QComboBox* combo)
{
    combo->clear(); // delete all previous entries

    for (int i=0; i<GAME_MAX_OBJS; i++) {
        if (game_data.game_npcs[i].id != -1) {
            QString temp_str = QString("[") + QString::number(game_data.game_npcs[i].id) + QString("] - ") + QString(game_data.game_npcs[i].name);
            combo->addItem(temp_str);
        }
    }
}

void common::fill_object_combo(QComboBox* combo)
{
    combo->clear(); // delete all previous entries

    for (int i=0; i<GAME_MAX_OBJS; i++) {
		if (game_data.objects[i].id != -1) {
            QString temp_str = QString("[") + QString::number(game_data.objects[i].id) + QString("] - ") + QString(game_data.objects[i].name);
			combo->addItem(temp_str);
		}
	}
}

void common::fill_weapons_combo(QComboBox *combo)
{
    combo->clear(); // delete all previous entries

    for (int i=0; i<MAX_WEAPON_N; i++) {
        if (game_data.weapons[i].id != -1) {
			QString temp_str = QString("[") + QString::number(game_data.weapons[i].id) + QString("] - ") + QString(game_data.weapons[i].name);
            combo->addItem(temp_str);
        }
    }
}

void common::fill_projectiles_combo(QComboBox *combo)
{
    combo->clear(); // delete all previous entries

    combo->addItem(QString("[") + QString::number(-1) + QString("] - ") + QString("DEFAULT"));
	for (int i=0; i<MAX_FILE_PROJECTILES; i++) {
		QString temp_str = QString("[") + QString::number(i) + QString("] - ") + QString(game_data.projectiles[i].name);
        combo->addItem(temp_str);
    }
}


void common::fill_trajectories_combo(QComboBox *combo)
{
    combo->clear(); // delete all previous entries

    for (int i=0; i<PROJECTILE_TRAJECTORIES_COUNT; i++) {
		std::string temp = PROJECTILE_TRAJECTORIES_NAMES[i];
		QString temp_str = QString("[") + QString::number(i) + QString("] - ") + QString(temp.c_str());
		combo->addItem(temp_str);
	}
}

void common::fill_ai_actions_combo(QComboBox *combo)
{
    combo->clear(); // delete all previous entries

    for (int i=0; i<AI_ACTION_NAMES.size(); i++) {
		std::string temp = AI_ACTION_NAMES[i];
		QString temp_str = QString("[") + QString::number(i) + QString("] - ") + QString(temp.c_str());
		combo->addItem(temp_str);
	}
}


void common::fill_ai_options_combo(int action, QComboBox *combo)
{
    combo->clear(); // delete all previous entries

    std::vector<std::string> list;
	if (action == AI_ACTION_WALK) {
		list = AI_ACTION_WALK_OPTIONS;
	} else if (action == AI_ACTION_FLY) {
		list = AI_ACTION_FLY_OPTIONS;
	} else if (action == AI_ACTION_JUMP) {
		list = AI_ACTION_JUMP_OPTIONS;
	} else if (action == AI_ACTION_AIR_WALK) {
		list = AI_ACTION_AIR_WALK_OPTIONS;
	} else if (action == AI_ACTION_TELEPORT) {
		list = AI_ACTION_TELEPORT_OPTIONS;
	} else if (action == AI_ACTION_DASH) {
		list = AI_ACTION_DASH_OPTIONS;
	} else if (action == AI_ACTION_GRAB_WALL) {
		list = AI_ACTION_GRAB_WALL_OPTIONS;
	}

	// add options
	for (int i=0; i<list.size(); i++) {
		std::string temp = list[i];
		QString temp_str = QString("[") + QString::number(i) + QString("] - ") + QString(temp.c_str());
		combo->addItem(temp_str);
	}
}

void common::fill_ai_list(QComboBox *combo)
{
    combo->clear(); // delete all previous entries
    for (int i=0; i<MAX_AI_TYPES; i++) {
		QString temp_str = QString("[") + QString::number(i) + QString("] - ") + QString(game_data.ai_types[i].name);
		combo->addItem(temp_str);
    }
}

void common::fill_stages_combo(QComboBox *combo)
{
    combo->clear(); // delete all previous entries
    for (int i=0; i<MAX_STAGES; i++) {
        combo->addItem(QString::number(i)+QString("[")+QString(stage_data.stages[i].name)+QString("]"));
    }
}

void common::fill_players_combo(QComboBox* combo)
{
    combo->clear(); // delete all previous entries
    for (int i=0; i<MAX_FILE_PLAYERS; i++) {
        combo->addItem(QString::number(i+1)+QString(" [")+QString(game_data.players[i].name)+QString("]"));
    }
}

void common::fill_map_list_combo(QComboBox *combo)
{
    char buffer[512];

    combo->clear();
    dataExchanger->centNumberFormat(dataExchanger->currentGame);
    sprintf(buffer, "%s/data/game/game_%s/", EDITOR_FILEPATH, dataExchanger->centNumber);

    for (int i=0; i<3; i++) {
        dataExchanger->centNumberFormat(i);
        sprintf(buffer, "%s", dataExchanger->centNumber);
        combo->addItem(buffer);
    }
}

void common::fill_npc_listwidget(QListWidget *listWidget)
{
    QListWidgetItem* item;

    listWidget->clear();

    for (int i=0; i<GAME_MAX_OBJS; i++) {
        if (game_data.game_npcs[i].id != -1) {
            item = new QListWidgetItem;
            item->setText(game_data.game_npcs[i].name);
            std::string filename = FILEPATH + "/data/images/sprites/enemies/" + game_data.game_npcs[i].graphic_filename;
            QPixmap image(filename.c_str());
            image = image.copy(0, 0, game_data.game_npcs[i].frame_size.width, game_data.game_npcs[i].frame_size.height);
            image = image.scaled(32, 32);
            item->setIcon(image);
            listWidget->addItem(item);
        }
    }

    listWidget->update();
}

void common::fill_object_listWidget(QListWidget *listWidget)
{
    QListWidgetItem* item;

    listWidget->clear();

    for (int i=0; i<GAME_MAX_OBJS; i++) {
        if (game_data.objects[i].id != -1) {
            item = new QListWidgetItem;
            item->setText(game_data.objects[i].name);
            std::string filename = FILEPATH + "/data/images/sprites/objects/" + game_data.objects[i].graphic_filename;
            QPixmap image(filename.c_str());
            image = image.copy(0, 0, game_data.objects[i].size.width, game_data.objects[i].size.height);
            image = image.scaled(32, 32);
            item->setIcon(image);
            listWidget->addItem(item);
        }
    }


    listWidget->update();
}


