#include <iostream>

#ifdef PSP
#include <pspkernel.h>
#include <psppower.h>
#endif

#ifdef ANDROID
#include <jni.h>
#include <android/log.h>
#include <SDL_screenkeyboard.h>
#endif

#include <SDL/SDL.h>				//Include da SDL
#include <SDL/SDL_image.h>		//Include da biblioteca SDL_Image
#include <SDL/SDL_mixer.h>	// Include da biblioteca SDL_Mixer
#include <SDL/SDL_ttf.h>		// Include da biblioteca SDL_ttf
#include <SDL/SDL_endian.h>
#include <SDL/SDL_mixer.h>


// GLOBAL/EXTERN VARIABLES
std::string FILEPATH;
SDL_Event event;

#include "graphicslib.h"
#include "inputlib.h"
#include "timerlib.h"
#include "soundlib.h"
#include "game.h"
#include "scenes/ending.h"
#ifdef PLAYSTATION2
	#include "ports/ps2/modules.h"
#endif


#ifdef DINGUX
	std::string EXEC_NAME("rockbot.dge");
#elif defined(WIN32)
	std::string EXEC_NAME("rockbot.exe");
#elif defined(PLAYSTATION2)
	std::string EXEC_NAME("rockbot.elf");
#elif defined(PSP)
	std::string EXEC_NAME("EBOOT.PBP");
#elif defined(ANDROID)
    std::string EXEC_NAME("");
#else
std::string EXEC_NAME("rockbot");
#endif

// INTERNAL GLOBALS
timerLib timer;
inputLib input;
soundLib soundManager;
graphicsLib graphLib;
game gameControl;
format_v_2_0_1::st_save game_save;
struct format_v_2_1_1::st_game_config game_config;
bool freeze_weapon_effect = false;
bool GAME_FLAGS[FLAG_COUNT];

#include "file/file_io.h"
CURRENT_FILE_FORMAT::file_game game_data;
format_v_2_0_3::file_stage stage_data;

#include "defines.h"

struct format_v_2_0_1::st_checkpoint checkpoint;

#ifdef PSP
PSP_MODULE_INFO("Rockbot", 0, 1, 0);
PSP_HEAP_SIZE_KB(-1024);

/* Exit callback */
int exit_callback(int arg1, int arg2, void *common) {
   sceKernelExitGame();
    return 0;
}

/* Callback thread */
int CallbackThread(SceSize args, void *argp) {
    int cbid;
    cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
    sceKernelRegisterExitCallback(cbid);
    sceKernelSleepThreadCB();
    return 0;
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void) {
    int thid = 0;
    thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, 0, 0);
    if(thid >= 0) {
        sceKernelStartThread(thid, 0, 0);
    }
    return thid;
}
#endif


int main(int argc, char *argv[])
{
#ifdef PSP
    SetupCallbacks();
    scePowerSetClockFrequency(333, 333, 166);
#endif

    //std::cout << "MAIN #1" << std::endl;
	for (int i=0; i<FLAG_COUNT; i++) {
		GAME_FLAGS[i] = false;
	}
	UNUSED(argc);


	//strncpy (FILEPATH, argv[0], strlen(argv[0])-11);

	string argvString = string(argv[0]);



#ifdef PLAYSTATION2
	SifIopReset(NULL, 0); // clean previous loading of irx by apps like ulaunchElf. Comment this line to get cout on ps2link

	/* SP193: Being creative (Do something while waiting for the slow IOP to be reset). =D */
	int main_id = GetThreadId();
	ChangeThreadPriority(main_id, 64);

	while(!SifIopSync()); /* Wait for the IOP to be reset. */

	/* Initialize and connect to all SIF services on the IOP. */
	SifInitRpc(0);
	SifInitIopHeap();
	SifLoadFileInit();
	fioInit();

	/* Apply the SBV LMB patch to allow modules to be loaded from a buffer in EE RAM. */
	sbv_patch_enable_lmb();

	PS2loadUSB();
#endif


	FILEPATH = argvString.substr(0, argvString.size()-EXEC_NAME.size());


#ifdef PSP
    std::string cout_file = FILEPATH + "/stdout.txt";
    //std::ofstream out(cout_file.c_str());
    //std::streambuf *coutbuf = std::cout.rdbuf(); //save old buf
    //std::cout.rdbuf(out.rdbuf()); //redirect std::cout to out.txt!
#endif

#ifdef PLAYSTATION2 // DEBUG
	FILEPATH = "mass:/PS2/Rockbot/";
#endif
	//std::cout << "main - FILEPATH: " << FILEPATH << std::endl;


	// check command-line paramethers
	if (argc > 1) {
		for (int i=1; i<argc; i++) {
			std::string temp_argv(argv[i]);
			if (temp_argv == "--fullscreen") {

			} else if (temp_argv == "--quickload") {
				GAME_FLAGS[FLAG_QUICKLOAD] = true;
			} else if (temp_argv == "--invencible") { // player have infinite HP
				GAME_FLAGS[FLAG_INVENCIBLE] = true;
			} else if (temp_argv == "--intangible") { // player is intangible
				GAME_FLAGS[FLAG_INTANGIBLE] = true;
			} else if (temp_argv == "--allweapons") { // player have all weapons available even if
				GAME_FLAGS[FLAG_ALLWEAPONS] = true;
			} else if (temp_argv == "--infinitejump") { // player can jump again and again
				GAME_FLAGS[FLAG_INFINITE_JUMP] = true;
			}
		}
	}

    GAME_FLAGS[FLAG_INVENCIBLE] = false;
    GAME_FLAGS[FLAG_QUICKLOAD] = false;
    game_config.selected_player = 2;
    gameControl.currentStage = TECHNOBOT;


#ifdef ANDROID
    FILEPATH = "/sdcard/Android/data/net.upperland.rockbot/files/";

    std::string cout_file = FILEPATH + "/stdout.txt";
    std::ofstream out(cout_file.c_str());
    std::streambuf *coutbuf = std::cout.rdbuf(); //save old buf
    std::cout.rdbuf(out.rdbuf()); //redirect std::cout to out.txt!

    std::cout << "FILEPATH: " << FILEPATH << std::endl;

#endif



	CURRENT_FILE_FORMAT::file_io fio;
	fio.read_game(game_data);
    gameControl.get_drop_item_ids();
	soundManager.init_audio_system();




	// INIT GRAPHICS
	if (graphLib.initGraphics() != true) {
        std::cout << "ERROR intializing graphic" << std::endl;
		return -1;
	}

    graphLib.preload();

    /*
    // *TEST FOR SKULL CASTLE INTRO
    soundManager.stop_music();
    scenesLib temp_scene;
    temp_scene.boss_intro(SKULLCASTLE5);
    input.wait_keypress();
    return 0;
    */



	// INIT GAME
	if (GAME_FLAGS[FLAG_QUICKLOAD] == false) {
		if (gameControl.showIntro() == false) {
            std::cout << "ERROW SHOWING INTRO" << std::endl;
			return 0;
		}
	} else {
        std::cout << "DEBUG #A.003" << std::endl;
		gameControl.quick_load_game();
        std::cout << "DEBUG #A.004" << std::endl;
        //ending game_ending;
        //game_ending.start();
	}

    std::cout << "DEBUG #A.005" << std::endl;
	input.clean();
	input.p1_input[BTN_START] = 0;
	input.waitTime(200);
	input.clean();
    std::cout << "DEBUG #A.006" << std::endl;

	while (true) {
		#ifdef PLAYSTATION2
			RotateThreadReadyQueue(_MIXER_THREAD_PRIORITY);
		#endif
		input.readInput();
		if (input.p1_input[BTN_QUIT] == 1) {
			//exit(-1);
			format_v_2_0_1::file_io fio;
			fio.write_save(game_save);
			return 0;
		}
		gameControl.showGame();
        gameControl.fps_count();
		graphLib.updateScreen();
	}
	/// @TODO: sdl quit sub-systems
    std::cout << "DEBUG #F.001" << std::endl;


	
#ifdef PSP
    sceKernelExitGame();
    return 0;
#else
    SDL_Quit();
#endif
	
	return 1;
}


#ifdef ANDROID
extern "C" {
    JNIEXPORT void JNICALL Java_net_upperland_rockbot_DemoRenderer_nativeInit(JNIEnv * env, jobject obj);
};

JNIEXPORT void JNICALL Java_net_upperland_rockbot_DemoRenderer_nativeInit(JNIEnv * env, jobject obj)
{
    char * argv[1];
    argv[0] = "./";
    main(1, argv);
}

#endif
