#include <cstdlib>
#include "projectilelib.h"
#include "timerlib.h"
#include "classmap.h"
#include <math.h>

#include "soundlib.h"
extern soundLib soundManager;

extern timerLib timer;
extern CURRENT_FILE_FORMAT::file_game game_data;

#define QUAKE_TIMER 80
#define QUAKE_SCREEN_MOVE 5
extern bool freeze_weapon_effect;
#define FREEZE_DURATION 3500


extern bool freeze_weapon_effect;


// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
projectile::projectile(int id, int set_direction, st_position set_position, classMap *set_map) : _move_type(-1), is_reflected(false), status(0), _effect_timer(0), _effect_n(0), _points(1), _target_position(NULL), _weapon_id(-1)
{
    set_default_values();
	_id = id; // -1 is default projectile
    map = set_map;
    position = set_position;
    direction = set_direction;
	_size = get_size();

	/// @TODO _ move this to game load
	if (_id == -1) {
		_size.width = 6;
		_size.height = 6;
	}


    animation_timer = timer.getTimer() + PROJECTILE_DEFAULT_ANIMATION_TIME; // used to control each frame duration

	position.y -= _size.height/2;
	_max_frames = get_surface()->width / _size.width;
	_move_type = get_trajectory();


	if (_move_type == TRAJECTORY_ARC) {
		if (direction == ANIM_DIRECTION_LEFT) {
			position0.x = position.x - RES_W/3;
		} else {
			position0.x = position.x + RES_W/3;
		}
	} else if (_move_type == TRAJECTORY_SIN) {
		if (direction == ANIM_DIRECTION_LEFT) {
			position0.x = 0;
		}
	} else if (_move_type == TRAJECTORY_QUAKE) {
		_quake_info.timer = timer.getTimer() + QUAKE_TIMER;
	} else if (_move_type == TRAJECTORY_FREEZE) {
		_quake_info.timer = timer.getTimer() + FREEZE_DURATION;
		freeze_weapon_effect = true;
		graphLib.blink_screen(235, 235, 235);
	} else {
		position0.x = position.x;
		position0.y = position.y;
	}

	_effect_timer = timer.getTimer() + 3600;


	//std::cout << "New projectile - id: " << _id << ", _move_type: " << _move_type << std::endl;

    //std::cout << ">> Added projectyle, move_type: " << _move_type << ", w: " << _size.width << ", h: " << _size.height << std::endl;


    // for size, use getsize
    // for speed, use get_speed
    // for damage, use get_damage
    // for trajectory, get_trajectory

}

void projectile::set_default_values()
{
    _owner_position = NULL;
    is_finished = false;
    move_timer = 0;
    move_delay = 20;
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
projectile::~projectile()
{
	if (_move_type == TRAJECTORY_FREEZE) {
		std::cout << "projectile::projectile - freeze projectile finishing" << std::endl;
		freeze_weapon_effect = false;
	}
}


st_size projectile::get_size()
{
    if (_id == -1) {
		return st_size(6, 6);
	}
	return game_data.projectiles[_id].size;
}

unsigned short projectile::get_speed()
{
    if (_id == -1) {
        return PROJECTILE_DEFAULT_SPEED;
    }
	if (game_data.projectiles[_id].speed > 0) {
		return game_data.projectiles[_id].speed;
	}
	return PROJECTILE_DEFAULT_SPEED;
}

unsigned short projectile::get_damage()
{
    if (_id == -1) {
        return PROJECTILE_DEFAULT_DAMAGE;
    }
	if (get_trajectory() == TRAJECTORY_BOMB && _effect_n == 0) {
		return 0;
	}
	if (game_data.projectiles[_id].damage <= 0) {
		return PROJECTILE_DEFAULT_DAMAGE;
	}
    return game_data.projectiles[_id].damage;
}

unsigned short projectile::get_trajectory()
{
    if (_id == -1) {
        return TRAJECTORY_LINEAR;
    }
	return game_data.projectiles[_id].trajectory;
}

void projectile::set_trajectory(short new_trajectory)
{
	_move_type = new_trajectory;
}

short projectile::get_max_projectiles()
{
	if (_id == -1) {
		return 3;
	}
	if (game_data.projectiles[_id].max_shots > 0) {
		return game_data.projectiles[_id].max_shots;
	}
	return 1;
}

void projectile::set_target_position(st_position *pos)
{
	_target_position = pos;
	if (_target_position == NULL && _move_type == TRAJECTORY_FOLLOW) { // if a follow projectile could not find any target, act as zig-zag
		_move_type = TRAJECTORY_ZIGZAG;
	} else 	if (_move_type == TRAJECTORY_PLAYER_DIRECTION) { // change type to be the best one to hit player
		if (_target_position->y < position.y-TILESIZE*2) {
			std::cout << "TRAJECTORY_PLAYER_DIRECTION - UP" << std::endl;
			_move_type = TRAJECTORY_DIAGONAL_UP;
		} else if (_target_position->y > position.y+TILESIZE*2) {
			std::cout << "TRAJECTORY_PLAYER_DIRECTION - DOWN" << std::endl;
			_move_type = TRAJECTORY_DIAGONAL_DOWN;
		} else {
			std::cout << "TRAJECTORY_PLAYER_DIRECTION - LINEAR" << std::endl;
			_move_type = TRAJECTORY_LINEAR;
		}
	}

}

graphicsLib_gSurface *projectile::get_surface()
{
	if (_id == -1) {
        if (&graphLib.projectile_surface[0].gSurface == NULL) {
            graphLib.show_debug_msg("projectile surface error #1");
        }
		return &graphLib.projectile_surface[0];
	} else {
        if (&graphLib.projectile_surface[_id].gSurface == NULL) {
            graphLib.show_debug_msg("projectile surface error #2");
        }
        return &graphLib.projectile_surface[_id];
	}
}

st_size projectile::move() {
	st_size moved;

	//std::cout << "projectile::move - TRAJECTORY: " << _move_type << ", x: " << position.x << ", y: " << position.y << ", direction: " << direction << std::endl;

	if (move_timer >= timer.getTimer()) {
		//std::cout << "projectile::projectile - return #1" << std::endl;
		return st_size(0, 0);
	}
	move_timer = timer.getTimer()+move_delay;




	if (_move_type == TRAJECTORY_LINEAR || _move_type == TRAJECTORY_DIAGONAL_UP || _move_type == TRAJECTORY_DIAGONAL_DOWN) {
		if (direction == ANIM_DIRECTION_LEFT) {
			position.x -= get_speed();
			moved.width -= get_speed();
		} else {
			position.x += get_speed();
			moved.width += get_speed();
		}

		//std::cout << "projectile::move - new position - " << ", x: " << position.x << ", y: " << position.y << std::endl;

		if (diagonal_flag == false) {
			if (_move_type == TRAJECTORY_DIAGONAL_UP) {
				position.y -= get_speed();
				moved.height -= get_speed();
			} else if (_move_type == TRAJECTORY_DIAGONAL_DOWN) {
				position.y += get_speed();
				moved.height += get_speed();
			}
			diagonal_flag = true;
		} else {
			diagonal_flag = false;
		}

	} else if (_move_type == TRAJECTORY_ARC) {
		if (direction == ANIM_DIRECTION_LEFT) {
			position.x -= get_speed();
			moved.width -= get_speed();
		} else {
			position.x += get_speed();
			moved.width += get_speed();
		}

		short int tempY1 = position1.y;
		int part1 = pow(static_cast<double>(RES_W/3), static_cast<double>(2));
		int part2 = pow(static_cast<double>(position.x - position0.x), static_cast<double>(2));
		int part3 = position0.y;
		position1.y = sqrt(static_cast<double>(part1 - part2)) + part3;
		short int diffY = tempY1 - position1.y;
		if (diffY > 0 || diffY > -20) {
			position.y += diffY;
		}
		std::cout << "projectile::projectile - EXEC TRAJECTORY_ARC - x: " + position.x << ", y: " << position.y << std::endl;

	} else if (_move_type == TRAJECTORY_SIN) {
		angle += 0.5;
		if (angle > 360.0) {
			angle = 0.0;
		}
		if (direction == ANIM_DIRECTION_LEFT) {
			position.x -= get_speed();
			moved.width -= get_speed();
		} else {
			position.x += get_speed();
			moved.width += get_speed();
		}
		position.y = sin(angle)*TILESIZE*2 + position0.y;
	} else if (_move_type == TRAJECTORY_BOMB) {
		if (_effect_n == 0 && _effect_timer < timer.getTimer()) {
			std::cout << "BOMB - TRANSFORM into explosion" << std::endl;
			/// morph into a bigger explosion
			_points = 5000;
			_effect_timer = timer.getTimer()+3600;

			_size.width = 56;
			_size.height = 56;
			position.x -= 28;
			position.y -= 28;
			_max_frames = get_surface()->width / _size.width;
			_effect_n++;
			soundManager.play_repeated_sfx(SFX_BIG_EXPLOSION, 3);
		} else if (_effect_n == 1 && _effect_timer < timer.getTimer()) {
			is_finished = true;
		}
		//std::cout << "projectile::move - BOMB" << std::endl;
		// do nothing, it is a bomb, it just stays until explodes
	} else if (_move_type == TRAJECTORY_CHAIN) {
		if (status == 0) { // increasing
			moved.width += get_speed();
			if (direction == ANIM_DIRECTION_LEFT) {
				position.x -= get_speed();
			} else {
				position.x += get_speed();
			}
			if (abs((float)position0.x-position.x) > _size.width) {
				status = 1;
			}
		} else { // returning
			moved.width -= get_speed();
			if (direction == ANIM_DIRECTION_LEFT) {
				position.x += get_speed();
			} else {
				position.x -= get_speed();
			}
			if (abs((float)position0.x-position.x) <= 0) {
				is_finished = true;
			}
		}
	} else if (_move_type == TRAJECTORY_QUAKE) { // shake the screen
		//std::cout << "TRAJECTORY_QUAKE - quake.timer: " << _quake_info.timer << ", timer: " << timer.getTimer() << std::endl;
		if (_quake_info.timer < timer.getTimer()) {
			if (_quake_info.counter1 == 1) {
				_quake_info.counter1 = 2;
				graphLib.set_screen_adjust(st_position(-QUAKE_SCREEN_MOVE, 0));
			} else {
				_quake_info.counter1 = 1;
				graphLib.set_screen_adjust(st_position(QUAKE_SCREEN_MOVE, 0));
			}
			_quake_info.counter2++;
			if (_quake_info.counter2 > 20) {
				_quake_info.counter1 = 0;
				graphLib.set_screen_adjust(st_position(0, 0));
				is_finished = true;
			}
			_quake_info.timer = timer.getTimer() + QUAKE_TIMER;
		}
	} else if (_move_type == TRAJECTORY_FREEZE) {
		std::cout << "projectile::projectile - freeze RUNNING, timer: " << timer.getTimer() << ", should stop at " << _quake_info.timer << std::endl;
		if (_quake_info.timer < timer.getTimer()) {
			std::cout << "projectile::projectile - freeze STOPPED, should stop at " << _quake_info.timer << std::endl;
			freeze_weapon_effect = false;
			is_finished = true;
		}
	} else if (_move_type == TRAJECTORY_CENTERED) {
		//std::cout << "TRAJECTORY_CENTERED CHECK - timer: " << timer.getTimer() << ", _quake_info.timer: " << _quake_info.timer << std::endl;
		// after some time, change to linear or diagonal
		if (_owner_position == NULL) {
			std::cout << "ERROR: owner positoon NOT set in centered projectile" << std::endl;
		} else {
			position.x = _owner_position->x - 15;
			position.y = _owner_position->y - 7;
			if (_owner_direction != NULL) {
				direction = *_owner_direction;
			}
			if (_effect_timer < timer.getTimer()) {
				_move_type = TRAJECTORY_LINEAR;
			}
		}
	} else if (_move_type == TRAJECTORY_ZIGZAG) {
		if (direction == ANIM_DIRECTION_LEFT) {
			position.x -= get_speed();
			moved.width -= get_speed();
		} else {
			position.x += get_speed();
			moved.width += get_speed();
		}
		if (check_map_colision(st_position(moved.width, moved.height)) == true) {
			status++;
			if (status > 3) {
				is_finished = true;
			} else {
				direction = !direction;
			}
		}
	} else if (_move_type == TRAJECTORY_FOLLOW && _target_position != NULL) {
		int xinc = 0;
		int yinc = 0;
		if (_target_position->x > position.x) {
			xinc = get_speed();
		} else if (_target_position->x < position.x) {
			xinc = -get_speed();
		}
		if (_target_position->y > position.y) {
			yinc = get_speed();
		} else if (_target_position->y < position.y) {
			yinc = -get_speed();
		}
		std::cout << "TRAJECTORY_FOLLOW - xinc: " << xinc << ", yinc: " << yinc << ", x: " << position.x << ", y: " << position.y << std::endl;
		position.x += xinc;
		moved.width += xinc;
		position.y += yinc;
		moved.height += yinc;
	} else if (_move_type == TRAJECTORY_FOLLOW && _target_position == NULL) {
		_move_type = TRAJECTORY_ZIGZAG;
	} else 	if (_move_type == TRAJECTORY_PLAYER_DIRECTION && _target_position == NULL) { // if do not have a target, act as linear
		if (direction == ANIM_DIRECTION_LEFT) {
			position.x -= get_speed();
			moved.width -= get_speed();
		} else {
			position.x += get_speed();
			moved.width += get_speed();
		}
	} else {
		std::cout << "projectile::move - UNKNOWN TRAJECTORY #" << _move_type << std::endl;
	}

	realPosition.x = position.x - map->getMapScrolling().x;
	realPosition.y = position.y - map->getMapScrolling().y;
	// check out of screen
	if (_move_type != TRAJECTORY_FREEZE && _move_type != TRAJECTORY_QUAKE) { // special effect weapons can work out of screen
		if (realPosition.x > RES_W+TILESIZE*2 || realPosition.x < 0-TILESIZE*2 || realPosition.y > RES_H+TILESIZE*2 || realPosition.y < 0-+TILESIZE*2) {
			is_finished = true;
		}
	}
	return moved;
}

void projectile::draw() {
	if (_move_type == TRAJECTORY_BOMB && _effect_n == 1) {
		graphLib.draw_explosion(realPosition);
		return;
	}

	if (_move_type == TRAJECTORY_QUAKE || _move_type == TRAJECTORY_FREEZE) { /// QTODO: freeze could use some "sparkling" effect
		//std::cout << "projectile::draw - invisible type" << std::endl;
		return;
	}

	if (animation_pos >= _max_frames) {
		//std::cout << "projectile::draw - RESET animation_pos" << std::endl;
		animation_pos = 0;
	}

	int anim_pos = animation_pos*_size.width;
	//std::cout << "projectile::draw - animation_pos: " << animation_pos << ", _max_frames: " << _max_frames << ", _size.width: " << _size.width << ", anim_pos: " << anim_pos << std::endl;


	if (direction == ANIM_DIRECTION_LEFT && get_surface()->height >= _size.height*2) {
		graphLib.showSurfaceRegionAt(get_surface(), st_rectangle(anim_pos, _size.height, _size.width, _size.height), realPosition);
	} else {
		graphLib.showSurfaceRegionAt(get_surface(), st_rectangle(anim_pos, 0, _size.width, _size.height), realPosition);
	}

	//std::cout << "projectile::draw - animation_timer: " << animation_timer << ", timer: " << timer.getTimer() << std::endl;
    if (animation_timer < timer.getTimer()) {
        animation_pos++;
		//std::cout << "projectile::draw - inc anim_pos to " << animation_pos << std::endl;
		animation_timer = timer.getTimer() + PROJECTILE_DEFAULT_ANIMATION_TIME;
	}
}

// TODO: width/height must come from editor instead of using graphLib.projectile_surface
bool projectile::check_colision(st_rectangle enemy_pos, st_position pos_inc)
{
	st_position p1, p2, p3, p4;
	st_position enemy1, enemy2, enemy3, enemy4;
	// point 1, top/left
	p1.x = position.x + pos_inc.x;
	p1.y = position.y + pos_inc.y;

	// ponto 2, baixo/esquerda
	p2.x = position.x + pos_inc.x;
	p2.y = position.y + pos_inc.y + _size.height;

	// ponto 3, topo/direita
	p3.x = position.x + pos_inc.x + _size.width;
	p3.y = position.y + pos_inc.y;

	// ponto 4, baixo/direita
	p4.x = position.x + pos_inc.x + _size.width;
	p4.y = position.y + pos_inc.y + _size.height;

	enemy1.x = enemy_pos.x;
	enemy1.y = enemy_pos.y;
	enemy2.x = enemy_pos.x + enemy_pos.w;
	enemy2.y = enemy_pos.y;
	enemy3.x = enemy_pos.x;
	enemy3.y = enemy_pos.y + enemy_pos.h;
	enemy4.x = enemy_pos.x + enemy_pos.w;
	enemy4.y = enemy_pos.y + enemy_pos.h;

	//std::cout << "projectile::check_colision - p1: " << p1.x << "." << p1.y << " p2: " << p2.x << "." << p2.y << " p3: " << p3.x << "." << p3.y << " p4: " << p4.x << "." << p4.y << std::endl;
	//std::cout << "projectile::check_colision - enemy1: " << enemy1.x << "." << enemy1.y << " enemy2: " << enemy2.x << "." << enemy2.y << " enemy3: " << enemy3.x << "." << enemy3.y << " enemy4: " << enemy4.x << "." << enemy4.y << std::endl << std::endl;

	/// check if any of the PROJECTILE points is inside NPC
	if (p1.x >= enemy1.x && p1.x <= enemy2.x && p1.y >= enemy1.y && p1.y <= enemy3.y) {
		//std::cout << "projectile::check_colision - HIT 1" << std::endl;
		return true;
	} else if (p2.x >= enemy1.x && p2.x <= enemy2.x && p2.y >= enemy1.y && p2.y <= enemy3.y) {
		//std::cout << "projectile::check_colision - HIT 2" << std::endl;
		return true;
	} else if (p3.x >= enemy1.x && p3.x <= enemy2.x && p3.y >= enemy1.y && p3.y <= enemy3.y) {
		//std::cout << "projectile::check_colision - HIT 3" << std::endl;
		return true;
	} else if (p4.x >= enemy1.x && p4.x <= enemy2.x && p4.y >= enemy1.y && p4.y <= enemy3.y) {
		//std::cout << "projectile::check_colision - HIT 4" << std::endl;
		return true;
	}
	// npc inside projectile
	if (p1.x >= enemy1.x && p1.x <= enemy2.x && p1.y <= enemy1.y && p2.y >= enemy2.y) {
		//std::cout << "projectile::check_colision - HIT 5" << std::endl;
		return true;
	}
	if (p3.x >= enemy1.x && p3.x <= enemy2.x && p3.y <= enemy1.y && p4.y >= enemy2.y) {
		//std::cout << "projectile::check_colision - HIT 6" << std::endl;
		return true;
	}
	//std::cout << "projectile::check_colision - missed" << std::endl;
	return false;
}

bool projectile::check_map_colision(st_position pos_inc)
{
	int p_y[3];
	p_y[0] = position.y + pos_inc.y;
	p_y[1] = position.y + get_size().height/2 + pos_inc.y;
	p_y[2] = position.y + get_size().height + pos_inc.y;
	int p_x;
	if (direction == ANIM_DIRECTION_LEFT) {
		p_x = position.x + pos_inc.x;
	} else {
		p_x = position.x + get_size().width + pos_inc.x;
	}
	for (int i=0; i<3; i++) {
		int lock = map->map_tiles.tiles[p_x/TILESIZE][p_y[i]/TILESIZE].locked;
		if (lock != TERRAIN_UNBLOCKED && lock != TERRAIN_WATER) {
			std::cout << ">> projectile::check_map_colision - point (" << p_x << ", " << p_y[i] << ") lock: " << lock << std::endl;
			return true;
		}
	}
	return false;
}


int projectile::get_direction()
{
	return direction;
}

void projectile::reflect()
{
	if (direction == ANIM_DIRECTION_LEFT) {
		direction = ANIM_DIRECTION_RIGHT;
	} else {
		direction = ANIM_DIRECTION_LEFT;
	}
	if (position.y%2) {
		_move_type = TRAJECTORY_DIAGONAL_UP;
	} else {
		_move_type = TRAJECTORY_DIAGONAL_DOWN;
	}
	is_reflected = true;
	soundManager.play_sfx(SFX_SHOT_REFLECTED);
}

short projectile::get_move_type()
{
    if (_id == -1) {
        return TRAJECTORY_LINEAR;
    }
    return game_data.projectiles[_id].trajectory;
}

void projectile::set_y(int sety)
{
	position.y = sety;
	position0.y = sety;
	position1.y = sety;
}

void projectile::consume_projectile()
{
	// non-consumable types and conditions
	if (get_trajectory() == TRAJECTORY_CHAIN) {
		return;
	} else if (get_trajectory() == TRAJECTORY_BOMB && _effect_n == 0) {
		return;
	}
	_points--;
	//std::cout << "projectile::consume_projectile - _points: " << _points << std::endl;
	if (_points <= 0) {
		is_finished = true;
	}
}

void projectile::set_weapon_id(short wpn_id)
{
	_weapon_id = wpn_id;
}

short projectile::get_weapon_id()
{
	return _weapon_id;
}

short projectile::get_effect_n()
{
	return _effect_n;
}

void projectile::set_owner_position(st_position *owner_position)
{
	_owner_position = owner_position;
}

void projectile::set_owner_direction(int *owner_direction)
{
	_owner_direction = owner_direction;
}
